<?php

// phpcs:disable BestIt.DocTags.RequiredPropertyTags.TagOccurrenceMinVar
// phpcs:disable BestIt.TypeHints.PropertyTypeHint.MissingAnyTypeHint
// phpcs:disable PSR2.ControlStructures.ElseIfDeclaration.NotAllowed

namespace App\Console\Commands\Crawlers;

use App\Console\CrawlerHelper;
use Exception;
use Google\Cloud\Functions\V1\CloudFunctionsServiceClient;
use Illuminate\Console\Command;
use stdClass;

/**
 * Cookie Crawler Crawler
 *
 * Command to run site crawler on a specific link
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Console\Commands\Crawlers
 */
class CookieCrawler extends Command
{
    /**
     * The console command description.
     */
    protected $description = 'Run Site Crawler and check compliances';

    /**
     * The name and signature of the console command.
     */
    protected $signature = 'crawler:cookie
        {link : Site URL to crawl}
        {zone : The website zone}
        {country : The website country}
        {additionalCrawler : Define additional parameters}
        {executed_at : Execution timestamp}
        {official : Define if is an official execution}
        ';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Run Cloud Function to get cookies from website
     *
     * @param string $link
     *
     * @return string
     */
    protected function cloudFunction(string $link): string
    {
        // Clean the url before crawl
        $originalLink = trim($link);

        if (strpos($originalLink, 'http') !== 0) {
            $link = 'http://' . $link;
        }

        // Go 4 leves up from the current directory
        $path = dirname(__DIR__, 4);

        $cloudFunctionsServiceClient = new CloudFunctionsServiceClient([
            'credentials' => json_decode(file_get_contents($path . '/' . env('GCP_COOKIE_CREDENTIALS_FILE')), true),
        ]);

        $gcp = new stdClass();
        $gcp->project = 'abi-martech-global-onetrustval';
        $gcp->region = 'us-central1';
        $gcp->script = 'get-cookies-from-website';

        // Define which cloud function run
        $formattedName = $cloudFunctionsServiceClient->cloudFunctionName($gcp->project, $gcp->region, $gcp->script);

        // Used to encode as json and pass as as parameter in the request
        $data['link'] = $link;

        $this->line('Auditing ' . $originalLink . ' with ' . $gcp->region . '/' . $gcp->script);

        // Run the CF
        $response = $cloudFunctionsServiceClient->callFunction($formattedName, json_encode($data))->getResult();

        return $response;
    }

    /**
     * Execute Crawler to retrieve source code and network requests
     *
     * @param string $link
     * @param string $additionalCrawler
     *
     * @return string
     */
    protected function crawl($link, $additionalCrawler = '{}'): string
    {
        $return = '';

        $this->line('Cookie Crawling: ' . $link);

        $additionalCrawler = trim(str_replace('""', '"', $additionalCrawler), '"');

        try {
            // TODO: Add a try..catch at script to return ERROR when crawl fails
            $output = shell_exec('node app/Console/Js/CookieCrawler.js \'' . $link . '\' \'' . $additionalCrawler . '\'');

            $return = trim($output);
        } catch (Exception $e) {
            $this->error($e->getMessage());
        }

        return $return;
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle(): int
    {
        $retorno = 0;

        $link = $this->argument('link');

        $additionalCrawler = $this->argument('additionalCrawler');

        $executedAt = $this->argument('executed_at');

        $official = ($this->argument('official') ? $this->argument('official') : false);

        $zone = $this->argument('zone');

        $country = ($this->argument('country') ? $this->argument('country') : null);

        try {
            $originalLink = $link;

            if (strpos($link, 'http://') !== 0) {
                $link = 'http://' . $link;
            }

            $data = $this->cloudFunction($link);

            // Removed to run on GCP CF
            // $data = $this->crawl($link, $additionalCrawler);

            $data = json_decode($data, false, 512, JSON_INVALID_UTF8_IGNORE);

            $content = null;

            if (!is_null($data)) {
                $content = array(
                    'cookies' => (isset($data->cookies) ? $data->cookies : []),
                    'siteStatus' => $data->siteStatus,
                    'cookieBannerImplemented' => $data->cookieBannerImplemented,
                    'cookieBannerDomain' => $data->cookieBannerDomain,
                    'cookieBannerSameDomain' => $data->cookieBannerSameDomain,
                );

                if ($data->status === 'failed') {
                    $content['errorMessage'] = $data->error;
                }

                app(CrawlerHelper::class)->storeSourceCode(
                    crawler: 'crawler:cookies',
                    link: $originalLink,
                    content: $content,
                    failed: false,
                    executedAt: $executedAt,
                    official: $official,
                );
            } else {
                $this->error('Failed to get cookies from ' . $link . '. Error message: null $data');

                app(CrawlerHelper::class)->storeSourceCode(
                    crawler: 'crawler:cookies',
                    link: $originalLink,
                    content: '',
                    failed: true,
                    executedAt: $executedAt,
                    official: $official,
                );
            }

            // TODO: we need to rethink this process - and validate only in the end, for all links
            // This current approach will have multiple requests to link, to get DGP links
            $this->call('validate:cookies', [
                'link' => $originalLink,
                'executed_at' => $executedAt,
                'official' => $official,
                'zone' => $zone,
                'country' => $country,
            ]);

            $retorno = 0;
        } catch (Exception $e) {
            $this->error('Cookie Crawler error at: ' . $link . ' - ' . $e->getMessage());

            $retorno = 1;
        } finally {
            return $retorno;
        }
    }
}
