<?php

// phpcs:disable BestIt.DocTags.RequiredPropertyTags.TagOccurrenceMinVar
// phpcs:disable BestIt.TypeHints.PropertyTypeHint.MissingAnyTypeHint
// phpcs:disable PSR2.ControlStructures.ElseIfDeclaration.NotAllowed

namespace App\Console\Commands\Crawlers;

use App\Console\CrawlerHelper;
use Exception;
use Illuminate\Console\Command;

/**
 * Facebook Crawler
 *
 * Command to check the compliance at Facebook Accounts
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Console\Commands\Crawlers
 */
class FacebookCrawler extends Command
{
	/**
	 * Run on debug mode
	 */
	protected bool $debug = false;

	/**
	 * The console command description.
	 */
	protected $description = 'Run Facebook Crawler and check compliance';

	/**
	 * List of iards needed to be tested
	 */
	protected $iards = ['coi', 'fan', 'rdm', 'agegate', 'ugc'];

	/**
	 * The name and signature of the console command.
	 */
	protected $signature = 'crawler:facebook
        {link : Facebook page URL}
        {locale : Define which keywords will be used}
        {--debug : Dump responses instead of store}
        {executed_at : Execution timestamp}
				{official : Define if this is an official execution}';

	/**
	 * Create a new command instance.
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();
	}

	/**
	 * For a given string url the about page
	 *
	 * @param string $link
	 *
	 * @return string
	 */
	protected static function aboutLink(string $link): string
	{
		// Firstly, remove all query string parameters
		$link = explode('?', $link)[0];

		// To ensure we've the profile link, not a specific section
		$link = explode('/about', $link)[0];

		// Ensure we don't have a slash at the end of the link
		$link = rtrim($link, '/');

		// Go to about page, and endure we'll not have redirect based on user location
		$link .= '/about/?brand_redir=DISABLE';

		return $link;
	}

	/**
	 * Execute Puppeteer with a Facebook user to crawler the page's about and retrieve source code
	 *
	 * @param string $link
	 *
	 * @return string
	 */
	protected function crawlLogged($link): string
	{
		$return = '';

		try {
			$fbUser = env('FACEBOOK_USER', '');
			$fbPass = env('FACEBOOK_PASS', '');

			if (empty($fbUser)) {
				throw new Exception('Failed to get Facebook\'s user from environment');
			}

			if (empty($fbPass)) {
				throw new Exception('Failed to get Facebook\'s pass from environment');
			}

			$output = shell_exec('node app/Console/Js/FacebookCrawler.js ' . $link . ' ' . $fbUser . ' ' . $fbPass);

			$return = trim($output);
		} catch (Exception $e) {
			$this->error('Facebook error crawlLogged. Message: ' . $e->getMessage());
		}

		return $return;
	}

	/**
	 * Execute the crawler without loggin into a Facebook user (used to test age gate)
	 *
	 * @param string $link
	 *
	 * @return string
	 */
	protected function crawlUnlogged($link): string
	{
		$return = '';

		try {
			$output = shell_exec('node app/Console/Js/browser.js ' . $link . ' ' . '' . 'false true');

			$return = trim($output);
		} catch (Exception $e) {
			$this->error('Facebook error CrawlUnlogged. Message: ' . $e->getMessage());
		}

		return $return;
	}

	/**
	 * Execute the console command.
	 *
	 * @return int
	 */
	public function handle(): int
	{
		$retorno = 0;

		if ($this->option('debug')) {
			$this->debug = true;
		}

		$link = $this->argument('link');
		$locale = $this->argument('locale');

		$executedAt = $this->argument('executed_at');

		$official = ($this->argument('official') ? $this->argument('official') : false);

		// Keep this to post the address to API later
		$originalLink = $link;

		try {
			if (strpos($originalLink, 'http') !== 0) {
				$link = 'https://' . $link;
			}

			// Get the about url
			$link = self::aboutLink($link);

			if ($this->debug) {
				$this->line('Going to check: ' . $link);
			}

			// TODO: if dont't have age gate, we can also use this as the source code logged
			// Crawl without a logged user, to test age gate
			$sourceCodeUnlogged = $this->crawlUnlogged($link);

			if (strpos($sourceCodeUnlogged, 'DGP ERROR @ Facebook Script') !== false) {
				$this->error('Failed at ' . $originalLink . ' - Facebook Crawler unlogged');

				app(CrawlerHelper::class)->storeSourceCode(
					crawler: 'crawler:facebook',
					link: $originalLink,
					content: $sourceCodeUnlogged,
					failed: true,
					executedAt: $executedAt,
					official: $official,
					age: 'under',
				);
			} else {
				app(CrawlerHelper::class)->storeSourceCode(
					crawler: 'crawler:facebook',
					link: $originalLink,
					content: $sourceCodeUnlogged,
					failed: false,
					executedAt: $executedAt,
					official: $official,
					age: 'under',
				);
			}
			// sleep(10);

			// Retrieve the source code from facebook page
			$sourceCodeLogged = $this->crawlLogged($link);

			if (strpos($sourceCodeLogged, 'DGP ERROR @ Facebook Script') !== false) {
				$this->error('Failed at ' . $originalLink . ' - Facebook Crawler');

				app(CrawlerHelper::class)->storeSourceCode(
					crawler: 'crawler:facebook',
					link: $originalLink,
					content: $sourceCodeLogged,
					failed: true,
					executedAt: $executedAt,
					official: $official,
					age: 'over',
				);

				app(CrawlerHelper::class)->storeResult(
					link: $originalLink,
					criteria: 'http-response',
					response: '0',
					executedAt: $executedAt,
					official: $official,
				);
			} else {
				app(CrawlerHelper::class)->storeSourceCode(
					crawler: 'crawler:facebook',
					link: $originalLink,
					content: $sourceCodeLogged,
					failed: false,
					executedAt: $executedAt,
					official: $official,
					age: 'over',
				);

				app(CrawlerHelper::class)->storeResult(
					link: $originalLink,
					criteria: 'http-response',
					response: '200',
					executedAt: $executedAt,
					official: $official,
				);

				$profileDescription = json_decode($sourceCodeLogged, true, 512, JSON_INVALID_UTF8_IGNORE)['content'];
				$textOnlyDescription = json_decode($sourceCodeLogged, true, 512, JSON_INVALID_UTF8_IGNORE)['additional_data'];

				foreach ($this->iards as $iard) {
					$approved = 'false';

					$kws = CrawlerHelper::keywords($iard, $locale, true);

					if (count($kws) === 0) {
						throw new Exception('There are no keywords for ' . $iard . ' at this link');
					}

					if (is_array($kws)) {
						// If is an associative array, we'll define the elements as the first position
						if (array_key_first($kws) !== 0) {
							$kws = ($kws[array_key_first($kws)]);
						} else if (is_array($kws[0])) {
							$kws = $kws[0];
						}

						foreach ($kws as $index => $kw) {
							if (is_array($kw)) {
								$kws = array_merge($kws, $kws[$index]);
								unset($kws[$index]);
							}
						}
					}

					switch ($iard) {
						case 'agegate':
							if (strpos($sourceCodeUnlogged, '"is_verified"') === false) {
								$approved = 'true';
							}
							break;
						case 'coi':
							if (strpos($profileDescription, '"is_verified":true') !== false) {
								$approved = 'true';
							} else {
								foreach ($kws as $kw) {
									if (strpos(mb_strtolower($textOnlyDescription), mb_strtolower($kw)) !== false) {
										$approved = 'true';
										break;
									}
								}
							}
							break;
						case 'ugc':
							$found = false;

							// Explode new lines (turn into array)
							$text = app(CrawlerHelper::class)->splitNl($textOnlyDescription);

							foreach ($text as $key => $line) {
								$down = mb_strtolower($line);

								if (!$found) {
									foreach ($kws as $kw) {
										$found = str_contains($down, mb_strtolower($kw));

										if ($found) {
											$urls = app(CrawlerHelper::class)->extractUrlsFromText($text[$key]);

											if (is_array($urls) && count($urls) > 0) {
												$status = app(CrawlerHelper::class)->getHttpCode($urls[0]);

												if ($status === 200) {
													$found = true;
													$approved = 'true';
													break;
												} else {
													$found = false;
													$approved = 'false';
													break;
												}
											} else {
												$found = true;
												$approved = 'true';
												break;
											}
										}
									}
								} else {
									break;
								}
							}

							break;
						default:
							foreach ($kws as $kw) {
								if (strpos(mb_strtolower($textOnlyDescription), mb_strtolower($kw)) !== false) {
									$approved = 'true';
									break;
								}
							}
							break;
					}

					app(CrawlerHelper::class)->storeResult(
						link: $originalLink,
						criteria: $iard,
						response: $approved,
						executedAt: $executedAt,
						official: $official,
					);
				}
			}
		} catch (Exception $e) {
			$this->error('Facebook Crawler failed at: ' . $originalLink . ' - ' . $e->getMessage());

			$retorno = 1;
		}

		return $retorno;
	}
}
