<?php

// phpcs:disable BestIt.DocTags.RequiredPropertyTags.TagOccurrenceMinVar
// phpcs:disable BestIt.TypeHints.PropertyTypeHint.MissingAnyTypeHint
// phpcs:disable PSR2.ControlStructures.ElseIfDeclaration.NotAllowed

namespace App\Console\Commands\Crawlers;

use App\Console\CrawlerHelper;
use Exception;
use Illuminate\Console\Command;

/**
 * Instagram Crawler
 *
 * Command to check the compliance at Instagram Accounts
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Console\Commands\Crawlers
 */
class InstagramCrawler extends Command
{
	/**
	 * Run on debug mode
	 */
	protected bool $debug = false;

	/**
	 * The console command description.
	 */
	protected $description = 'Run Instagram Crawler and check compliance';

	/**
	 * List of iards needed to be tested
	 */
	protected $iards = ['coi', 'fan', 'rdm', 'agegate', 'ugc'];

	/**
	 * The name and signature of the console command.
	 */
	protected $signature = 'crawler:instagram
        {link : Instagram profile URL}
        {locale : Define which keywords will be used}
        {--debug : Dumpr response instead of store}
				{official : Define if this is an official execution}
        {executed_at : Execution timestamp}';

	/**
	 * Create a new command instance.
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();
	}

	/**
	 * Execute Puppeteer with a Instagram user to crawler and retrieve source code
	 *
	 * @param string $link
	 * @param bool $overage
	 *
	 * @return string
	 */
	protected function crawl($link, $overage): string
	{
		$return = '';

		try {
			$dataDir = '';
			$user = '';
			$pass = '';

			if ($overage) {
				$dataDir = 'instaOverage';

				$user = env('INSTAGRAM_USER_OVERAGE', '');
				$pass = env('INSTAGRAM_PASS_OVERAGE', '');
			} else {
				$dataDir = 'instaUnderage';

				$user = env('INSTAGRAM_USER_UNDERAGE', '');
				$pass = env('INSTAGRAM_PASS_UNDERAGE', '');
			}

			if (empty($user)) {
				throw new Exception('Failed to get user from environment');
			}

			if (empty($pass)) {
				throw new Exception('Failed to get password from environment');
			}

			// TODO: Add a try..catch at script to return ERROR when crawl fails
			$output = shell_exec('node app/Console/Js/InstagramCrawler.js \'' . $link . '\' \'' . $user . '\' \'' . $pass . '\' \'' . $dataDir . '\'');

			$return = trim($output);
		} catch (Exception $e) {
			$this->error($e->getMessage());
		}

		return $return;
	}

	/**
	 * Execute the console command.
	 *
	 * @return int
	 */
	public function handle(): int
	{
		$retorno = 0;

		if ($this->option('debug')) {
			$this->debug = true;
		}

		if ($this->debug) {
			$this->line('Running Instagram Crawler in debug mode');
		}

		$link = $this->argument('link');
		$locale = $this->argument('locale');

		$executedAt = $this->argument('executed_at');

		$official = ($this->argument('official') ? $this->argument('official') : false);

		// Keep this to post the address to API later
		$originalLink = $link;

		try {
			if (strpos($originalLink, 'http') !== 0) {
				$link = 'https://' . $link;
			}

			// Remove any query string from URL before crawl
			$link = rtrim(explode('?', $link)[0], '/');

			/*
			// Removed. Seems like this were blocked by Instagram
			//Used to access the profile as json
			$link = $link . '/channel/?__a=1';
			*/

			// To keep a pattern, let's ensure the interface is in portuguese
			$link = $link . '?hl=pt-br';

			if ($this->debug) {
				$this->line('Going to check: ' . $link);
			}

			$failedUnder = false;
			$sourceCodeUnderAge = $this->crawl($link, false);

			$ageGated = false;

			if (strpos($sourceCodeUnderAge, 'DGP ERROR @ Instagram Script') !== false) {
				$failedUnder = true;
				$this->error('Failed at ' . $originalLink . ' - underage');
			} else {
				if (strpos($sourceCodeUnderAge, 'Perfil restrito') !== false || strpos($sourceCodeUnderAge, 'Restricted profile') !== false) {
					$ageGated = true;
				}
			}

			$failedOver = false;

			// If Profile doesnt have age gate, we can use the underage data
			if ($ageGated) {
				$sourceCodeOverAge = $this->crawl($link, true);
			} else {
				$sourceCodeOverAge = $sourceCodeUnderAge;
			}

			$profileData = $sourceCodeOverAge;

			$verified = false;
			if (strpos($sourceCodeOverAge, 'title="Verified"') !== false || (strpos($sourceCodeOverAge, 'title="Verificado"') !== false)) {
				$verified = true;
			}

			if (strpos($sourceCodeOverAge, 'DGP ERROR @ Instagram Script') !== false) {
				$failedOver = true;

				$this->error('Failed at ' . $originalLink . ' - overage');
			}

			app(CrawlerHelper::class)->storeSourceCode(
				crawler: 'crawler:instagram',
				link: $originalLink,
				content: $sourceCodeOverAge,
				failed: $failedOver,
				executedAt: $executedAt,
				official: $official,
				age: 'overage',
			);

			app(CrawlerHelper::class)->storeSourceCode(
				crawler: 'crawler:instagram',
				link: $originalLink,
				content: $sourceCodeUnderAge,
				failed: $failedOver,
				executedAt: $executedAt,
				official: $official,
				age: 'underage',
			);

			if ((!($failedUnder)) && (!($failedOver)) && (!(empty($profileData)))) {
				foreach ($this->iards as $iard) {
					$approved = 'false';

					$kws = CrawlerHelper::keywords($iard, $locale, true);

					if (count($kws) === 0) {
						throw new Exception('There are no locale keywords for this link');
					}

					if (is_array($kws)) {
						// If is an associative array, we'll define the elements as the first position
						if (array_key_first($kws) !== 0) {
							$kws = ($kws[array_key_first($kws)]);
						} else if (is_array($kws[0])) {
							$kws = $kws[0];
						}

						foreach ($kws as $index => $kw) {
							if (is_array($kw)) {
								$kws = array_merge($kws, $kws[$index]);
								unset($kws[$index]);
							}
						}
					}

					switch ($iard) {
						case 'agegate':
							$approved = ($ageGated) ? 'true' : 'false';
							break;
						case 'coi':
							if ($verified) {
								$approved = 'true';
							} else {
								if (!($failedOver)) { // @phpstan-ignore-line
									foreach ($kws as $kw) {
										if (strpos(mb_strtolower($profileData), mb_strtolower($kw)) !== false) {
											$approved = 'true';
											break;
										}
									}
								}
							}
							break;
						case 'ugc':
							$found = false;

							// Explode new lines (turn into array)
							$text = app(CrawlerHelper::class)->splitNl($profileData);

							foreach ($text as $key => $line) {
								$down = mb_strtolower($line);

								if (!$found) {
									foreach ($kws as $kw) {
										$found = str_contains($down, mb_strtolower($kw));

										if ($found) {
											$urls = app(CrawlerHelper::class)->extractUrlsFromText($text[$key]);

											if (is_array($urls) && count($urls) > 0) {
												$status = app(CrawlerHelper::class)->getHttpCode($urls[0]);

												if ($status === 200) {
													$found = true;
													$approved = 'true';
													break;
												} else {
													$found = false;
													$approved = 'false';
													break;
												}
											} else {
												$found = true;
												$approved = 'true';
												break;
											}
										}
									}
								} else {
									break;
								}
							}

							break;
						default:
							if (!($failedOver)) { // @phpstan-ignore-line
								foreach ($kws as $kw) {
									if (strpos(mb_strtolower($profileData), mb_strtolower($kw)) !== false) {
										$approved = 'true';
										break;
									}
								}
							}
							break;
					}

					if ($this->debug) {
						$this->line('Storing: ' . $approved . ' for ' . $iard . ' at: ' . $originalLink);
					} else {
						app(CrawlerHelper::class)->storeResult(
							link: $originalLink,
							criteria: $iard,
							response: $approved,
							executedAt: $executedAt,
							official: $official,
						);
					}
				}

				if ($this->debug) {
					$this->line('Storing: 200 for http-response at: ' . $originalLink);
				} else {
					app(CrawlerHelper::class)->storeResult(
						link: $originalLink,
						criteria: 'http-response',
						response: '200',
						executedAt: $executedAt,
						official: $official,
					);
				}
			} else {
				if ($this->debug) {
					$this->line('Storing: 0 for http-response at: ' . $originalLink);
				} else {
					app(CrawlerHelper::class)->storeResult(
						link: $originalLink,
						criteria: 'http-response',
						response: '0',
						executedAt: $executedAt,
						official: $official,
					);
				}
			}
		} catch (Exception $e) {
			$this->error('Instagram Crawler error at: ' . $originalLink . ' - ' . $e->getMessage());

			$retorno = 1;
		}

		return $retorno;
	}
}
