<?php

// phpcs:disable BestIt.DocTags.RequiredPropertyTags.TagOccurrenceMinVar
// phpcs:disable BestIt.TypeHints.PropertyTypeHint.MissingAnyTypeHint
// phpcs:disable PSR2.ControlStructures.ElseIfDeclaration.NotAllowed

namespace App\Console\Commands\Crawlers;

use App\Console\CrawlerHelper;
use App\Models\Crawls;
use Exception;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\Log;

/**
 * Site Crawler
 *
 * Command to run site crawler on a specific link
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Console\Commands\Crawlers
 */
class SiteCrawler extends Command
{
	/**
	 * The console command description.
	 */
	protected $description = 'Run Site Crawler and check compliances';

	/**
	 * List of iards needed to be tested
	 */
	protected $iards = [
		'agegate',
		'coi',
		'rdm',
	];

	/**
	 * The name and signature of the console command.
	 */
	protected $signature = 'crawler:site
        {link : Site URL to crawl}
        {locale : Define which keywords will be used}
        {alcoholic : Define if site is alcoholic}
        {executed_at : Execution timestamp}
        {additionalCrawler? : Define additional parameters}
				{official? : Define if is an official execution}
				{ignoreInCriteria? : Ignore criteria for specific asset}';

	/**
	 * Create a new command instance.
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();
	}

	/**
	 * Check partial array data
	 *
	 * @return array
	 */
	protected function array_search_partial($arr, $keyword): array
	{
		$arrFound = [];
		foreach ($arr as $index => $string) {
			if (strpos($string, $keyword) !== FALSE)
				$arrFound[] = $string;
		}
		return $arrFound;
	}

	/**
	 * Clean source code
	 *
	 * @return string
	 */
	protected function cleanSourceCode($sourceCode): string
	{
		$arrReplace = array('<BR>', '<br>', '<BR/>', '<br/>', '<BR />', '<br />');
		// Removing unnecessary blank spaces, extra blank spaces, br tags, and turning all characters to lower case.
		return trim(preg_replace('/\s\s+/', ' ', str_replace($arrReplace, ' ', mb_strtolower($sourceCode))));
	}

	/**
	 * Execute Crawler to retrieve source code and network requests
	 *
	 * @param string $link
	 * @param string $originalLink
	 * @param bool $official
	 * @param string $executedAt
	 * @param string $additionalCrawler
	 *
	 * @return string
	 */
	protected function crawl($link, $originalLink, $official, $executedAt, $additionalCrawler = '{}'): string
	{
		$official = ($official ? 'true' : 'false');

		$return = '';

		$this->line('Crawling: ' . $link);

		try {
			$command = 'node app/Console/Js/site.js ';

			$command .= '\'' . $link . '\' ';
			$command .= '\'' . $originalLink . '\' ';
			$command .= '\'' . $official . '\' ';
			$command .= '\'' . $executedAt . '\' ';
			$command .= '\'' . $additionalCrawler . '\'';

			$output = shell_exec($command);

			$return = $output;
		} catch (Exception $e) {
			$this->error($e->getMessage());
		} finally {
			return $return;
		}
	}

	/**
	 * Execute the console command.
	 *
	 * @return int
	 */
	public function handle(): int
	{
		$retorno = 0;

		$link = $this->argument('link');
		$locale = $this->argument('locale');
		$alcoholic = ($this->argument('alcoholic') ? $this->argument('alcoholic') : true);
		$additionalCrawler = ($this->argument('additionalCrawler') ? $this->argument('additionalCrawler') : '');
		$executedAt = $this->argument('executed_at');
		$official = ($this->argument('official') ? $this->argument('official') : false);
		$ignoreInCriteria = ($this->argument('ignoreInCriteria') ? $this->argument('ignoreInCriteria') : false);

		// Keep this to post the address to API later
		$originalLink = $link;

		$sourceCode = '';
		$requests = null;

		try {
			// Clean the url before crawl
			$link = trim($link);

			if (strpos($originalLink, 'http') !== 0) {
				$link = 'http://' . $link;
			}

			$crawled = $this->crawl($link, $originalLink, $official, $executedAt, $additionalCrawler);

			$data = json_decode($crawled);

			if (json_last_error() === 0 && $data->status === 'success') {
				$sourceCode = Crawls::where([
					['link', $originalLink],
					['data_type', 'crawler:htmlwithjs'],
				])->first()->content;

				$requests = json_decode(Crawls::where([
					['link', $originalLink],
					['data_type', 'crawler:browsersrequest'],
				])->first()->content);
			} else {
				app(CrawlerHelper::class)->storeResult(
					link: $originalLink,
					criteria: 'http-response',
					response: '0',
					executedAt: $executedAt,
					official: $official,
				);

				throw new Exception('SiteCrawler - failed at ' . $originalLink);
			}

			app(CrawlerHelper::class)->storeResult(
				link: $originalLink,
				criteria: 'http-response',
				response: '200',
				executedAt: $executedAt,
				official: $official,
			);

			$this->line('Running IARDS');

			$sourceCode = $this->cleanSourceCode($sourceCode);

			/////// NEW
			foreach ($this->iards as $iard) {
				if ((!$alcoholic) && ($iard === 'rdm' || $iard === 'agegate')) { // @phpstan-ignore-line
					try {
						app(CrawlerHelper::class)->storeResult(
							link: $originalLink,
							criteria: $iard,
							response: 'NA',
							executedAt: $executedAt,
							official: $official,
						);
					} catch (Exception $e) {
						$this->error('SiteCrawler error (222): ' . $e->getMessage());
					}
					continue;
				}

				try {
					$arrIard = CrawlerHelper::keywords($iard, $locale, false);
					if (count($arrIard) === 0) {
						continue (1);
					}
				} catch (Exception $e) {
					Log::notice('There are no locale keywords for this website');
					continue (1);
				}

				$found = 0;

				// Without any keywords group - Normally age gate and RDM criterias
				if (count($arrIard) === count($arrIard, COUNT_RECURSIVE)) {
					foreach ($arrIard as $iardKwItem) {
						if ($found === 1) break;
						$found = (str_contains($sourceCode, trim(mb_strtolower($iardKwItem)))) ? 1 : 0;
					}
				} else {
					$iardFoundeds = 0;
					$groupsCount = 0;
					$groupFounds = 0;
					$groups = array();
					foreach ($arrIard as $type => $iardGroup) {
						if (!is_array($iardGroup)) {
							if (str_contains($sourceCode, trim(mb_strtolower($iardGroup)))) {
								$iardFoundeds++;
							}
						} else {
							$groupsCount++;
							foreach ($iardGroup as $key => $iardTypeItem) {
								if (empty($groups[$type])) $groups[$type] = 0;
								if ($groups[$type] > 0) break;

								if ($iard === 'coi' and !is_integer($type)) {
									$regexp = "<a(.*)\\1[^>]*>(.*)<\/a>";
									preg_match_all("/$regexp/siU", $sourceCode, $result);
									foreach ($result[2] as $text) {
										if ($type === 'info') {
											if ((str_replace(' ', '', htmlspecialchars_decode(trim(strip_tags(mb_strtolower(trim($text)))))) === mb_strtolower(trim($iardTypeItem)))) {
												$groupFounds++;
												$groups[$type]++;
												break;
											}
										} else {
											// Policy group
											if (strpos(htmlspecialchars_decode(trim(strip_tags(mb_strtolower($text)))), mb_strtolower(trim($iardTypeItem))) !== false) {
												$groupFounds++;
												$groups[$type]++;
												break;
											}
										}
									}
								} else {
									if (str_contains($sourceCode, trim(mb_strtolower($iardTypeItem)))) {
										$groupFounds++;
										$groups[$type]++;
									}
								}
							}
						}
					}

					if ($iardFoundeds > 0 && $groupFounds >= $groupsCount) {
						$found = 1;
					}
				}

				if ($found > 0) {
					$response = 'true';
				} else {
					$response = 'false';
				}

				try {
					app(CrawlerHelper::class)->storeResult(
						link: $originalLink,
						criteria: $iard,
						response: $response,
						executedAt: $executedAt,
						official: $official,
					);
				} catch (Exception $e) {
					$this->error('SiteCrawler error (222): ' . $e->getMessage());
				}
			}

			$this->line('Validating network requests');

			$siteRequests = array_merge($requests->allowAll, $requests->initial);

			// CDP
			$search = $this->array_search_partial($siteRequests, 'treasuredata');
			if (count($search) >= 1) {
				$searchInternal = $this->array_search_partial($search, 'page_views');
				if (count($searchInternal) >= 1) {
					$response = 'true';
				} else {
					$response = 'false';
				}
			} else {
				$response = 'false';
			}

			if (!$ignoreInCriteria) {
				app(CrawlerHelper::class)->storeResult(
					link: $originalLink,
					criteria: 'cdp-implementation',
					response: $response,
					executedAt: $executedAt,
					official: $official,
				);
			}
		} catch (Exception $e) {
			$this->error('Site Crawler error at: ' . $originalLink . ' - ' . $e->getMessage());
			$retorno = 1;
		}

		return $retorno;
	}
}
