<?php

// phpcs:disable PSR2.ControlStructures.ElseIfDeclaration.NotAllowed

namespace App\Console\Commands\Crawlers;

use App\Console\CrawlerHelper;
use Atymic\Twitter\Facade\Twitter;
use Exception;
use GuzzleHttp\Client;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\Log;

/**
 * TwitterCrawler Class
 *
 * Command to run Twitter on a particular link
 *
 * @author João Batista Fulgêncio <joao.fulgencio-ext@ab-inbev.com>
 * @package App\Console\Commands\Crawlers
 */
class TwitterCrawler extends Command
{
	/**
	 * List of iards needed to be tested
	 */
	protected $iards = ['coi', 'fan', 'rdm', 'agegate', 'ugc'];

	/**
	 * The signature to call the command
	 */
	protected $signature = 'crawler:twitter {link} {locale} {executed_at} {official?}';

	/**
	 * Handle the command
	 *
	 * @return int
	 */
	public function handle(): int
	{
		$originalLink = $this->argument('link');
		$locale = $this->argument('locale');
		$executedAt = $this->argument('executed_at');

		$official = ($this->argument('official') ? $this->argument('official') : false);

		$client = new Client();

		$link = explode('?', $originalLink)[0];
		preg_match('/[^\/]+$/', trim($link, '/'), $matches);

		try {
			$twitter = Twitter::getUsers(['screen_name' => $matches[0]]);

			$completeDescription = $twitter->description;

			// We concat the url at description to check if UGC is at profile's links
			if (isset($twitter->entities->url)) {
				foreach ($twitter->entities->url->urls as $url) {
					$completeDescription .= "\n" . $url->expanded_url;
				}
			}

			// We also need get the complete (non-shortened) links - Twitter use t.co as default link shortener
			if (isset($twitter->entities->description->urls)) {
				foreach ($twitter->entities->description->urls as $url) {
					$completeDescription .= "\n" . $url->expanded_url;
				}
			}

			$completeDescription .= "\n" . $twitter->name;

			foreach ($this->iards as $iard) {
				$result = 'reproved';
				switch ($iard) {
					case 'coi':
						if ($twitter->verified === true) {
							$result = 'approved';

							break;
						} else {
							// There are cases where the COI is at account name
							try {
								$kws = CrawlerHelper::keywords($iard, $locale, true);

								if (count($kws) === 0) {
									throw new Exception('There are no locale keywords for this link');
								}

								if (is_array($kws)) {
									// If is an associative array, we'll define the elements as the first position
									if (array_key_first($kws) !== 0) {
										$kws = ($kws[array_key_first($kws)]);
									} else if (is_array($kws[0])) {
										$kws = $kws[0];
									}

									foreach ($kws as $index => $kw) {
										if (is_array($kw)) {
											$kws = array_merge($kws, $kws[$index]);
											unset($kws[$index]);
										}
									}
								}

								foreach ($kws as $kw) {
									if (strpos(mb_strtolower($completeDescription), mb_strtolower($kw)) !== false) {
										$result = 'approved';

										break;
									}
								}
								break;
							} catch (Exception $e) {
								Log::notice($e->getMessage());
								break;
							}
						}
						// If user don't have verified badge, fallback is check the about text confirming as oficial account
					case 'rdm':
					case 'fan':
						try {
							$kws = CrawlerHelper::keywords($iard, $locale, true);

							if (count($kws) === 0) {
								throw new Exception('There are no locale keywords for this link');
							}

							if (is_array($kws)) {
								// If is an associative array, we'll define the elements as the first position
								if (array_key_first($kws) !== 0) {
									$kws = ($kws[array_key_first($kws)]);
								} else if (is_array($kws[0])) {
									$kws = $kws[0];
								}

								foreach ($kws as $index => $kw) {
									if (is_array($kw)) {
										$kws = array_merge($kws, $kws[$index]);
										unset($kws[$index]);
									}
								}
							}
						} catch (Exception $e) {
							Log::notice($e->getMessage());
							break;
						}
						$found = 0;
						foreach ($kws as $keyword) {
							if ($found === 0) {
								$found = (strpos(mb_strtolower(str_replace('<BR>', ' ', str_replace('<br>', ' ', $completeDescription))), mb_strtolower($keyword)) !== false) ? 1 : 0;
							} else {
								break;
							}
						}
						if ($found === 1) {
							$result = 'approved';
							break;
						}
						break;
					case 'agegate':
						// TODO: check the cases this isn't working as expected
						try {
							Twitter::postFollow(['screen_name' => $matches[0]]);
						} catch (Exception $e) {
							if ((strpos($e->getMessage(), 'age screened') !== false) || $e->getCode() === 250) {
								$result = 'approved';
							}
						}

						break;
					case 'ugc':
						$found = false;

						// Explode new lines (turn into array)
						$text = app(CrawlerHelper::class)->splitNl($completeDescription);

						$kws = CrawlerHelper::keywords($iard, $locale, true);

						foreach ($text as $key => $line) {
							$down = mb_strtolower($line);

							if (!$found) {
								foreach ($kws as $kw) {
									$found = str_contains($down, mb_strtolower($kw));

									if ($found) {
										$urls = app(CrawlerHelper::class)->extractUrlsFromText($text[$key]);

										if (is_array($urls) && count($urls) > 0) {
											$status = app(CrawlerHelper::class)->getHttpCode($urls[0]);

											if ($status === 200) {
												$found = true;
												$result = 'approved';
												break;
											} else {
												$found = false;

												$result = 'reproved';

												break;
											}
										} else {
											$found = true;

											$result = 'approved';

											break;
										}
									}
								}
							} else {
								break;
							}
						}

						break;
				}

				app(CrawlerHelper::class)->storeResult(
					link: $originalLink,
					criteria: $iard,
					response: ($result === 'approved') ? 'true' : 'false',
					executedAt: $executedAt,
					official: $official,
				);
			}

			app(CrawlerHelper::class)->storeSourceCode(
				crawler: 'crawler:twitter',
				link: $originalLink,
				content: $completeDescription,
				failed: false,
				executedAt: $executedAt,
				official: $official,
			);

			app(CrawlerHelper::class)->storeResult(
				link: $originalLink,
				criteria: 'http-response',
				response: '200',
				executedAt: $executedAt,
				official: $official,
			);
		} catch (Exception $e) {
			$this->error('TwitterCrawler handle error. Message: ' . $e->getMessage());

			app(CrawlerHelper::class)->storeSourceCode(
				crawler: 'crawler:twitter',
				link: $originalLink,
				content: $e->getMessage(),
				failed: true,
				executedAt: $executedAt,
				official: $official,
			);

			app(CrawlerHelper::class)->storeResult(
				link: $originalLink,
				criteria: 'http-response',
				response: '0',
				executedAt: $executedAt,
				official: $official,
			);
		}

		return Command::SUCCESS;
	}
}
