<?php

// phpcs:disable BestIt.DocTags.RequiredPropertyTags.TagOccurrenceMinVar
// phpcs:disable BestIt.TypeHints.PropertyTypeHint.MissingAnyTypeHint
// phpcs:disable PSR2.ControlStructures.ElseIfDeclaration.NotAllowed

namespace App\Console\Commands;

use App\Console\CrawlerHelper;
use App\Models\Crawls;
use App\Models\Links;
use Exception;
use Google\Cloud\BigQuery\BigQueryClient;
use Google_Client;
use Google_Service_Analytics;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\Cache;
use stdClass;

/**
 * Google Analytics (GA) Checker
 *
 * Command to check the compliance with implemented GAs on sites
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Console\Commands
 */
class GAChecker extends Command
{
    /**
     * The console command description.
     */
    protected $description = 'Using the latest (current) requests crawler result, check the compliance (implementation, access and taxonomy) for Google Analytics on site(s)';

    // TODO: Make executed_at required
    /**
     * The signature to call the command
     */
    protected $signature = 'ga:criteria
        {url? : Check a specific link}
        {--complete-dump : Complete dump of all GAs Accounts informations that we\'ve access}
        {--id=? : Check a specific Analytics, ex: UA-XXX-X}
        {--dump-accounts : Show distinct GA accounts}
        {--dump-identifiers : Show distinct GA IDs (UAs)}
        {--until=? : Used to retrieve until a specified GA version (3, 4 -- default 4)}
        {executed_at? : Execution timestamp}
        {official? : Define if this is an official execution}';

    /**
     * Store all the analytics into memory, allow only one request to GCP
     */
    protected array $analytics = [];

    // TODO: get this from keywords
    /**
     * Define the pattern for GA's account name
     */
    protected array $allowedTaxonomy = [
        'AFRICA_MOZ',
        'AFRICA_NGA',
        'AFRICA_TZA',
        'AFRICA_UGA',
        'AFRICA_ZAF',
        'AFRICA_ZMB',
        'AFRICA_BWA',
        'AFRICA_ZAF_LEGACY',
        'APAC_KOR',
        'APAC_JPN',
        'EUR_DNK',
        'GHQ_Bees',
        'EUR_BEL',
        'EUR_CHE',
        'EUR_DEU',
        'EUR_ESP',
        'EUR_FRA',
        'EUR_GBR',
        'EUR_ITA',
        'EUR_LUX',
        'EUR_NLD',
        'EUR_SWE',
        'EUR_UKR',
        'EUR_POL',
        'EUR_CZE',
        'EUR_NOR',
        'EUR_IRL',
        'GHQ_GLOBAL',
        'MIDAM_COL',
        'MIDAM_DOM',
        'MIDAM_ECU',
        'MIDAM_HND',
        'MIDAM_MEX',
        'MIDAM_MEX2',
        'MIDAM_PAN',
        'MIDAM_PER',
        'MIDAM_SLV',
        'NAZ_CAN',
        'NAZ_USA',
        'SAZ_ARG',
        'SAZ_BOL',
        'SAZ_BRA',
        'SAZ_CHL',
        'SAZ_PRY',
        'MAZ_GTM',
    ];

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Dump distinct accounts names
     *
     * @return array
     */
    protected function distinctAccounts(): array
    {
        $accounts = [];

        foreach ($this->analytics as $value) {
            if (!(in_array($value->account, $accounts))) {
                array_push($accounts, $value->account);
            }
        }

        return $accounts;
    }

    /**
     * Dump distinct accounts identifiers
     *
     * @return array
     */
    protected function distinctIdentifiers(): array
    {
        $ids = [];

        foreach ($this->analytics as $value) {
            if (!(in_array($value->measurement_id, $ids))) {
                array_push($ids, $value->measurement_id);
            }
        }

        return $ids;
    }

    /**
     * Extract Analytics Property Identifier (UA-XXX-X) from url
     *
     * @param string $url
     * @param int $version
     *
     * @return string
     */
    protected static function extractIdentifier(string $url, int $version = 3): string
    {
        $retorno = '';

        try {
            $urlComponents = parse_url($url);

            parse_str($urlComponents['query'], $params);

            if ($version === 4) {
                $retorno = $params['id'];
            } else {
                $retorno = $params['tid'];
            }
        } catch (Exception $e) {
            $retorno = '';
        } finally {
            return $retorno;
        }
    }

    /**
     * Return GA4 Account Summarires
     *
     * @param string $token
     * @param int $limit
     *
     * @return array
     */
    protected function ga4AccountSummaries(string $token, int $limit = 999): array
    {
        $accounts = [];

        try {
            $curl = curl_init();

            curl_setopt_array($curl, array(
                CURLOPT_URL => 'https://analyticsadmin.googleapis.com/v1alpha/accountSummaries?pageSize=' . $limit,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => 'GET',
                CURLOPT_HTTPHEADER => array(
                    'Authorization: Bearer ' . $token,
                ),
            ));

            $response = json_decode(curl_exec($curl), true);

            curl_close($curl);

            if (isset($response['accountSummaries'])) {
                $accounts = $response['accountSummaries'];
            } else {
                throw new Exception($response['error']['message'], $response['error']['code']);
            }
        } catch (Exception $e) {
            $this->error('ga4AccountSummaries', $e->getMessage());
        } finally {
            return $accounts;
        }
    }

    /**
     * For a given GA4 Property, get all Web Data Streams
     *
     * @param string $property
     * @param string $token
     * @param int $limit
     *
     * @return array
     */
    protected function ga4WebDataStreams(string $property, string $token, int $limit = 999): array
    {
        $stream = [];
        try {
            $curl = curl_init();

            curl_setopt_array($curl, array(
                CURLOPT_URL => 'https://analyticsadmin.googleapis.com/v1alpha/' . $property . '/webDataStreams?pageSize=' . $limit,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => 'GET',
                CURLOPT_HTTPHEADER => array(
                    'Authorization: Bearer ' . $token
                ),
            ));

            $response = json_decode(curl_exec($curl), true);
            curl_close($curl);

            if (isset($response['webDataStreams'])) {
                $stream = $response['webDataStreams'];
            }
        } catch (Exception $e) {
            $this->error('ga4WebDataStreams', $e->getMessage());
        } finally {
            return $stream;
        }
    }

    /**
     * Retrieve and normalize all Google Analytics Accounts (GA3, GA4)
     *
     * @param int $untilVersion
     *
     * @return void
     */
    protected function getAnalyticsFromApi($untilVersion = 4): void
    {
        $retorno = [];

        $this->line('Getting Analytics');

        try {
            if (Cache::has('google-analytics-accounts-' . $untilVersion)) {
                $retorno = Cache::get('google-analytics-accounts-' . $untilVersion);
            } else {
                $client = $this->initializeGA();

                $accessToken = $client->getAccessToken();

                if (is_null($client->getAccessToken())) {
                    throw new Exception('Failed to authenticate with GCP. Check environment variables');
                }

                $token = $accessToken['access_token'];

                // GA <= 3 Flow
                $service = $this->initializeGAService($client);

                $accounts = $service->management_accountSummaries->listManagementAccountSummaries();

                $accountsGa = [];

                $count = 0;

                foreach ($accounts->getItems() as $item) {
                    foreach ($item->getWebProperties() as $wp) {
                        $views = $wp->getProfiles();
                        if (!is_null($views)) {
                            foreach ($wp->getProfiles() as $view) {
                                $accountsGa[$count]['view'] = $view['name'];
                                $accountsGa[$count]['view_id'] = $view['id'];

                                $accountsGa[$count]['property'] = $wp['name'];
                                $accountsGa[$count]['property_id'] = $wp['id'];

                                // Hold the account identifier
                                $accountsGa[$count]['id'] = $wp['id'];

                                $accountsGa[$count]['account'] = $item['name'];
                                $accountsGa[$count]['account_id'] = $item['id'];

                                $count++;
                            }
                        }
                    }
                }

                // Optimizing to cases to check a specific GA <= 3
                if ($untilVersion === 4) {
                    //GA4 Flow
                    $ga4Accounts = $this->ga4AccountSummaries($token);

                    foreach ($ga4Accounts as $account) {
                        if (isset($account['propertySummaries'])) {
                            foreach ($account['propertySummaries'] as $property) {
                                $propertyDetails = $this->ga4WebDataStreams($property['property'], $token);

                                foreach ($propertyDetails as $stream) {
                                    $accountsGa[$count]['view'] = $stream['displayName'];
                                    $accountsGa[$count]['view_id'] = $stream['name'];

                                    // Hold the account identifier
                                    $accountsGa[$count]['id'] = $stream['measurementId'];

                                    $accountsGa[$count]['property'] = $property['displayName'];
                                    $accountsGa[$count]['property_id'] = $property['property'];

                                    $accountsGa[$count]['account'] = $stream['displayName'];
                                    $accountsGa[$count]['account_id'] = $account['account'];
                                    $count++;
                                }
                            }
                        }
                    }
                }

                $retorno = $accountsGa;

                Cache::put('google-analytics-accounts-' . $untilVersion, $retorno, now()->addMinutes(120));
            }
        } catch (Exception $e) {
            $this->error($e->getMessage());
        } finally {
            $this->line('Finished Getting Analytics');

            $this->analytics = $retorno;
        }
    }

    /**
     * Retrieve Google Analytics account details from BQ
     *
     * @return void
     */
    protected function getAnalyticsFromBq(): void
    {
        $data = [];

        try {
            $big = $this->initializeBqClient();

            $query = $big->query(
                'SELECT * FROM `abi-martech-global-wa.ga_std.vw_view_classification_info` WHERE \'Tracking_ID\' <> \'NA\'',
            );

            $queryResults = $big->runQuery($query);

            foreach ($queryResults as $value) {
                $entry = new stdClass();

                $entry->account = $value['Account_name'];
                $entry->account_id = $value['Account_ID'];
                $entry->property = $value['Property_name'];
                $entry->property_id = $value['Property_ID'];
                $entry->view = $value['view_name'];
                $entry->view_id = $value['View_ID'];
                $entry->measurement_id = $value['Tracking_ID'];

                array_push($data, $entry);
            }
        } catch (Exception $e) {
            $data = [];
            $this->error('Failed at Google Analytics crawler\' getAnalyticsFromBq function. Error message: ' . $e->getMessage());
        } finally {
            $this->analytics = $data;
        }
    }

    // TODO: Check why links aren't being found (ex: https://tapit.com.co/)
    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle(): int
    {
        $retorno = 2;

        $executedAt = $this->argument('executed_at');

        $official = ($this->argument('official') ? $this->argument('official') : false);

        try {
            if ($this->option('dump-accounts')) {
                $this->getAnalyticsFromBq();

                if (count($this->analytics) === 0) {
                    throw new Exception('Error getting Analytics Accounts');
                }

                $accounts = $this->distinctAccounts();

                foreach ($accounts as $account) {
                    $this->info($account);
                }
            } else if ($this->option('complete-dump')) {
                $this->getAnalyticsFromBq();

                if (count($this->analytics) === 0) {
                    throw new Exception('Error getting Analytics Accounts');
                }

                // Output table header
                $header = [
                    'account',
                    'account_id',
                    'property',
                    'property_id',
                    'view',
                    'view_id',
                    'measurement_id',
                ];

                // Output data
                $table = [];

                array_push($table, $header);

                foreach ($this->analytics as $value) {
                    array_push($table, [
                        $value->account,
                        $value->account_id,
                        $value->property,
                        $value->property_id,
                        $value->view,
                        $value->view_id,
                        $value->measurement_id,
                    ]);
                }

                echo ($this->str_putcsv($table));
            } else if ($this->option('dump-identifiers')) {
                $this->getAnalyticsFromBq();

                if (count($this->analytics) === 0) {
                    throw new Exception('Error getting Analytics Accounts');
                }

                $ids = $this->distinctIdentifiers();

                foreach ($ids as $id) {
                    $this->info($id);
                }
            } else if ($this->option('id') !== '?') {
                $this->getAnalyticsFromBq();

                $account = $this->identifyAccount($this->option('id'));

                if (!is_null($account)) {
                    $this->info('Has access');

                    if (strpos($this->option('id'), 'G-') === 0) {
                        $this->info('There aren\'t a pattern of taxonomy to GA4');
                    } else {
                        $taxonomy = $this->validateTaxonomy($account);

                        if ($taxonomy) {
                            $this->info('Correct taxonomy');
                        } else {
                            $this->error('Wrong taxonomy. Account name: ' . $account->account);
                        }
                    }
                } else {
                    $this->error('No access');
                }
            } else {
                $this->line('Running Google Analytics Checker');

                $sites = [];
                $link = '';
                $cleanLink = '';

                if ($this->argument('url') && $this->argument('url') !== '?') {
                    $link = $this->argument('url');

                    $cleanLink = strtolower(rtrim(preg_replace('(^https?://)', '', $link), '/'));

                    $sites = Crawls::where([
                        ['data_type', 'crawler:browsersrequest'],
                        ['link', $cleanLink],
                    ])->get()->toArray();
                } else {
                    $domainList = Links::with(['channel'])->where([
                        ['consider', true],
                    ])->whereRelation('channel', 'slug', 'site')->pluck('link');

                    foreach ($domainList as $domain) {
                        $data = Crawls::where([
                            ['data_type', 'crawler:browsersrequest'],
                            ['link', $domain],
                        ])->first();

                        if (!(is_null($data))) {
                            array_push($sites, $data);
                        }
                    }
                }

                if (count($sites) > 0) {
                    $this->getAnalyticsFromBq();

                    if (count($this->analytics) === 0) {
                        throw new Exception('Error getting Analytics Accounts');
                    }

                    foreach ($sites as $site) {
                        if (is_null($site)) {
                            throw new Exception('Null. ' . $site);
                        }

                        if (is_array($site)) {
                            $link = $site['link'];
                        } else {
                            $link = $site->link;
                        }

                        $assetCountry = Links::join('countries', 'links.country_id', '=', 'countries.id')->where([['links.link', $link]])->value('countries.name');
                        if ($assetCountry === 'Ukraine' || $assetCountry === 'Russia') continue;

                        // For each site, Initializing the results with an auto-fail
                        $implemented = false;
                        $access = false;
                        $taxonomy = false;

                        $gaRequests = [];
                        $ga4access = false;

                        $content = json_decode($site['content']);

                        $siteRequests = array_merge($content->allowAll, $content->initial);

                        foreach ($siteRequests as $request) {
                            // Only check GA <= 3 Pageview's event
                            if (preg_match('/(.)*google-analytics.com(.*)pageview(.*)/', $request)) {
                                $entry = new stdClass();
                                $entry->request = $request;
                                $entry->version = 3;

                                array_push($gaRequests, $entry);
                            } else if (preg_match('/(.*)googletagmanager.com\/gtag\/(.*)id=G-(.*)/', $request)) {
                                $entry = new stdClass();
                                $entry->request = $request;
                                $entry->version = 4;

                                array_push($gaRequests, $entry);
                            }
                        }

                        if (count($gaRequests) > 0) {
                            $implemented = true;

                            $this->storeResult('ga-implementation', $link, true, $executedAt, $official);
                        } else {
                            $implemented = false;

                            $this->storeResult('ga-implementation', $link, false, $executedAt, $official);
                            $this->storeResult('ga-access', $link, false, $executedAt, $official);
                            $this->storeResult('ga-taxonomy', $link, false, $executedAt, $official);
                        }

                        if ($implemented) {
                            foreach ($gaRequests as $request) {
                                $ua = self::extractIdentifier($request->request, $request->version);

                                $account = $this->identifyAccount($ua);

                                if (!is_null($account)) {
                                    $access = true;

                                    if ($request->version < 4) {
                                        $taxonomy = $this->validateTaxonomy($account);
                                    } else {
                                        $ga4access = true;
                                    }
                                }
                            }
                        }

                        if ($access) {
                            $this->storeResult('ga-access', $link, true, $executedAt, $official);
                        } else {
                            $this->storeResult('ga-access', $link, false, $executedAt, $official);
                        }

                        if ($taxonomy) {
                            $this->storeResult('ga-taxonomy', $link, true, $executedAt, $official);
                        } else if ($ga4access) {
                            $this->storeResult('ga-taxonomy', $link, true, $executedAt, $official);
                        } else {
                            $this->storeResult('ga-taxonomy', $link, false, $executedAt, $official);
                        }
                    }
                } else {
                    throw new Exception('Failed to retrive requests from Mongo for current period');
                }
            }
            $retorno = 0;
        } catch (Exception $e) {
            $this->error('Error. Line: ' . $e->getLine() . ' - Message: ' . $e->getMessage());

            app(CrawlerHelper::class)->storeSourceCode(
                crawler: 'crawler:googleAnalytics',
                link: '-',
                content: $e->getMessage(),
                failed: true,
                executedAt: $executedAt,
                official: $official,
            );

            $retorno = 1;
        } finally {
            return $retorno;
        }
    }

    /**
     * Identify account inside our accounts
     *
     * @param string $ua
     *
     * @return ?stdClass
     */
    protected function identifyAccount(string $ua): ?stdClass
    {
        $account = null;

        foreach ($this->analytics as $value) {
            if ($ua === $value->measurement_id) {
                $account = $value;
                break;
            }
        }

        return $account;
    }

    /**
     * Initialize BigQuery Client
     *
     * @return BigQueryClient
     */
    protected function initializeBqClient(): BigQueryClient
    {
        $big = null;

        try {
            // Go 3 leves up from the current directory
            $path = dirname(__DIR__, 3);

            $big = new BigQueryClient([
                'keyFile' =>
                json_decode(file_get_contents($path . '/' . env('GCP_CREDENTIALS_FILE')), true),
            ]);
        } catch (Exception $e) {
            $this->line('Failed to initialize BQ Client. Error message: ' . $e->getMessage());
        } finally {
            return $big;
        }
    }

    /**
     * Initialize Google Client with GCP Credentials
     *
     * @return Google_Client
     */
    protected function initializeGA(): Google_Client
    {
        $client = new Google_Client();

        try {
            $client->setClientId(env('GCP_CLIENT_ID'));
            $client->setClientSecret(env('GCP_CLIENT_SECRET'));
            $client->setScopes(array('https://www.googleapis.com/auth/analytics.readonly'));
            $client->setAccessType('offline');
            $client->setApprovalPrompt('consent');
            $client->setIncludeGrantedScopes(true);

            $client->refreshToken(env('GA_REFRESH_TOKEN'));
        } catch (Exception $e) {
            $this->error($e->getMessage());
        }

        return $client;
    }

    /**
     * Initialize Google Analytics Service
     *
     * @param Google_Client $client
     *
     * @return Google_Service_Analytics
     */
    protected function initializeGAService($client): Google_Service_Analytics
    {
        return new Google_Service_Analytics($client);
    }

    /** Store the result for an specific test
     *
     * @param string $test
     * @param string $link
     * @param bool $response
     * @param string $executedAt
     *
     * @return void
     */
    protected function storeResult(string $test, string $link, bool $response, string $executedAt = '', bool $official = false): void
    {
        $response = ($response) ? 'true' : 'false';

        try {
            $storer = app(CrawlerHelper::class)->storeResult(
                link: $link,
                criteria: $test,
                response: $response,
                executedAt: $executedAt,
                official: $official,
            );

            if ($storer['message'] !== 'success') {
                throw new Exception($storer['message']);
            }

            // $this->line('Storing ' . $response . ' for ' . $test . ' at ' . $link);
        } catch (Exception $e) {
            $this->line('Error! Link: ' . $link . ', test: ' . $test . '  - error: ');
            $this->error($e->getMessage());
        }

        return;
    }

    /**
     * Convert array to a CSV string
     *
     * @param array $data
     *
     * @return string
     */
    protected function str_putcsv($data): string
    {
        # Generate CSV data from array
        $fh = fopen('php://temp', 'rw'); # don't create a file, attempt
        # to use memory instead

        # write out the headers
        // fputcsv($fh, array_keys(current($data)));

        # write out the data
        foreach ($data as $row) {
            fputcsv($fh, $row);
        }
        rewind($fh);
        $csv = stream_get_contents($fh);
        fclose($fh);

        return $csv;
    }

    /**
     * Validate if there access to an specific Analytics Property
     *
     * @param string $ua
     *
     * @return bool
     */
    protected function validateAccess(string $ua): bool
    {
        $access = false;

        foreach ($this->analytics as $value) {
            if ($ua === $value->measurement_id) {
                $access = true;
                break;
            }
        }

        return $access;
    }

    /**
     * Validate if Analytics Taxonomy is follow patterns
     *
     * @param stdClass $account
     *
     * @return bool
     */
    protected function validateTaxonomy(stdClass $account): bool
    {
        $taxonomy = false;

        if (in_array($account->account, $this->allowedTaxonomy)) {
            $taxonomy = true;
        }

        return $taxonomy;
    }
}
