<?php

// phpcs:disable PSR2.ControlStructures.ElseIfDeclaration.NotAllowed

namespace App\Console\Commands;

use App\Models\BpocWebsiteList;
use App\Models\CookieComplianceReport;
use App\Models\CookiesWebsite;
use App\Models\Crawls;
use App\Models\Links;
use App\Models\OnetrustCookies;
use Exception;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\Cache;
use stdClass;

/**
 * Cookie Validator
 *
 * For a given domain, validate the cookie compliance
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Console\Commands
 */

class ValidateCookies extends Command
{
    /**
     * The console command description.
     */
    protected $description = 'For a given domain validate if all cookies are compliance';

    /**
     * Hold current DGP Site List
     *
     * @var array
     */
    protected array $dgpSiteList = [];

    /**
     * List of cookies to be ignored
     */
    protected $ignoreCookieList = [
        'OptanonConsent',
        'OptanonAlertBoxClosed',
        'ki_t',
        'km_vs',
        'sp_t',
        'incap_ses_',
        'nlbi_',
        'visid_incap_',
        'reese84',
        'x-d-token',
        'STYXKEY_anonymousUserBirthDate',
        'ARRAffinitySameSite',
        'ARRAffinity',
        '___utm',
        '.ASPXAUTH',
        'PHPSESSID',
        '__insp_pad',
        '__insp_targlpu',
        '__insp_nv',
        '__insp_wid',
        '__insp_targlpt',
        '__insp_sid',
        '__insp_slim',
        '__insp_uid',
    ];

    /**
     * List of websites to be ignored
     */
    protected $ignoreSiteList = [
        '100accelerator.com',
        'pedidadehoje.com.br',
    ];

    /**
     * Exception applied only for VTEX cookies
     */
    protected $onlyVtexException = [
        '_hjAbsoluteSessionInProgress',
        '_hjIncludedInPageviewSample',
        '_hjFirstSeen',
        '_hjIncludedInSessionSample',
        '_hjSessionUser_',
        '_hjSession_',
    ];

    /**
     * Shopify Cookie List (exception outside EU)
     */
    protected $shopifyCookieList = [
        '_shopify_m',
        '_shopify_tm',
        '_shopify_tw',
        '_tracking_consent',
        'cart',
        'cart_currency',
        'cart_sig',
        'cart_ts',
        'cart_ver',
        'keep_alive',
        'secure_customer_sig',
        'shopify_pay_redirect',
        '_landing_page',
        '_orig_referrer',
        '_s',
        '_shopify_s',
        '_shopify_sa_p',
        '_shopify_sa_t',
        '_shopify_y',
        '_y',
        '_shopify_evids',
        '_session_id',
        'ki_t',
        'localization',
    ];

    /**
     * The name and signature of the console command.
     */
    protected $signature = 'validate:cookies
    {link : Domain link to be evaluated}
    {zone : Zone that the link belongs to}
    {country : Country that the links belongs to}
    {executed_at : The execution date}
    {official : If this execution is official}';

    /**
     * VTEX cookie exception list
     */
    protected $vtexCookieList = [
        'biggy-session-',
        'biggy-anonymous',
        'biggy-event-queue',
        'checkout.vtex.com',
        'vtex_segment',
        'vtex_session',
        'ASP.NET_SessionId',
        'connect.sid',
        'VtexRCMacIdv7',
        'VtexRCSessionIdv7',
        'VtexWorkspace',
    ];

    /**
     * Based on cookies on the website, try to discover the site generator
     *
     * @param stdClass $cookies
     *
     * @return ?string
     */
    protected function discoverGenerator(stdClass $cookies): ?string
    {
        $generator = null;

        $provider = (object) array(
            'vtex' => 0,
            'shopify' => 0,
        );

        foreach ($cookies->strictlyNeeded as $cookie) {
            $match = false;

            foreach ($this->vtexCookieList as $vtexCookie) {
                if (strpos($cookie, $vtexCookie) === 0) {
                    $provider->vtex++;
                    $match = true;
                    break;
                }
            }

            if ($match) {
                continue;
            }

            foreach ($this->shopifyCookieList as $shopifyCookie) {
                if (strpos($cookie, $shopifyCookie) === 0) {
                    $provider->shopify++;
                    $match = true;
                    break;
                }
            }
        }

        if ($provider->vtex > 4) {
            $generator = 'vtex';
        } else if ($provider->shopify > 5) {
            $generator = 'shopify';
        } else {
            $generator = null;
        }

        return $generator;
    }

    /**
     * For a given cookie and cookie list, find the closest match inside the list
     *
     * @param string $cookieName
     * @param array $cookies
     *
     * @return stdClass
     */
    protected function foundClosest(string $cookieName, array $cookies): stdClass
    {
        $entry = new stdClass();
        $entry->cookie = null;
        $entry->category = null;
        $entry->distance = null;

        $currentDistance = -1;

        foreach ($cookies as $value) {
            $distance = levenshtein($cookieName, $value['cookie'], 1, 1, 1);

            // Exact match
            if ($distance === 0) {
                $entry->cookie = $value['cookie'];
                $entry->category = $value['defaultCategory'];
                $entry->distance = 0;

                break;
            }

            // New match
            if (substr($cookieName, 0, 3) === substr($value['cookie'], 0, 3) && ($distance <= $currentDistance || $currentDistance < 0)) {
                $currentDistance = $distance;
                $entry->cookie = $value['cookie'];
                $entry->category = $value['defaultCategory'];
                $entry->distance = $distance;
            }
        }

        return $entry;
    }

    /**
     * Retrieve the current DGP Website List
     *
     * @return void
     */
    protected function getDGpSites(): void
    {
        if (Cache::has('dgp-site-list')) {
            $links = Cache::get('dgp-site-list');
            $this->line('from cache');
        } else {
            try {
                $links = Links::select('link')->where('consider', true)->get();

                foreach ($links as $link) {
                    array_push($this->dgpSiteList, strtolower($link->link));
                }

                Cache::put('dgp-site-list', $links, now()->addMinutes(120));
            } catch (Exception $e) {
                $this->error('Failed to retrieve DGP website list. Error message: ' . $e->getMessage());
            }
        }
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle(): int
    {
        $return = 0;

        $domain = $this->argument('link');
        $executedAt = $this->argument('executed_at');
        $zone = strtolower($this->argument('zone'));
        $country = ($this->argument('country') ? $this->argument('country') : null);

        try {
            $linkData = BpocWebsiteList::where('link', 'ilike', $domain)->firstOrFail();

            $this->getDGpSites();

            // Delete all informations stored for this website
            CookiesWebsite::where('link', $domain)->delete();

            $crawl = Crawls::where([
                ['data_type', 'crawler:cookies'],
                ['link', $domain],
            ])->first();

            if (is_null($crawl)) {
                throw new Exception('No cookies related information for ' . $domain);
            }

            $content = json_decode($crawl->content);

            $scriptStatusData = OnetrustCookies::where('domain', 'ilike', $linkData->link)->orWhere('domain', 'ilike', 'www.' . $linkData->link)->distinct('scriptStatus')->get('scriptStatus');

            $scriptStatus = null;

            if (count($scriptStatusData) > 0) {
                foreach ($scriptStatusData as $innerStatus) {
                    if (is_null($scriptStatus)) {
                        $scriptStatus = $innerStatus->scriptStatus;
                    }

                    if ($innerStatus->scriptStatus === 'Published') {
                        $scriptStatus = $innerStatus->scriptStatus;
                        break;
                    }
                }
            }

            $cookieBannerImplemented = false;
            $cookieBannerSameDomain = '';

            if (isset($content->cookieBannerImplemented)) {
                $cookieBannerImplemented = ($content->cookieBannerImplemented ? 1 : 0);
            }

            if (isset($content->cookieBannerDomain)) {
                $cookieBannerDomain = $content->cookieBannerDomain;
                $cookieBannerSameDomain = $content->cookieBannerSameDomain; // @phpstan-ignore-line
            }

            $domainCompliance = $cookieBannerImplemented || ($scriptStatus === 'Published');

            $wrongStrictly = 0;

            $domainCookiesExpected = [];

            $expectedCookies = OnetrustCookies::where('domain', 'ilike', $domain)->orWhere('domain', 'ilike', 'www.' . $domain)->get();

            $expectedCookiesNumber = (count($expectedCookies));

            if (count($content->cookies) > 0) {
                $generator = $this->discoverGenerator($content->cookies);

                foreach ($content->cookies as $key => $value) {
                    $category = $this->normalizeCategory($key);

                    if (is_array($value)) {
                        foreach ($value as $cookie) {
                            if (!($this->in_IgnoreCookieList($cookie))) {
                                $cookieStatus = '';

                                if (!is_null($generator)) {
                                    $cookieStatus = $this->handleException($cookie, $generator, $zone);
                                }

                                if ($expectedCookiesNumber > 0) {
                                    foreach ($expectedCookies as $expectedCookie) {
                                        if (!empty($expectedCookie->cookieName)) {
                                            $count = count($domainCookiesExpected);
                                            $domainCookiesExpected[$count]['cookie'] = $expectedCookie->cookieName;
                                            $domainCookiesExpected[$count]['defaultCategory'] = $expectedCookie->defaultCategory;
                                        }
                                    }
                                } else {
                                    $count = count($domainCookiesExpected);
                                    $domainCookiesExpected[$count]['cookie'] = $cookie;
                                    $domainCookiesExpected[$count]['defaultCategory'] = null;
                                }

                                $closestCookie = $this->foundClosest($cookie, $domainCookiesExpected);

                                if ($cookieStatus === '') {
                                    if (($zone === 'afr' || $zone === 'africa') && $category === 'Strictly Necessary Cookies' && $closestCookie->category === 'Performance Cookies') {
                                        $cookieStatus = 'Correct';
                                    } else if ((!is_null($country)) && strtolower($country) === 'korea') {
                                        $cookieStatus = 'Correct';
                                    } else if ($closestCookie->category === 'Strictly Necessary Cookies') {
                                        $cookieStatus = 'Correct';
                                    } else if ($category === $closestCookie->category) {
                                        $cookieStatus = 'Correct';
                                    } else if (!is_null($closestCookie->category)) {
                                        $cookieStatus = 'Different categorization';
                                    } else {
                                        $cookieStatus = 'Uncategorized cookie';
                                    }
                                }

                                if ($category === 'Strictly Necessary Cookies') {
                                    $cookieDroppingCompliance = ($cookieStatus === 'Correct' ? 1 : 0);

                                    if (!$cookieDroppingCompliance) {
                                        $domainCompliance = false;

                                        $wrongStrictly++;
                                    }
                                }

                                $cookieWebsite = new CookiesWebsite();
                                $cookieWebsite->link = $linkData->link;
                                $cookieWebsite->zone = $linkData->zone;
                                $cookieWebsite->country = $linkData->country;
                                $cookieWebsite->dgp = in_array(strtolower($linkData->link), $this->dgpSiteList) ? true : false;
                                $cookieWebsite->cookie = $cookie;
                                $cookieWebsite->trigger = $category;
                                $cookieWebsite->expected = $closestCookie->category;
                                $cookieWebsite->status = $cookieStatus;

                                $cookieWebsite->save();
                            } else {
                                $cookieWebsite = new CookiesWebsite();
                                $cookieWebsite->link = $linkData->link;
                                $cookieWebsite->zone = $linkData->zone;
                                $cookieWebsite->country = $linkData->country;
                                $cookieWebsite->dgp = in_array(strtolower($linkData->link), $this->dgpSiteList) ? true : false;
                                $cookieWebsite->cookie = $cookie;
                                $cookieWebsite->trigger = $category;
                                $cookieWebsite->expected = null;
                                $cookieWebsite->status = 'Correct';

                                $cookieWebsite->save();
                            }
                        }
                    }
                }
            }

            $complianceReportEntry = new CookieComplianceReport();

            $complianceReportEntry->link = $linkData->link;
            $complianceReportEntry->zone = $linkData->zone;
            $complianceReportEntry->country = $linkData->country;
            $complianceReportEntry->dgp = in_array($linkData->link, $this->dgpSiteList) ? true : false;

            if ($this->in_IgnoreSiteList($linkData->link)) {
                $complianceReportEntry->cookie_banner_implemented = null; // @phpstan-ignore-line
                $complianceReportEntry->cookie_compliance = null; // @phpstan-ignore-line
                $complianceReportEntry->domain_compliance = true;
            } else {
                $complianceReportEntry->cookie_banner_implemented = $cookieBannerImplemented;
                $complianceReportEntry->cookie_compliance = ($wrongStrictly === 0 ? true : false);
                $complianceReportEntry->domain_compliance = $domainCompliance;
            }

            $complianceReportEntry->non_strictly = $wrongStrictly;
            $complianceReportEntry->executed_at = $executedAt;
            $complianceReportEntry->site_status = $content->siteStatus;
            $complianceReportEntry->scriptStatus = $scriptStatus;

            if (isset($cookieBannerDomain)) {
                $complianceReportEntry->banner_domain = $cookieBannerDomain;
                $complianceReportEntry->banner_same_domain = $cookieBannerSameDomain;
            }

            $complianceReportEntry->save();
        } catch (Exception $e) {
            $return = 1;
            $this->error('Cookie validation failed for ' . $domain . '. Error message: ' . $e->getMessage() . 'Line: ' . $e->getLine());
        } finally {
            $this->line('Validated ' . $domain . ' with status: ' . $return);
            return $return;
        }
    }

    /**
     * For a given cookie, platform provider (SaaS) and zone, handle with potential exceptions
     *
     * @param string $cookie
     * @param string $provider
     * @param string $zone
     *
     * @return string
     */
    protected function handleException(string $cookie, string $provider, string $zone): string
    {
        $exceptionStatus = '';

        if ($provider === 'vtex') {
            foreach ($this->vtexCookieList as $vtexCookie) {
                if (strpos($cookie, $vtexCookie) === 0) {
                    $exceptionStatus = 'Correct';
                    break;
                }
            }

            if ($exceptionStatus === '') {
                foreach ($this->onlyVtexException as $vtexCookie) {
                    if (strpos($cookie, $vtexCookie) === 0) {
                        $exceptionStatus = 'Correct';
                        break;
                    }
                }
            }
        } else if ($provider === 'shopify' && strtolower($zone) !== 'eur') {
            foreach ($this->shopifyCookieList as $shopifyCookie) {
                if (strpos($cookie, $shopifyCookie) === 0) {
                    $exceptionStatus = 'Correct';
                    break;
                }
            }
        }

        return $exceptionStatus;
    }

    /**
     * Verify if cookie is inside ignore list
     *
     * @param string $cookie
     *
     * @return boolean
     */
    protected function in_IgnoreCookieList(string $cookie): bool
    {
        $return = false;
        foreach ($this->ignoreCookieList as $ignored) {
            if (strpos($cookie, $ignored) === 0) {
                $return = true;
                break;
            }
        }

        return $return;
    }

    /**
     * Verify if site is inside ignore list
     *
     * @param string $site
     *
     * @return boolean
     */
    protected function in_IgnoreSiteList(string $site): bool
    {
        $link = strtolower(rtrim(preg_replace('(^https?://)', '', $site), '/'));

        $return = false;
        foreach ($this->ignoreSiteList as $ignored) {
            if (strpos($link, $ignored) === 0) {
                $return = true;
                break;
            }
        }

        return $return;
    }

    /**
     * Normalize category name
     *
     * @param string $category
     *
     * @return String
     */
    protected function normalizeCategory(string $category): String
    {
        $return = null;

        switch ($category) {
            case 'strictlyNeeded':
                $return = 'Strictly Necessary Cookies';
                break;
            case 'performance':
                $return = 'Performance Cookies';
                break;
            case 'functionality':
                $return = 'Functional Cookies';
                break;
            case 'socialNetwork':
                $return = 'Social Media Cookies';
                break;
            case 'targeting':
                $return = 'Targeting Cookies and Other Trackers';
                break;

            default:
                $return = $category;
                break;
        }

        return $return;
    }
}
