<?php
// phpcs:disable BestIt.DocTags.RequiredPropertyTags.TagOccurrenceMinVar
// phpcs:disable BestIt.TypeHints.PropertyTypeHint.MissingAnyTypeHint
// phpcs:disable BestIt.DocTags.ParamTag.MixedType

namespace App\Console;

use App\Models\Crawls;
use App\Models\CrawlsFailed;
use App\Models\Criteria;
use App\Models\Keywords;
use App\Models\Locale as Locale;
use Exception;
use GuzzleHttp\Client;

/**
 * CrawlerHelper Class
 *
 * Crawler's common functions
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Console
 */
class CrawlerHelper
{
	/**
	 * Retrieve the urls inside a string
	 *
	 * @param string $text
	 *
	 * @return array
	 */
	public function extractUrlsFromText(string $text): array
	{
		$return = [];

		$urls = [];

		$regex = "#[-a-zA-Z0-9@:%_\+.~\#?&//=]{2,256}\.[a-z]{2,4}\b(\/[-a-zA-Z0-9@:%_\+.~\#?&//=]*)?#si";

		$url = (preg_match_all($regex, $text, $matches, PREG_OFFSET_CAPTURE));

		if ($url) {
			foreach ($matches[0] as $value) {
				// Validate if is an url
				if (filter_var($value[0], FILTER_VALIDATE_URL) === false) {
					continue;
				}

				// Ensure we removed additional dot at the end of the url
				$link = trim($value[0], '.');

				array_push($urls, $link);
			}

			$return = $urls;
		}

		return $return;
	}

	/**
	 * For a given url return the http code
	 *
	 * @param string $url
	 *
	 * @return int
	 */
	public function getHttpCode(string $url): int
	{
		$code = 0;

		try {
			$agent = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/104.0.0.0 Safari/537.36';

			$handle = curl_init($url);
			curl_setopt($handle, CURLOPT_RETURNTRANSFER, TRUE);
			curl_setopt($handle, CURLOPT_FOLLOWLOCATION, TRUE);
			curl_setopt($handle, CURLOPT_USERAGENT, $agent);
			curl_setopt($handle, CURLOPT_SSL_VERIFYPEER, false);
			curl_setopt($handle, CURLOPT_SSL_VERIFYHOST, false);

			curl_setopt($handle, CURLOPT_MAXREDIRS, 10);

			curl_setopt($handle, CURLOPT_CONNECTTIMEOUT, 30);
			curl_setopt($handle, CURLOPT_TIMEOUT, 30);

			curl_exec($handle);

			$httpCode = curl_getinfo($handle, CURLINFO_HTTP_CODE);
			$finalUrl = curl_getinfo($handle, CURLINFO_EFFECTIVE_URL);

			curl_close($handle);

			// Recursively ensure, we got the status for the final url
			if ($finalUrl !== $url) {
				$code = $this->getHttpCode($finalUrl);
			} else {
				$code = $httpCode;
			}
		} catch (Exception $e) {
			$code = 0;
		} finally {
			return $code;
		}
	}

	/**
	 * Return keywords
	 *
	 * @param string $criteria
	 * @param string $locale
	 * @param ?bool $social
	 *
	 * @return array
	 */
	public static function keywords(string $criteria, string $locale, ?bool $social): array
	{
		$keywords = [];

		try {
			$criteria = Criteria::where('slug', $criteria)->firstOrFail();

			$locale = explode('_', $locale)[0];

			$locales = Locale::where('locale', 'like', $locale . '%')->orWhere('locale', 'like', 'en%')->pluck('id')->toArray();

			$keywords = Keywords::whereIn('locale_id', $locales)->where([
				['criteria_id', $criteria->id],
			])->get();

			$return = array();

			foreach ($keywords as $value) {
				if ($value->group !== 'default') {
					if (!array_key_exists($value->group, $return)) {
						$return[$value->group] = [];
					}

					if (!(in_array($value->keyword, $return[$value->group]))) {
						array_push($return[$value->group], $value->keyword);
					}
				} else {
					if (!(in_array($value->keyword, $return))) {
						array_push($return, $value->keyword);
					}
				}
			}
		} catch (Exception $e) {
			//dd($e->getMessage());
			$return = [];
		}

		return $return;
	}

	/**
	 * Send a notification to Teams with the given text
	 *
	 * @param string $text
	 *
	 * @return void
	 */
	public function sendNotification(string $text = '', $team = 'GWA'): void
	{
		$url = '';
		switch ($team) {
			case 'GWA':
				$url = env('TEAMS_WEBHOOK_URL');
				break;
			case 'Cookie':
				$url = env('TEAMS_COOKIE_WEBHOOK_URL');
				break;
		}

		if (!empty($url)) {
			$curl = curl_init();

			curl_setopt_array($curl, array(
				CURLOPT_URL => $url,
				CURLOPT_RETURNTRANSFER => true,
				CURLOPT_ENCODING => '',
				CURLOPT_MAXREDIRS => 10,
				CURLOPT_TIMEOUT => 0,
				CURLOPT_FOLLOWLOCATION => true,
				CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
				CURLOPT_CUSTOMREQUEST => 'POST',
				CURLOPT_POSTFIELDS => '{"text": "' . $text . '"}',
				CURLOPT_HTTPHEADER => array(
					'Content-Type: application/json'
				),
			));

			curl_exec($curl);

			curl_close($curl);
		}
	}

	/**
	 * For a given text, split new lines
	 *
	 * @param string $text
	 *
	 * @return array
	 */
	public function splitNl($text): array
	{
		$text = str_replace('<BR>', "\n", str_replace('<br>', "\n", $text));

		return preg_split('/\n|\r\n?/', $text);
	}

	/**
	 * Store manual validation
	 *
	 * @param string $link
	 * @param string $exec_date
	 * @param bool $validated
	 * @param bool $use_on_report
	 *
	 * @return array
	 */
	public function storeManualValidationHelper(string $link, string $exec_date, bool $validated, bool $use_on_report): array
	{
		$return = array(
			'status' => 0,
			'message' => ''
		);

		try {
			$client = new Client();

			$res = $client->request('POST', env('API_DOMAIN') . '/api/manual-validation', [
				'json' => [
					'link' => $link,
					'exec_date' => $exec_date,
					'validated' => $validated,
					'use_on_report' => $use_on_report,
				],
			]);

			$return = array(
				'status' => 0,
				'message' => 'success'
			);
		} catch (Exception $e) {
			$return = array(
				'status' => 1,
				'message' => $e->getMessage()
			);
		} finally {
			return $return;
		}
	}

	/**
	 * Store response for a given criteria
	 *
	 * @param string $link
	 * @param string $criteria
	 * @param string $response
	 * @param string $executedAt
	 * @param bool $official
	 *
	 * @return array
	 */
	public function storeResult(string $link, string $criteria, string $response, string $executedAt, bool $official = false): array
	{
		$return = array(
			'status' => 0,
			'message' => ''
		);

		try {
			$client = new Client();

			$client->request('POST', env('API_DOMAIN') . '/api/link-criteria', [
				'json' => [
					'criteria' => $criteria,
					'link' => $link,
					'response' => $response,
					'executed_at' => $executedAt,
					'official' => $official,
				],
			]);

			$return = array(
				'status' => 0,
				'message' => 'success'
			);
		} catch (Exception $e) {
			$return = array(
				'status' => 1,
				'message' => $e->getMessage()
			);
		} finally {
			return $return;
		}
	}

	/**
	 * Store Source Code
	 *
	 * @param string $crawler
	 * @param string $link
	 * @param mixed $content
	 * @param bool $failed
	 * @param string $executedAt
	 * @param bool $official
	 * @param string $age
	 *
	 * @return void
	 */
	public function storeSourceCode(string $crawler, string $link, mixed $content, bool $failed, string $executedAt = '', bool $official = false, string $age = null): void
	{
		try {
			if (!$failed) {
				$crawl = new Crawls();

				if (!(is_null($age))) {
					$crawl->age = $age;
				}

				if (is_array($content) || is_object($content)) {
					$content = json_encode($content);
				}

				$crawl->data_type = $crawler;
				$crawl->content = $content;
				$crawl->link = $link;
				$crawl->executed_at = $executedAt;
				$crawl->official = $official;

				$crawl->save();

				$createdId = $crawl->id;

				if (is_numeric($createdId)) {
					$params = [];
					array_push($params, ['data_type', $crawler]);
					array_push($params, ['link', $link]);

					if (!(is_null($age))) {
						array_push($params, ['age', $age]);
					}

					$crawls = Crawls::where($params)->get();

					foreach ($crawls as $value) {
						if ($value->id !== $createdId) {
							Crawls::destroy($value->id);
						}
					}
				}
			} else {
				$crawl = new CrawlsFailed();

				$notificationMessage = '<b>Error!</b><ul><li><b>Crawler: </b>' . $crawler . '</li><b>Link:</b> ' . $link . '</li><li><b>Error:</b> ' . $content . '</li>';
				$this->sendNotification($notificationMessage);

				if (!(is_null($age))) {
					$crawl->age = $age;
				}

				if (is_array($content)) {
					$content = json_encode($content);
				}

				$crawl->data_type = $crawler;
				$crawl->content = $content;
				$crawl->link = $link;
				$crawl->executed_at = $executedAt;
				$crawl->official = $official;

				$crawl->save();

				$createdId = $crawl->id;

				if (is_numeric($createdId)) {
					$params = [];
					array_push($params, ['data_type', $crawler]);
					array_push($params, ['link', $link]);

					if (!(is_null($age))) {
						array_push($params, ['age', $age]);
					}

					$crawls = CrawlsFailed::where($params)->get();

					foreach ($crawls as $value) {
						if ($value->id !== $createdId) {
							CrawlsFailed::destroy($value->id);
						}
					}
				}
			}
		} catch (Exception $e) {
			echo $e->getMessage();
		}
	}
}
