/* eslint-disable no-unsafe-finally */
/* eslint-disable camelcase */
const { Pool } = require('pg');
const axios = require('axios');

require('dotenv').config();

async function psqlConnect() {
  if (global.connection) {
    return global.connection.connect();
  }

  try {
    const username = process.env.DB2_USERNAME;
    const password = process.env.DB2_PASSWORD;
    const host = process.env.DB2_HOST;
    const database = process.env.DB2_DATABASE;

    const pool = new Pool({
      connectionString: `postgres://${username}:${password}@${host}:5432/${database}`,
    });

    global.connection = pool;
    return pool.connect();
  } catch (error) {
    console.log(
      'Connection with database failed. Error message: ' + error.message
    );

    notifyTeams(
      'Lighthouse Parallel Crawler. Failed whrn trying to connect to database. Error message: ' +
        error.message
    );

    return null;
  }
}

async function getDgpLinks() {
  try {
    const db = await psqlConnect();

    if (db === undefined || db === null) {
      throw new Error('Unable to connect to database');
    }

    const links = await db.query(
      'SELECT l.id AS link_id, l.link, l.in_acquia, l.additional_crawler, c."name" AS country, z."name" AS zone, b."name" as brand FROM links l JOIN countries c ON c.id = l.country_id JOIN zones z ON z.id = c.zone_id join brands b on b.id  = l.brand_id  WHERE l.consider = true AND l.channel_id = (SELECT id FROM channels c2 WHERE c2.slug = \'site\') AND l.deleted_at is null order by l.link'
    );

    return links.rows;
  } catch (error) {
    console.log('Failed to get DGP Links. Error message: ' + error.message);

    notifyTeams(
      'Lighthouse Parallel Crawler. Failed to get DGP links. Error message: ' +
        error.message
    );

    return null;
  }
}

async function resolveRegion(zone, inAcquia) {
  zone = zone.toLowerCase();

  const cloudLocation = {
    region: '',
    script: '',
    url: '',
  };

  if (inAcquia || zone === 'Europe') {
    cloudLocation.region = process.env.GCP_CF_LIGHTHOUSE_EUR_REGION;
    cloudLocation.script = process.env.GCP_CF_LIGHTHOUSE_EUR_SCRIPT;
    cloudLocation.url = process.env.GCP_CF_LIGHTHOUSE_EUR_SCRIPTv2;
  } else if (zone === 'Asia Pacific') {
    cloudLocation.region = process.env.GCP_CF_LIGHTHOUSE_APAC_REGION;
    cloudLocation.script = process.env.GCP_CF_LIGHTHOUSE_APAC_SCRIPT;
    cloudLocation.url = process.env.GCP_CF_LIGHTHOUSE_APAC_SCRIPTv2;
  } else {
    cloudLocation.region = process.env.GCP_CF_LIGHTHOUSE_NAZ_REGION;
    cloudLocation.script = process.env.GCP_CF_LIGHTHOUSE_NAZ_SCRIPT;
    cloudLocation.url = process.env.GCP_CF_LIGHTHOUSE_NAZ_SCRIPTv2;
  }

  return cloudLocation;
}

async function crawl(entry, executed_at) {
  console.log('Starting ' + entry.link);
  try {
    const region = await resolveRegion(entry.zone, entry.in_acquia);

    console.log('Region: ' + region.region + '. Script: ' + region.script);

    let link = entry.link;

    if (entry.link.indexOf('http') !== 0) {
      link = 'http://' + link;
    }

    // Added to avoid JS redirect to us.budweiser.com
    if (
      link.toLowerCase().replace('http://', '').replace('https://', '') ===
      'budweiser.com'
    ) {
      link = link + '?cc=EU&nr=1';
    }

    const config = {
      method: 'post',
      url: region.url,
      headers: {
        'Content-Type': 'application/json',
      },
      data: JSON.stringify({
        link,
      }),
    };

    let rawResult = '';

    await axios(config)
      .then(function (response) {
        rawResult = response.data;
      })
      .catch(function (error) {
        throw new Error(error.response.data.error);
      });

    const fcp = rawResult.audits['first-contentful-paint'].displayValue;
    const lcp = rawResult.audits['largest-contentful-paint'].displayValue;
    const tbt = rawResult.audits['total-blocking-time'].displayValue;
    const si = rawResult.audits['speed-index'].displayValue;
    const tti = rawResult.audits.interactive.displayValue;
    const cls = rawResult.audits['cumulative-layout-shift'].displayValue;

    const score = rawResult.categories.performance.score * 100;

    await storeResults(
      entry.link_id,
      entry.link,
      entry.country,
      entry.zone,
      entry.brand,
      executed_at.getTime() / 1000,
      fcp,
      lcp,
      tbt,
      si,
      tti,
      cls,
      parseInt(score)
    );

    await storeSource(
      entry.link,
      executed_at.getTime() / 1000,
      JSON.stringify(rawResult),
      true,
      false
    );
    return;
  } catch (error) {
    notifyTeams(
      'Lighthouse Parallel Crawler. Failed when crawling: <b>' +
        entry.link +
        '</b>. Error message: ' +
        error.message
    );

    await storeSource(
      entry.link,
      executed_at.getTime() / 1000,
      error.message,
      true,
      true
    );

    return error.message;
  }
}

async function deleteOldSources(link) {
  try {
    const db = await psqlConnect();

    await db.query(
      `DELETE FROM crawls where data_type = 'crawler:lighthouse' and link = '${link}';`
    );

    await db.query(
      `DELETE FROM crawls_failed where data_type = 'crawler:lighthouse' and link = '${link}';`
    );

    db.release();

    return 0;
  } catch (error) {
    console.log(
      'Failed when deleting old Mongo entries for: ' +
        link +
        '. Error message: ' +
        error.message
    );

    notifyTeams(
      'Lighthouse Parallel Crawler. Failed when deleting old sources. Error message: ' +
        error.message
    );

    return null;
  }
}

async function storeSource(link, executed_at, content, official, failed) {
  console.log('Storing source for ' + link);
  try {
    await deleteOldSources(link);

    const db = await psqlConnect();

    const table = failed ? 'crawls_failed' : 'crawls';

    await db.query(
      `INSERT INTO ${table} (link, data_type, content, executed_at, official) VALUES ($1, 'crawler:lighthouse', $2, to_timestamp($3), $4)`,
      [link, content, executed_at, official]
    );

    await db.release();
  } catch (error) {
    console.log(error.message);

    notifyTeams(
      'Lighthouse Parallel Crawler. Failed when storing source for <b>' +
        link +
        '</b>. Error message: ' +
        error.message
    );
  } finally {
    return;
  }
}

async function storeResults(
  link_id,
  link,
  country,
  zone,
  brand,
  executed_at,
  fcp,
  lcp,
  tbt,
  si,
  tti,
  cls,
  last
) {
  console.log('Storing results for ' + link);
  try {
    const db = await psqlConnect();

    if (db === undefined || db === null) {
      throw new Error('Unable to connect to database');
    }

    const averageData = await db.query(
      `select avg(score) as average from (select (lighthouse_last) as score from lighthouse_data ld where ld.link_id = ${link_id} and ld.executed_at <= to_timestamp(${executed_at}) order by executed_at desc limit 4) as inn`
    );

    let average = averageData.rows[0].average;

    if (average === undefined || average === null) {
      average = last;
    }

    average = parseInt(average);

    await db.query(
      `INSERT into lighthouse_data (link_id, link, country, zone, brand, executed_at, first_contentful_paint, largest_contentful_paint, total_blocking_time, speed_index, time_to_interactive, cumulative_layout_shift, lighthouse_avg, lighthouse_last) VALUES($1, $2, $3, $4, $5, to_timestamp($6), $7, $8, $9, $10, $11, $12, $13, $14);`,
      [
        link_id,
        link,
        country,
        zone,
        brand,
        executed_at,
        fcp,
        lcp,
        tbt,
        si,
        tti,
        cls,
        average,
        last,
      ]
    );

    db.release();

    return 0;
  } catch (error) {
    console.log(
      'Failed to store result for: ' +
        link_id +
        '. Error message: ' +
        error.message
    );

    notifyTeams(
      'Lighthouse Parallel Crawler. Failed when storing results for <b>' +
        link +
        '</b>. Error message: ' +
        error.message
    );

    return 1;
  }
}

async function notifyTeams(text) {
  const data = JSON.stringify({
    text,
  });

  const config = {
    method: 'post',
    url: process.env.TEAMS_WEBHOOK_URL,
    headers: {
      'Content-Type': 'application/json',
    },
    data,
  };

  await axios(config)
    .then(() => {
      console.log('Notification sent');
    })
    .catch((error) => {
      console.log(
        'Failed when sending notification. Erro message: ' + error.message
      );
    });
}

const delay = (ms) =>
  new Promise((resolve) => {
    console.log('delayed');
    setTimeout(resolve, ms);
  });

async function getFailedLinks() {
  try {
    const db = await psqlConnect();

    if (db === undefined || db === null) {
      throw new Error('Unable to connect to database');
    }

    const links = await db.query(
      'SELECT l.id AS link_id, l.link, l.in_acquia, l.additional_crawler, c."name" AS country, z."name" AS zone, b."name" as brand from crawls_failed cf join links l on l.link = cf.link JOIN countries c ON c.id = l.country_id JOIN zones z ON z.id = c.zone_id join brands b on b.id  = l.brand_id  WHERE l.consider = true AND l.channel_id = (SELECT id FROM channels c2 WHERE c2.slug = \'site\') AND l.deleted_at is null and cf.data_type = \'crawler:lighthouse\' order by l.link'
    );

    return links.rows;
  } catch (error) {
    console.log('Failed to get failed Links. Error message: ' + error.message);

    notifyTeams(
      'Lighthouse Parallel Crawler. Failed to get failed links. Error message: ' +
        error.message
    );

    return null;
  }
}

async function handleQueue(links, executed_at, failed) {
  if (failed) {
    console.log('Starting to reattempt failed links');

    notifyTeams('Starting to reattempt failed links');
  } else {
    console.log('Starting usual flow');
  }

  let count = 0;

  const queue = [];

  for await (const entry of links) {
    if (entry.country === 'Ukraine' || entry.country === 'Russia') {
      continue;
    } else {
      queue.push(entry);
    }
  }

  if (queue.length > 0) {
    do {
      const current = queue.shift();

      // If our queue is empty, lets wait some time to ensure all process ended
      if (current === undefined) {
        await delay(300000);

        return;
      } else {
        crawl(current, executed_at);
      }

      count++;

      if (count % 25 === 0) {
        // eslint-disable-next-line promise/param-names
        await new Promise((r) => setTimeout(r, 150000));
      }
    } while (true);
  }
}

async function clearCache() {
  let data = '';

  const config = {
    method: 'get',
    url: 'https://dgp.ab-inbev.com/php/clear-cache',
  };

  await axios(config)
    .then(() => {
      data = 'ok';
    })
    .catch(() => {
      data = 'failed';
    });

  return data;
}

async function main() {
  const executed_at = new Date();

  let links = [];

  links = await getDgpLinks();

  await handleQueue(links, executed_at, false);

  links = await getFailedLinks();

  await handleQueue(links, executed_at, true);

  await clearCache();
}

async function handle() {
  await notifyTeams('Starting Parallel Lighthouse Crawler');
  await main();
  await notifyTeams(
    'Finished the Parallel Ligthouse Crawler execution. Have a nice week!'
  );
  process.exit();
}

handle();
