<?php

namespace App\Http\Controllers;

use App\Models\ssoSessions;
use App\Models\Tokens;
use App\Models\User;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Str;
use stdClass;

/**
 * ApiToken Controller
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Http\Controllers
 */

class ApiTokenController extends Controller
{
    /**
     * Generate a new token for the user
     *
     * @param Request $request
     *
     * @return JsonResponse
     */
    public function generate(Request $request): JsonResponse
    {
        $response = [];

        $identifier = '';
        $status = 0;

        if ($request->header('ssoToken') !== null) {
            $identifier = explode('|', Crypt::decryptString(urldecode($request->header('ssoToken'))));

            $identifier = $identifier[count($identifier) - 1];
        }

        if (strlen($identifier) === 0) {
            $identifier = $request->cookie('session_identifier');
        }

        try {
            $session = ssoSessions::where('session_identifier', $identifier)->first();

            if (is_null($session)) {
                throw new Exception('Failed to get session for ' . $identifier);
            }

            $token = new Tokens();

            $tok = Crypt::encryptString(Str::random(32));

            $token->user_id = $session->user_id;
            $token->token = $tok;
            $token->save();

            $status = 201;

            $response = [
                'status' => 'success',
                'message' => 'token created',
                'created_at' => $token->created_at,
                'token' => $tok,
            ];
        } catch (Exception $e) {
            $response = [
                'status' => 'error',
                'message' => $e->getMessage(),
                'line' => $e->getLine(),
            ];

            $status = 500;
        } finally {
            return response()->json($response, $status);
        }
    }

    /**
     * Retrieve tokens for current user
     *
     * @param Request $request
     *
     * @return JsonResponse
     */
    public function retrieve(Request $request): JsonResponse
    {
        $identifier = '';

        if ($request->header('ssoToken') !== null) {
            $identifier = explode('|', Crypt::decryptString(urldecode($request->header('ssoToken'))));

            $identifier = $identifier[count($identifier) - 1];
        }

        if (strlen($identifier) === 0) {
            $identifier = $request->cookie('session_identifier');
        }

        $response = [];

        $status = 0;

        try {
            $session = ssoSessions::where('session_identifier', $identifier)->first();

            if (is_null($session)) {
                throw new Exception('Failed to get session for ' . $identifier);
            }

            $tokens = Tokens::select('token', 'created_at', 'deleted_at')->where('user_id', $session->user_id)->withTrashed()->orderBy('created_at', 'desc')->get();

            $data = [];

            foreach ($tokens as $value) {
                $maskedToken = substr($value->token, 0, 30);

                $maskedToken .= '●●●●●';

                $maskedToken .= substr($value->token, 226);

                array_push($data, [
                    'token' => $maskedToken,
                    'created_at' => $value->created_at,
                    'deleted_at' => $value->deleted_at,
                ]);
            }

            $response = [
                'status' => 'success',
                'message' => 'tokens retrieved',
                'data' => $data,
            ];

            $status = 200;
        } catch (Exception $e) {
            $response = [
                'status' => 'error',
                'message' => $e->getMessage(),
                'line' => $e->getLine(),
            ];

            $status = 500;
        } finally {
            return response()->json($response, $status);
        }
    }

    /**
     * Validate the Bearer token and return user id
     *
     * @param String $bearer
     *
     * @return stdClass|null
     */
    public function validateBearer(String $bearer): ?stdClass
    {
        $return = null;

        try {
            $token = Tokens::where('token', $bearer)->first();

            if (is_null($token)) {
                throw new Exception('Failed to get token');
            }

            $user = User::where('id', $token->user_id)->first();

            if (is_null($user)) {
                throw new Exception('Failed to get user');
            }

            $return = new stdClass();
            $return->user_id = $user->id;
            $return->token_id = $token->id;
        } catch (Exception $e) {
            $return = null;
        } finally {
            return $return;
        }
    }
}
