<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use App\Models\ssoSessions;
use App\Models\User;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Cookie;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Str;
use stdClass;

/**
 * UserAuth (API authentication) Controller
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Http\Controllers\Auth
 */

class UserAuthController extends Controller
{
	/**
	 * Finr or Create a User based on the given email
	 *
	 * @param string $name
	 * @param string $email
	 *
	 * @return User|null
	 */
	protected function findOrCreateUser(string $name, string $email, bool $canCreate = true): User|null
	{
		$user = null;

		try {
			$user = User::where('email', $email)->first();

			// User doesn't exists
			if (is_null($user)) {
				if ($canCreate) {
					$user = new User();

					$user->name = $name;
					$user->email = $email;

					$user->save();
				}
			}
		} catch (Exception $e) {
			// TODO: handle with error message
			$user = null;
		} finally {
			return $user;
		}
	}

	/**
	 * For a given code from Azure AD SSO decode informations
	 *
	 * @param string $code
	 *
	 * @return stdClass
	 */
	protected function sso_getAccessToken(string $code): stdClass
	{
		$return = new stdClass();

		try {
			$post = [
				'client_id' => env('SSO_CLIENT_ID'),
				'scope' => 'openid email offline_access',
				'code' => $code,
				'grant_type' => 'authorization_code',
				'client_secret' => env('SSO_CLIENT_SECRET'),
				'redirect_uri' => env('SSO_REDIRECT_URL'),
			];

			$curl = curl_init();

			$url = 'https://login.microsoftonline.com/' . env('SSO_TENANT_ID') . '/oauth2/v2.0/token';

			curl_setopt($curl, CURLOPT_URL, $url);
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($post));
			curl_setopt($curl, CURLOPT_POST, true);
			curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type: application/x-www-form-urlencoded'));

			$tokenRequest = curl_exec($curl);
			curl_close($curl);

			if (curl_errno($curl)) {
				throw new Exception(curl_error($curl));
			}

			$data = json_decode($tokenRequest);

			if (isset($data->error)) {
				throw new Exception($data->error_description); // @phpstan-ignore-line
			}

			$token = explode('.', $data->access_token)[1];

			$decoded = json_decode(base64_decode($token));

			$return->status = 'success';

			$return->email = strtolower($decoded->upn);
			$return->exp = $decoded->exp;
			$return->name = $decoded->given_name . ' ' . $decoded->family_name;
			$return->refresh_token = $data->refresh_token;
		} catch (Exception $e) {
			$return->status = 'error';
			$return->message = $e->getMessage();
		} finally {
			return $return;
		}
	}

	/**
	 * Handle with a success authentication at Azure AD
	 *
	 * @param Request $request
	 *
	 * @return RedirectResponse
	 */
	public function sso_login(Request $request): RedirectResponse
	{
		$statusCode = 0;

		$expiresCookie = null;
		$sessionCookie = null;

		try {
			// Ensure that the login process was initialized from us
			if ($request->state === 'dgp_sso_login') {
				$ssoData = $this->sso_getAccessToken($request->code);

				$user = $this->findOrCreateUser($ssoData->name, $ssoData->email);

				$session = $this->storeSession($user, $ssoData);

				$expiresCookie = Cookie::make(
					name: 'sso_expires',
					value: $ssoData->exp,
					httpOnly: false,
					sameSite: 'none',
					secure: true,
				);

				$sessionCookie = Cookie::make(
					name: 'session_identifier',
					value: $session->session_identifier,
					httpOnly: false,
					sameSite: 'none',
					secure: true,
				);

				$statusCode = 200;
			} else {
				$statusCode = 401;
			}
		} catch (Exception $e) {
			$statusCode = 500;
		} finally {
			$return = null;

			if ($statusCode === 200) {
				$return = redirect(env('APP_URL', 'http://localhost/php') . '/vue-dgp?success')->withCookies([$expiresCookie, $sessionCookie]);
			} else {
				$return = redirect(env('APP_URL', 'http://localhost/php') . '/vue-dgp?error');
			}

			return $return;
		}
	}

	/**
	 * Renew SSO Authentication
	 *
	 * @param ssoSessions $session
	 *
	 * @return stdClass
	 */
	protected function sso_renewAuthentication($session): stdClass
	{
		$refreshToken = $session->refresh_token;

		$return = new stdClass();

		$basicAuthentication = false;

		$token = '';

		try {
			$token = explode('|', Crypt::decryptString($refreshToken));

			$token = $token[count($token) - 1];

			$basicAuthentication = true;
		} catch (Exception $e) {
			$token = '';

			$basicAuthentication = false;
		}

		try {
			if (!$basicAuthentication) {
				$post = [
					'client_id' => env('SSO_CLIENT_ID'),
					'scope' => 'openid email offline_access',
					'refresh_token' => $refreshToken,
					'grant_type' => 'refresh_token',
					'client_secret' => env('SSO_CLIENT_SECRET'),
					'redirect_uri' => env('SSO_REDIRECT_URL'),
				];

				$curl = curl_init();

				$url = 'https://login.microsoftonline.com/' . env('SSO_TENANT_ID') . '/oauth2/v2.0/token';

				curl_setopt($curl, CURLOPT_URL, $url);
				curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
				curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($post));
				curl_setopt($curl, CURLOPT_POST, true);
				curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type: application/x-www-form-urlencoded'));

				$tokenRequest = curl_exec($curl);
				curl_close($curl);

				if (curl_errno($curl)) {
					throw new Exception(curl_error($curl));
				}

				$data = json_decode($tokenRequest);

				if (isset($data->error)) {
					throw new Exception($data->error_description); // @phpstan-ignore-line
				}

				$token = explode('.', $data->access_token)[1];

				$decoded = json_decode(base64_decode($token));

				$return->status = 'success';

				$return->email = strtolower($decoded->upn);
				$return->exp = $decoded->exp;
				$return->name = $decoded->given_name . ' ' . $decoded->family_name;
				$return->refresh_token = $data->refresh_token;
			} else {
				$return->status = 'success';

				$user = $session->user;

				$expirationTimestamp = now()->addHours(24)->timestamp;

				$return->email = strtolower($user->email);
				$return->exp = $expirationTimestamp;
				$return->name = $user->name;
				$return->refresh_token = Crypt::encryptString(Str::orderedUuid());
			}
		} catch (Exception $e) {
			$return->status = 'error';
			$return->message = $e->getMessage();
		} finally {
			return $return;
		}
	}

	/**
	 * Validate SSO token stored at cookies
	 *
	 * @param Request $request
	 * @param bool $canStoreSession
	 * @param bool $fromMiddleware
	 *
	 * @return JsonResponse
	 */
	public function sso_validateToken(Request $request, bool $canStoreSession = true, bool $fromMiddleware = false): JsonResponse
	{
		$jsonResponse = [];
		$statusCode = 500;

		$expirationTime = 0;

		try {
			$currentEpoch = time();

			$expires = '';
			$sessionIdentifier = '';

			if ($request->header('ssoExpires') !== null) {
				$expires = explode('|', Crypt::decryptString(urldecode($request->header('ssoExpires'))));

				$expires = $expires[count($expires) - 1];

				$expirationTime = $expires;

				$sessionIdentifier = explode('|', Crypt::decryptString(urldecode($request->header('ssoToken'))));

				$sessionIdentifier = $sessionIdentifier[count($sessionIdentifier) - 1];
			}

			if (strlen($expires) === 0 && strlen($sessionIdentifier) === 0) {
				$expirationTime = (int) $request->cookie('sso_expires');
				$sessionIdentifier = $request->cookie('session_identifier');
			}

			if ($currentEpoch >= $expirationTime) {
				$session = ssoSessions::where('session_identifier', $sessionIdentifier)->latest('id')->first();

				if (is_null($session)) {
					throw new Exception($sessionIdentifier);
				}

				$ssoData = $this->sso_renewAuthentication($session);

				$user = $this->findOrCreateUser($ssoData->name, $ssoData->email, false);

				if (is_null($user)) {
					throw new Exception('Failed to find user');
				}

				if ($canStoreSession) {
					$session = $this->storeSession($user, $ssoData);
				}

				$expires = Crypt::encryptString($ssoData->exp);

				$sessionId = Crypt::encryptString($session->session_identifier);

				$statusCode = 201;

				$userRegister = User::join('user_level', 'users.user_level_id', '=', 'user_level.id')
					->where('users.id', $user->id)
					->first([
						'users.id as userId',
						'user_level.level as userLevel',
						'users.name as userName',
						'users.email as userEmail',
					]);

				if (is_null($userRegister)) {
					throw new Exception('Failed to find user register and level.');
				}

				$jsonResponse = [
					'status' => 'success',
					'action' => 'renewed',
					'message' => 'Renewed token',
					'expires' => urlencode($expires),
					'sessionId' => urlencode($sessionId),
					'userId' => $userRegister->userId, // @phpstan-ignore-line
					'userLevel' => $userRegister->userLevel, // @phpstan-ignore-line
				];
				if ($fromMiddleware) {
					$jsonResponse['user'] = $userRegister;
				}
			} else {
				$statusCode = 200;
				$session = ssoSessions::where('session_identifier', $sessionIdentifier)->latest('id')->first();
				$user = User::join('user_level', 'users.user_level_id', '=', 'user_level.id')
					->where('users.id', $session->user_id)
					->first([
						'users.id as userId',
						'user_level.level as userLevel',
						'users.name as userName',
						'users.email as userEmail',
					]);

				$jsonResponse = [
					'status' => 'success',
					'action' => 'valid',
					'message' => 'Token valid',
					'sessionId' => urlencode(Crypt::encryptString($sessionIdentifier)),
					'userId' => $user->userId, // @phpstan-ignore-line
					'userLevel' => $user->userLevel, // @phpstan-ignore-line
				];

				if ($fromMiddleware) {
					$jsonResponse['user'] = $user;
				}
			}
		} catch (Exception $e) {
			$statusCode = 500;

			$jsonResponse = [
				'status' => 'error',
				'message' => $e->getMessage(),
				'line' => $e->getLine(),
			];
		} finally {
			return response()->json($jsonResponse, $statusCode);
		}
	}

	/**
	 * Store the new SSO session
	 *
	 * @param User $user
	 * @param stdClass $ssoData
	 *
	 * @return ssoSessions
	 */
	public function storeSession(User $user, stdClass $ssoData): ?ssoSessions
	{
		$session = null;
		try {
			$session = new ssoSessions();

			$session->session_identifier = Str::orderedUuid();
			$session->refresh_token = $ssoData->refresh_token;
			$session->user_id = $user->id;
			$session->expires = $ssoData->exp;

			$session->save();
		} catch (Exception $e) {
			// TODO: Handle with potential erros
			$session = null;
		} finally {
			return $session;
		}
	}

	/**
	 * For a given bearer token (header) validate if is valid
	 *
	 * @return JsonResponse
	 */
	public function validateToken(): JsonResponse
	{
		$return = [];
		$statusCode = 0;

		try {
			$valid = Auth::guard('api')->check();
			if ($valid) {
				$return = [
					'status' => 'success',
					'message' => 'Token valid',
				];

				$statusCode = 200;
			} else {
				$return = [
					'status' => 'error',
					'message' => 'invalid token',
				];

				$statusCode = 401;
			}
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => $e->getMessage(),
			];

			$statusCode = 500;
		}

		return response()->json($return, $statusCode);
	}
}
