<?php

namespace App\Http\Controllers;

use App\Models\Brand;
use Exception;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Str;

/**
 * Brand Controller
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Http\Controllers
 */

class BrandController extends Controller
{
    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     *
     * @return JsonResponse
     */
    public function destroy(int $id): JsonResponse
    {
        try {
            Brand::destroy($id);
            $return = [
                'status' => 'success',
                'message' => __('Deleted successfully!'),
            ];
            $statusCode = 201;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to delete!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Generate the slug for a given name
     *
     * @param string $name
     *
     * @return string
     */
    public function generateSlug(string $name): string
    {
        // Slugify
        $slug = Str::slug($name, '-');

        // Retrieve the last slug to the slug
        $exist = app(BrandController::class)->getLastBrandBySlug($slug);

        // If exist already exist a slug, get it
        if (!(is_null($exist))) {
            $exist = $exist->slug;
        }

        //Retrieve the last position for this separator
        $counter = last(explode('-', $exist));

        /**
         * If the last array position is a number, means that there are another entry to this name
         * In another hand, if is empty meands that is the first one.
         */
        if (is_numeric($counter)) {
            $counter += 1;
        } elseif (!(empty($counter))) {
            $counter = 1;
        }

        // If there are another one, increase the counter - to ensure that slug is unique
        if ($counter) {
            $slug = $slug . '-' . $counter;
        }

        return $slug;
    }

    /**
     * Return last Brand to a given slug
     *
     * @param string $slug
     *
     * @return Brand|null
     */
    public function getLastBrandBySlug(string $slug): Brand|null
    {
        return Brand::withTrashed()->where('slug', $slug)->orWhere('slug', '~', '^' . $slug . '-[0-9]+')->orderBy('id', 'DESC')->first();
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index(): JsonResponse
    {
        try {
            $brands = Brand::orderBy('name')->get();

            $return = [
                'status' => 'success',
                'data' => $brands,
            ];
            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Display a paginated listing of the resource.
     *
     * @param ?Request $request
     *
     * @return JsonResponse
     */
    public function index_paginated(Request $request = null): JsonResponse
    {
        try {
            if ((!is_null($request)) && (!empty($request->filter)) && strlen($request->filter) > 0) {
                $brands = Brand::where('name', 'ilike', '%' . $request->filter . '%')->orderByRaw('UPPER(name) ASC')->paginate($request->per_page);
            } else {
                $perPage = (!is_null($request)) ? $request->per_page : 20;
                $brands = Brand::orderByRaw('UPPER(name) ASC')->paginate($perPage);
            }

            $return = [
                'status' => 'success',
                'data' => $brands,
            ];
            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Show the brand for a given id.
     *
     * @param int $id
     *
     * @return JsonResponse
     */
    public function show(int $id): JsonResponse
    {
        try {
            $brand = Brand::find($id);

            $return = [
                'status' => 'success',
                'data' => $brand,
            ];

            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param string $params
     *
     * @return JsonResponse
     */
    public function store(string $params): JsonResponse
    {
        $params = json_decode($params, true);

        try {
            $brand = new Brand();
            $brand->name = $params['name'];
            $brand->slug = $this->generateSlug($params['name']);

            if (isset($params['image'])) {
                $brand->image = $params['image'];
            }

            $brand->save();

            $return = [
                'status' => 'success',
                'message' => __('Created successfully'),
            ];
            $statusCode = 201;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to Create!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param int $id
     * @param string $params
     *
     * @return JsonResponse
     */
    public function update(int $id, string $params): JsonResponse
    {
        $params = json_decode($params, true);

        try {
            $brand = Brand::find($id);

            if (!empty($params['name'])) {
                $brand->name = $params['name'];
            }

            if (!empty($params['reslug'])) {
                $brand->slug = $this->generateSlug($params['name']);
            }

            if (!empty($params['image'])) {
                $brand->image = $params['image'];
            }

            try {
                $brand->save();

                $return = [
                    'status' => 'success',
                    'message' => __('Updated successfully'),
                ];
                $statusCode = 200;
            } catch (Exception $e) {
                throw new Exception($e);
            }
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to Update!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }
}
