<?php

// phpcs:disable PSR2.ControlStructures.ElseIfDeclaration.NotAllowed

namespace App\Http\Controllers;

use App\Exports\ReportExport;
use App\Http\Controllers\Helpers\HelpersController;
use App\Models\Brand;
use App\Models\Channel;
use App\Models\Country;
use App\Models\LighthouseData;
use App\Models\LinkCriteria;
use App\Models\Ownership;
use App\Models\ReportDGP;
use App\Models\Zone;
use DateTime;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Maatwebsite\Excel\Facades\Excel;
use Symfony\Component\HttpFoundation\BinaryFileResponse;

/**
 * CommonController Controller
 *
 * @author Tércio Lacerda <Tercio.lacerda-ext@ab-inbev.com>
 * @package App\Http\Controllers
 */
class CommonController extends Controller
{
	/**
	 * Map of request parameters to respective table.column in db
	 */
	protected $bdMapColumns = [
		'text' => 'link',
		'brands' => 'brand',
		'countries' => 'country',
		'zones' => 'zone',
		'department' => 'department',
		'lighthouse' => 'lighthouse_last',
		'channels' => 'channel',
		'age_gate' => 'age_gate',
		'coi' => 'coi',
		'fan' => 'fan',
		'rdm' => 'rdm',
		'ugc' => 'ugc',
		'semrush' => 'semrush',
		'lighthouse_avg' => 'lighthouse_avg',
		'risk_rate' => 'risk_rate',
		'vulnerability_level' => 'vulnerability_level',
		'pia_completion' => 'pia_completion',
		'cookie_banner_implemented' => 'cookie_banner_implemented',
		'cookie_dropping_compliance' => 'cookie_dropping_compliance',
		'ga_implementation' => 'ga_implementation',
		'ga_access' => 'ga_access',
		'ga_taxonomy' => 'ga_taxonomy',
		'cdp_implementation' => 'cdp_implementation',
		'website_data' => 'website_data',
		'property_excellence' => 'property_excellence',
		'privacy_security' => 'privacy_security',
		'iard' => 'iard',
		'date' => 'executed_at',
		'criteria' => 'map',
	];

	/**
	 * Returning infos to be used on Filter Components
	 *
	 * @return JsonResponse
	 */
	public function filtersData(Request $request): JsonResponse
	{
		$pagename = 'default';

		if (isset($request->page) && !empty($request->page)) {
			$pagename = $request->page;
		}

		try {
			$brands = Brand::join('links', 'links.brand_id', '=', 'brands.id')
				->join('countries', 'countries.id', '=', 'links.country_id')
				->orderByRaw('brands.name ASC')
				->distinct(['brands.name'])
				->get([
					'brands.id as id',
					'brands.name as name',
					'brands.slug as slug',
					'links.country_id as country_id',
					'countries.zone_id as zone_id',
				]);
			$channels = Channel::orderByRaw('UPPER(name) ASC')->get(['name', 'slug']);
			$countries = Country::orderByRaw('UPPER(name) ASC')->get(['name', 'code', 'zone_id', 'id']);
			$zones = Zone::orderByRaw('UPPER(name) ASC')->get(['name', 'id']);
			$departments = ReportDGP::distinct()->orderBy('department', 'asc')->get(['department']);

			if ($pagename === 'pageSpeed') {
				$officialDates = LighthouseData::groupBy('executed_at')->orderBy('executed_at', 'desc')->get(['executed_at'])->toArray();
			} else {
				$officialDates = LinkCriteria::where('official', true)->groupBy('executed_at')->orderBy('executed_at', 'desc')->get(['executed_at'])->toArray();
			}

			$return = [
				'status' => 'success',
				'data' => array(
					'brands' => $brands,
					'channels' => $channels,
					'countries' => $countries,
					'zones' => $zones,
					'departments' => $departments,
					'officialDates' => $officialDates
				),
			];
			$statusCode = 200;
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'detail' => $e->getMessage(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Right name to Excel index
	 *
	 * @param string $indexName
	 *
	 * @return String
	 */
	public function fixIndexToExcel(string $indexName): String
	{
		$returnVal = '';
		switch ($indexName) {
			case 'executed_at':
				$returnVal = 'Executed at';
				break;
			case 'age_gate':
				$returnVal = 'Age Gate';
				break;
			case 'risk_rate':
				$returnVal = 'Risk Rate';
				break;
			case 'vulnerability_level':
				$returnVal = 'Vulnerability Level';
				break;
			case 'pia_completion':
				$returnVal = 'PIA Completion';
				break;
			case 'cdp_implementation':
				$returnVal = 'CDP Implementation';
				break;
			case 'dmp_implementation':
				$returnVal = 'DMP Implementation';
				break;
			case 'lighthouse_avg':
				$returnVal = 'Lighthouse Average';
				break;
			case 'ga_implementation':
				$returnVal = 'GA Implementation';
				break;
			case 'ga_access':
				$returnVal = 'GA Access';
				break;
			case 'ga_taxonomy':
				$returnVal = 'GA Taxonomy';
				break;
			case 'lighthouse_last':
				$returnVal = 'Lighthouse';
				break;
			case 'first_contentful_paint':
				$returnVal = 'WebVitals - First Contentful Paint';
				break;
			case 'largest_contentful_paint':
				$returnVal = 'WebVitals - Largest Contentful Paint';
				break;
			case 'total_blocking_time':
				$returnVal = 'WebVitals - Total Blocking Time';
				break;
			case 'speed_index':
				$returnVal = 'WebVitals - Speed Index';
				break;
			case 'time_to_interactive':
				$returnVal = 'WebVitals - Time To Interactive';
				break;
			case 'cumulative_layout_shift':
				$returnVal = 'WebVitals - Cumulative Layout Shift';
				break;
			case 'privacy_security':
				$returnVal = 'Privacy and Security';
				break;
			case 'property_excellence':
				$returnVal = 'Owned Property Excellence';
				break;
			case 'website_data':
				$returnVal = 'Website Data';
				break;
			case 'coi':
			case 'fan':
			case 'rdm':
			case 'ugc':
			case 'iard':
				$returnVal = strtoupper($indexName);
				break;
			default:
				$returnVal = ucfirst($indexName);
				break;
		}
		return $returnVal;
	}

	/**
	 * Get history data to mount Line Chart on frontend
	 *
	 * @param array $getExecDatesParam
	 * @param string $whereRawParam, default = ''
	 *
	 * @return Array
	 */
	public function getLastExecsDataNormalized(array $getExecDatesParam, String $whereRawParam = ''): array
	{
		$selectLastExecutionsData = '';

		// Only last 12 executions
		foreach ($getExecDatesParam as $key => $value) {
			if ($key > 11) break;

			if ($key > 0) $selectLastExecutionsData = $selectLastExecutionsData . ',';

			$selectLastExecutionsData = $selectLastExecutionsData . "
			(select executed_at from reportdgp where executed_at = '" . $value['executed_at'] . "' limit 1) as executed_at_" . $key . ",
			count(executed_at) filter (where executed_at = '" . $value['executed_at'] . "') as count_" . $key . ",
			count(executed_at) filter (where executed_at = '" . $value['executed_at'] . "' and channel = 'Site') as count_site_" . $key . ",
			count(executed_at) filter (where executed_at = '" . $value['executed_at'] . "' and channel = 'Site' and country != 'Ukraine' and country != 'Russia') as count_site_without_countries_" . $key . ",
			count(executed_at) filter (where executed_at = '" . $value['executed_at'] . "' and channel != 'Site') as count_social_networks_" . $key . ",
			count(executed_at) filter (where iard = true and executed_at = '" . $value['executed_at'] . "') as iard_count_" . $key . ",
			sum(privacy_security) filter (where executed_at = '" . $value['executed_at'] . "') as privacy_security_count_" . $key . ",
			sum(property_excellence) filter (where executed_at = '" . $value['executed_at'] . "') as property_excellence_count_" . $key . ",
			sum(website_data) filter (where executed_at = '" . $value['executed_at'] . "') as website_data_count_" . $key;
		}

		$GLOBALS['whereRawParam'] = $whereRawParam;

		$lastExecsData = ReportDGP::where('official', true)
			->when((strlen($GLOBALS['whereRawParam']) > 0), function ($query) {
				$query->whereRaw($GLOBALS['whereRawParam']);
			})
			->selectRaw($selectLastExecutionsData)
			->get()->toArray();

		// 9 number of columns returneds by query
		$executionsCount = count($lastExecsData[0]) / 9;

		$separatedDataExecs = array();
		for ($i = 0; $i < $executionsCount; $i++) {
			foreach ($lastExecsData[0] as $key => $value) {
				if (substr($key, -strlen("_$i")) === "_$i") {
					$separatedDataExecs[$i][str_replace('_' . $i, '', $key)] = $value;
				}
			}
		}

		$lastExecsDataNormalized = array();

		foreach ($separatedDataExecs as $key => $value) {
			foreach ($value as $subKey => $subValue) {
				// First index - normalize array and break
				if ($subKey === 'executed_at') {
					$totalDivisor = $value['count'] > 0 ? $value['count'] : 1;

					$totalDivisorSite = $value['count_site'] > 0 ? $value['count_site'] : 1;

					$totalDivisorSiteWithoutCountries = $value['count_site_without_countries'] > 0 ? $value['count_site_without_countries'] : 1;

					$lastExecsDataNormalized[$key]['executed_at'] = $value['executed_at'];

					$geralScore = 0;
					$modulesEnabled = 0;

					if (env('IARD_MODULE')) {
						$lastExecsDataNormalized[$key]['iard'] = number_format(((int)$value['iard_count'] / (int)$totalDivisor) * 100, 1);

						$geralScore += ((int) $value['iard_count'] / (int) $totalDivisor) * 100;
						$modulesEnabled++;
					}

					if (env('PRIVACY_MODULE')) {
						$lastExecsDataNormalized[$key]['privacy_security'] = number_format(((int)$value['privacy_security_count'] / (int)$totalDivisorSite), 1);

						$geralScore += (int) $value['privacy_security_count'] / (int) $totalDivisorSite;
						$modulesEnabled++;
					}

					if (env('EXCELLENCE_MODULE')) {
						$lastExecsDataNormalized[$key]['property_excellence'] = number_format(((int)$value['property_excellence_count'] / (int)$totalDivisorSiteWithoutCountries), 1);

						$geralScore += (int)$value['property_excellence_count'] / (int)$totalDivisorSiteWithoutCountries;
						$modulesEnabled++;
					}

					if (env('DATA_MODULE')) {
						$lastExecsDataNormalized[$key]['website_data'] = number_format(((int)$value['website_data_count'] / (int)$totalDivisorSiteWithoutCountries), 1);

						$geralScore += (int)$value['website_data_count'] / (int)$totalDivisorSiteWithoutCountries;
						$modulesEnabled++;
					}

					$lastExecsDataNormalized[$key]['dgp_score'] = number_format((float) ($geralScore) / (float) (($modulesEnabled === 0) ? 1 : $modulesEnabled), 1);
				}

				break;
			}
		}

		return $lastExecsDataNormalized;
	}

	/**
	 * Display a ownership details from a specific asset
	 *
	 * @param int $id
	 *
	 * @return JsonResponse
	 */
	public function getOwnershipData(int $id): JsonResponse
	{
		try {
			$asset = Ownership::query()->where('link_id', $id)->first();

			$return = [
				'status' => 'success',
				'asset' => $asset,
			];
			$statusCode = 200;
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'details' => $e->getMessage(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Display resources for DGP homepage
	 *
	 * @param Request $request
	 *
	 * @return JsonResponse
	 *
	 */
	public function homePage(Request $request): JsonResponse
	{
		$date = $request->date;

		$GLOBALS['whereRaw'] = HelpersController::mountWhereRaw($request);

		try {
			$getExecDates = ReportDGP::where('official', true)->groupBy('executed_at')->orderBy('executed_at', 'desc')->take(12)->get(['executed_at'])->toArray();

			$selectLastExecStr = "
			zone,
			count(executed_at) as count,
			count(executed_at) filter (where channel != 'Site') as count_social_media,   
			count(executed_at) filter (where channel = 'Site') as count_site,
			count(executed_at) filter (where channel = 'Site' and country != 'Ukraine' and country != 'Russia') as count_site_without_countries";

			if (env('IARD_MODULE')) {
				$selectLastExecStr .= ",count(executed_at) filter (where iard = true) as iard_count,
			count(executed_at) filter (where iard = true and channel != 'Site') as iard_social_media_count,
			count(executed_at) filter (where iard = true and channel = 'Site') as iard_site_count";
			}

			if (env('PRIVACY_MODULE')) {
				$selectLastExecStr .= ',sum(privacy_security) as privacy_security_count';
			}

			if (env('EXCELLENCE_MODULE')) {
				$selectLastExecStr .= ',sum(property_excellence) as property_excellence_count';
			}

			if (env('DATA_MODULE')) {
				$selectLastExecStr .= ',sum(website_data) as website_data_count';
			}

			$lastExecutionDataByZone = ReportDGP::where('official', true)
				->whereDate('executed_at', $date ? $date : $getExecDates[0]['executed_at'])
				->when((strlen($GLOBALS['whereRaw']) > 0), function ($query) {
					$query->whereRaw($GLOBALS['whereRaw']);
				})
				->selectRaw($selectLastExecStr)
				->groupBy('zone')
				->get()->toArray();

			$globalValues = array(
				'total' => 0,
				'total_site' => 0,
				'total_site_without_countries' => 0,
				'total_social_media' => 0,
				'iard' => 0,
				'privacy_security' => 0,
				'property_excellence' => 0,
				'website_data' => 0,
				'iard_count' => 0,
				'privacy_security_count' => 0,
				'property_excellence_count' => 0,
				'website_data_count' => 0,
			);

			$zonesLength = count($lastExecutionDataByZone);

			$lastExecDataByZoneNormalized = array();

			foreach ($lastExecutionDataByZone as $keyQuery => $valueQuery) {
				$geralScore = 0;
				$modulesEnabled = 0;

				foreach ($valueQuery as $key => $value) {
					// First index - normalize array and break
					if ($key === 'zone') {
						$lastExecDataByZoneNormalized[$keyQuery]['zone'] = $valueQuery['zone'];
						$globalValues['total'] += $valueQuery['count'];
						$globalValues['total_site'] += $valueQuery['count_site'];
						$globalValues['total_site_without_countries'] += $valueQuery['count_site_without_countries'];
						$globalValues['total_social_media'] += $valueQuery['count_social_media'];

						$divisorG = (int)$valueQuery['count'] > 0 ? (int)$valueQuery['count'] : 1;
						$divisorSm = (int)$valueQuery['count_social_media'] > 0 ? (int)$valueQuery['count_social_media'] : 1;
						$divisorSite = (int)$valueQuery['count_site'] > 0 ? (int)$valueQuery['count_site'] : 1;
						$divisorSiteWithoutCountries = (int)$valueQuery['count_site_without_countries'] > 0 ? (int)$valueQuery['count_site_without_countries'] : 1;

						if (env('IARD_MODULE')) {
							$lastExecDataByZoneNormalized[$keyQuery]['iard'] = number_format(((int)$valueQuery['iard_count'] / $divisorG) * 100, 1);

							$globalValues['iard'] += ((int)$valueQuery['iard_count'] / $divisorG) * 100;

							$geralScore += $globalValues['iard'];
							$modulesEnabled++;

							$lastExecDataByZoneNormalized[$keyQuery]['iard_social_media'] = number_format(((int)$valueQuery['iard_social_media_count'] / $divisorSm) * 100, 1);

							$lastExecDataByZoneNormalized[$keyQuery]['iard_site'] = number_format(((int)$valueQuery['iard_site_count'] / $divisorSite) * 100, 1);
						} else {
							$lastExecDataByZoneNormalized[$keyQuery]['iard'] = 0;
							$lastExecDataByZoneNormalized[$keyQuery]['iard_social_media'] = 0;
							$lastExecDataByZoneNormalized[$keyQuery]['iard_site'] = 0;
						}

						if (env('PRIVACY_MODULE')) {
							$lastExecDataByZoneNormalized[$keyQuery]['privacy_security'] = number_format(((int)$valueQuery['privacy_security_count'] / $divisorSite), 1);

							$geralScore += (int)$valueQuery['privacy_security_count'] / $divisorSite;
							$modulesEnabled++;

							$globalValues['privacy_security'] += (int)$valueQuery['privacy_security_count'] / $divisorSite;
						} else {
							$lastExecDataByZoneNormalized[$keyQuery]['privacy_security'] = 0;
						}

						if (env('EXCELLENCE_MODULE')) {
							$lastExecDataByZoneNormalized[$keyQuery]['property_excellence'] = number_format(((int)$valueQuery['property_excellence_count'] / $divisorSiteWithoutCountries), 1);

							$geralScore += (int)$valueQuery['property_excellence_count'] / $divisorSiteWithoutCountries;
							$modulesEnabled++;

							$globalValues['property_excellence'] += (float)(int)$valueQuery['property_excellence_count'] / $divisorSiteWithoutCountries;
						} else {
							$lastExecDataByZoneNormalized[$keyQuery]['property_excellence'] = 0;
						}

						if (env('DATA_MODULE')) {
							$lastExecDataByZoneNormalized[$keyQuery]['website_data'] = number_format(((int)$valueQuery['website_data_count'] / $divisorSiteWithoutCountries), 1);

							$globalValues['website_data'] += $valueQuery['website_data_count'] / $divisorSiteWithoutCountries;
							$modulesEnabled++;
						} else {
							$lastExecDataByZoneNormalized[$keyQuery]['website_data'] = 0;
						}

						$lastExecDataByZoneNormalized[$keyQuery]['dgp_score'] = number_format(((float) $lastExecDataByZoneNormalized[$keyQuery]['iard'] + (float) $lastExecDataByZoneNormalized[$keyQuery]['privacy_security'] + (float) $lastExecDataByZoneNormalized[$keyQuery]['property_excellence'] + (float) $lastExecDataByZoneNormalized[$keyQuery]['website_data']) / ($modulesEnabled === 0 ? 1 : $modulesEnabled), 1);

						$lastExecDataByZoneNormalized[$keyQuery]['dgp_score_website'] = number_format(((float) $lastExecDataByZoneNormalized[$keyQuery]['iard_site'] + (float) $lastExecDataByZoneNormalized[$keyQuery]['privacy_security'] + (float) $lastExecDataByZoneNormalized[$keyQuery]['property_excellence'] + (float) $lastExecDataByZoneNormalized[$keyQuery]['website_data']) / ($modulesEnabled === 0 ? 1 : $modulesEnabled), 1);

						if (env('IARD_MODULE')) {
							$lastExecDataByZoneNormalized[$keyQuery]['dgp_score_social_media'] = number_format(((float) $lastExecDataByZoneNormalized[$keyQuery]['iard_social_media']), 1);

							$globalValues['iard_count'] += $valueQuery['iard_count'];
						}

						if (env('PRIVACY_MODULE')) {
							$globalValues['privacy_security_count'] += $valueQuery['privacy_security_count'];
						}

						if (env('EXCELLENCE_MODULE')) {
							$globalValues['property_excellence_count'] += $valueQuery['property_excellence_count'];
						}

						if (env('DATA_MODULE')) {
							$globalValues['website_data_count'] += $valueQuery['website_data_count'];
						}
					}
					break;
				}
				// Last zone array - generate global DGP score
				if (($keyQuery + 1) === $zonesLength) {
					$globalValues['iard'] = number_format(($globalValues['iard_count'] / $globalValues['total']) * 100, 1);

					$globalValues['privacy_security'] = number_format(($globalValues['privacy_security_count'] / $globalValues['total_site']), 1);

					$globalValues['property_excellence'] = number_format(($globalValues['property_excellence_count'] / $globalValues['total_site_without_countries']), 1);

					$globalValues['website_data'] = number_format(($globalValues['website_data_count'] / $globalValues['total_site_without_countries']), 1);

					$globalValues['dgp_score'] = number_format(((float)$globalValues['iard'] + (float)$globalValues['privacy_security'] + (float)$globalValues['property_excellence'] + (float)$globalValues['website_data']) / ($modulesEnabled === 0 ? 1 : $modulesEnabled), 1);
				}
			}

			$lastExecsDataNormalized = $this->getLastExecsDataNormalized($getExecDates, $GLOBALS['whereRaw']);

			$returnArr = array(
				'globalValues' => $globalValues,
				'lastExecutionDataByZone' => $lastExecDataByZoneNormalized,
				'lastExecsData' => $lastExecsDataNormalized,
			);

			$return = [
				'status' => 'success',
				'data' => $returnArr,
				'lastRun' => $date ? $date : $getExecDates[0]['executed_at'],
			];
			$statusCode = 200;
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'details' => $e->getMessage(),
				'line' => $e->getLine(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Display a paginated listing of a criteria
	 *
	 * @param string $type
	 * @param Request $request
	 *
	 * @return JsonResponse
	 */
	public function reportsData(string $type, Request $request): JsonResponse
	{
		$date = $request->date;

		$GLOBALS['whereRaw'] = HelpersController::mountWhereRaw($request);

		try {
			$getExecDates = ReportDGP::where('official', true)->groupBy('executed_at')->orderBy('executed_at', 'desc')->take(12)->get(['executed_at'])->toArray();

			$selectFilterCountApproveds = "
			count(executed_at) as total,
			count(executed_at) filter (where channel = 'Site') as total_site,
			count(executed_at) filter (where channel = 'Site' and country != 'Ukraine' and country != 'Russia') as total_site_without_countries,
			count(executed_at) filter (where channel != 'Site') as total_social_media,
			count(executed_at) filter (where iard = true) as iard_count,
			count(executed_at) filter (where age_gate = 'true') as agegate_count,
			count(executed_at) filter (where coi = 'true') as coi_count,
			count(executed_at) filter (where fan = 'true') as fan_count,
			count(executed_at) filter (where rdm = 'true') as rdm_count,
			count(executed_at) filter (where ugc = 'true') as ugc_count,
			count(executed_at) filter (where cdp_implementation = true) as cdp_count,
			count(executed_at) filter (where ga_implementation = true) as gai_count,
			count(executed_at) filter (where ga_access = true) as gaa_count,
			count(executed_at) filter (where ga_taxonomy = true) as gat_count,
			count(executed_at) filter (where ga_taxonomy = true) as gat_count,
			sum(privacy_security) as privacy_security_count,
			sum(property_excellence) as property_excellence_count,
			sum(website_data) as website_data_count";

			$filterCountApproveds = ReportDGP::where('official', true)
				->whereDate('executed_at', $date ? $date : $getExecDates[0]['executed_at'])
				->when((strlen($GLOBALS['whereRaw']) > 0), function ($query) {
					$query->whereRaw($GLOBALS['whereRaw']);
				})
				->selectRaw($selectFilterCountApproveds)
				->get()->toArray();


			$totalDivisor = $filterCountApproveds[0]['total'] > 0 ? $filterCountApproveds[0]['total'] : 1;

			$totalDivisorSite = $filterCountApproveds[0]['total_site'] > 0 ? $filterCountApproveds[0]['total_site'] : 1;

			$totalDivisorSocial = $filterCountApproveds[0]['total_social_media'] > 0 ? $filterCountApproveds[0]['total_social_media'] : 1;

			$totalDivisorSiteWithoutCountries = $filterCountApproveds[0]['total_site_without_countries'] > 0 ? $filterCountApproveds[0]['total_site_without_countries'] : 1;

			$globalValues = array(
				'total' => $filterCountApproveds[0]['total'],
				'total_site' => $filterCountApproveds[0]['total_site'],
				'total_site_without_countries' => $filterCountApproveds[0]['total_site_without_countries'],
				'total_social_media' => $filterCountApproveds[0]['total_social_media'],
				'iard_count' => $filterCountApproveds[0]['iard_count'],
				'privacy_security_count' => $filterCountApproveds[0]['privacy_security_count'],
				'property_excellence_count' => $filterCountApproveds[0]['property_excellence_count'],
				'website_data_count' => $filterCountApproveds[0]['website_data_count'],
			);

			$globalValues['date'] = $date ? $date : $getExecDates[0]['executed_at'];

			$globalValues['where'] = (strlen(json_encode($GLOBALS['whereRaw'])) > 6 ? 1 : 0);

			$geralScore = 0;
			$modulesEnabled = 0;

			if (env('IARD_MODULE')) {
				$globalValues['iard'] = number_format(($filterCountApproveds[0]['iard_count'] / $totalDivisor) * 100, 1);

				$globalValues['age_gate'] = number_format(($filterCountApproveds[0]['agegate_count'] / $totalDivisor) * 100, 1);
				$globalValues['coi'] = number_format(($filterCountApproveds[0]['coi_count'] / $totalDivisor) * 100, 1);
				$globalValues['fan'] = number_format(($filterCountApproveds[0]['fan_count'] / $totalDivisorSocial) * 100, 1);
				$globalValues['rdm'] = number_format(($filterCountApproveds[0]['rdm_count'] / $totalDivisor) * 100, 1);
				$globalValues['ugc'] = number_format(($filterCountApproveds[0]['ugc_count'] / $totalDivisorSocial) * 100, 1);

				$geralScore += ($filterCountApproveds[0]['iard_count'] / $totalDivisor) * 100;

				$modulesEnabled++;
			} else {
				$globalValues['iard'] = 0;
				$globalValues['age_gate'] = 0;
				$globalValues['coi'] = 0;
				$globalValues['fan'] = 0;
				$globalValues['rdm'] = 0;
				$globalValues['ugc'] = 0;
			}

			if (env('PRIVACY_MODULE')) {
				$globalValues['privacy_security'] = number_format(($filterCountApproveds[0]['privacy_security_count'] / $totalDivisorSite), 1);

				$geralScore += ($filterCountApproveds[0]['privacy_security_count'] / $totalDivisorSite);

				$modulesEnabled++;
			} else {
				$globalValues['privacy_security'] = 0;
			}

			if (env('EXCELLENCE_MODULE')) {
				$globalValues['property_excellence'] = number_format(($filterCountApproveds[0]['property_excellence_count'] / $totalDivisorSiteWithoutCountries), 1);

				$geralScore += ($filterCountApproveds[0]['property_excellence_count'] / $totalDivisorSiteWithoutCountries);

				$modulesEnabled++;
			} else {
				$globalValues['property_excellence'] = 0;
			}

			if (env('DATA_MODULE')) {
				$globalValues['website_data'] = number_format(($filterCountApproveds[0]['website_data_count'] / $totalDivisorSiteWithoutCountries), 1);

				$globalValues['cdp'] = number_format(($filterCountApproveds[0]['cdp_count'] / $totalDivisorSite) * 100, 1);
				$globalValues['gai'] = number_format(($filterCountApproveds[0]['gai_count'] / $totalDivisorSite) * 100, 1);
				$globalValues['gaa'] = number_format(($filterCountApproveds[0]['gaa_count'] / $totalDivisorSite) * 100, 1);
				$globalValues['gat'] = number_format(($filterCountApproveds[0]['gat_count'] / $totalDivisorSite) * 100, 1);

				$geralScore += ($filterCountApproveds[0]['website_data_count'] / $totalDivisorSiteWithoutCountries);

				$modulesEnabled++;
			} else {
				$globalValues['website_data'] = 0;

				$globalValues['cdp'] = 0;
				$globalValues['gai'] = 0;
				$globalValues['gaa'] = 0;
				$globalValues['gat'] = 0;
			}

			$globalValues['dgp_score'] = number_format((float) ($geralScore) / (($modulesEnabled === 0) ? 1 : $modulesEnabled), 1);

			$returnFields = array(
				'link',
				'zone',
				'country',
				'brand',
				'executed_at'
			);

			if ($type !== 'page-speed') {
				$returnFields = array_merge($returnFields, array('channel'));
			}

			if ($type === 'page-speed') {
				$returnFields = array_merge($returnFields, array('lighthouse_last', 'first_contentful_paint', 'largest_contentful_paint', 'total_blocking_time', 'speed_index', 'time_to_interactive', 'cumulative_layout_shift'));
			} else if ($type === 'iard') {
				$returnFields = array_merge($returnFields, array('age_gate', 'coi', 'fan', 'rdm', 'ugc'));
			} else if ($type === 'privacy-and-security') {
				$returnFields = array_merge($returnFields, array('risk_rate', 'vulnerability_level', 'pia_completion', 'cookie_banner_implemented', 'cookie_dropping_compliance'));
			} else if ($type === 'excelence') {
				$returnFields = array_merge($returnFields, array('semrush', 'lighthouse_avg'));
			} else if ($type === 'website-data') {
				$returnFields = array_merge($returnFields, array('ga_implementation', 'ga_access', 'ga_taxonomy', 'cdp_implementation'));
			} else if ($type === 'google-analytics') {
				$returnFields = array_merge($returnFields, array('ga_implementation', 'ga_access', 'ga_taxonomy'));
			}

			if (empty($request->date)) {
				$lastRun = '';

				if ($type === 'page-speed') {
					$lastRun = LighthouseData::orderBy('executed_at', 'desc')->first(['executed_at'])->toArray();
				} else {
					$lastRun = ReportDGP::orderBy('executed_at', 'desc')->first(['executed_at'])->toArray();
				}

				$request->merge(['date' => $lastRun['executed_at']]);
			}

			$query = null;

			if ($type === 'page-speed') {
				$query = LighthouseData::query();
			} else {
				$query = ReportDGP::query();
			}

			HelpersController::buildRequestWhere($query, $request->all(), $this->bdMapColumns, $type);

			$query = $query->orderByRaw('UPPER(zone) ASC, UPPER(country) ASC, UPPER(link) ASC')
				->paginate($request->per_page, $returnFields);

			$return = [
				'status' => 'success',
				'data' => $query,
				'globalValues' => $globalValues,
				'lastRun' => $request->date,
			];
			$statusCode = 200;
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'details' => $e->getMessage(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Export data from a criteria
	 *
	 * @param string $type
	 * @param Request $request
	 *
	 * @return BinaryFileResponse|JsonResponse
	 */
	public function reportsDataExport(string $type, Request $request): BinaryFileResponse|JsonResponse
	{
		try {
			$returnFields = array(
				'link',
				'zone',
				'country',
				'brand',
			);

			if ($type !== 'page-speed') {
				$returnFields = array_merge($returnFields, array('channel'));
			}

			if ($type === 'page-speed') {
				$returnFields = array_merge($returnFields, array('lighthouse_last', 'first_contentful_paint', 'largest_contentful_paint', 'total_blocking_time', 'speed_index', 'time_to_interactive', 'cumulative_layout_shift'));
			} else if ($type === 'iard') {
				$returnFields = array_merge($returnFields, array('age_gate', 'coi', 'fan', 'rdm', 'ugc'));
			} else if ($type === 'privacy-and-security') {
				$returnFields = array_merge($returnFields, array('risk_rate', 'vulnerability_level', 'pia_completion'));
			} else if ($type === 'excelence') {
				$returnFields = array_merge($returnFields, array('semrush', 'lighthouse_avg'));
			} else if ($type === 'website-data') {
				$returnFields = array_merge($returnFields, array('ga_implementation', 'ga_access', 'ga_taxonomy', 'cdp_implementation'));
			} else if ($type === 'google-analytics') {
				$returnFields = array_merge($returnFields, array('ga_implementation', 'ga_access', 'ga_taxonomy'));
			}

			array_push($returnFields, 'executed_at');

			if (empty($request->date)) {
				$lastRun = '';

				if ($type === 'page-speed') {
					$lastRun = LighthouseData::orderBy('executed_at', 'desc')->first(['executed_at'])->toArray();
				} else {
					$lastRun = ReportDGP::orderBy('executed_at', 'desc')->first(['executed_at'])->toArray();
				}

				$request->merge(['date' => $lastRun['executed_at']]);
			}

			$query = null;

			if ($type === 'page-speed') {
				$query = LighthouseData::query();
			} else {
				$query = ReportDGP::query();
			}

			HelpersController::buildRequestWhere($query, $request->all(), $this->bdMapColumns, $type);
			$query = $query->orderByRaw('UPPER(zone) ASC, UPPER(country) ASC, UPPER(link) ASC')
				->get($returnFields)
				->toArray();

			$headers = array();
			foreach ($returnFields as $key => $value) {
				array_push($headers, $this->fixIndexToExcel($value));
			}

			foreach ($query as $key => $value) {
				foreach ($query[$key] as $innerKey => $innerValue) {
					if ($innerValue === 'true' || $innerValue === true) {
						$query[$key][$innerKey] = 'Approved';
					} else if ($innerValue === 'false' || $innerValue === false) {
						$query[$key][$innerKey] = 'Not approved';
					} else if (strlen(var_export($innerValue, true)) === 0 || is_null($innerValue)) {
						$query[$key][$innerKey] = '-';
					}
				}
			}

			array_unshift($query, $headers);

			$report = new ReportExport($query);
			$date = new DateTime($request->date);
			$filename = 'report-' . $type . '-' . $date->format('Y-m-d') . '.xlsx';
			$return = Excel::download($report, $filename);
		} catch (Exception $e) {
			$return = response()->json([
				'status' => 'error',
				'message' => __('Failed to get!'),
				'detail' => $e->getMessage(),
				'statusCode' => 500,
			]);
		}
		return $return;
	}

	/**
	 * Display resources for DGP Results By Country
	 *
	 * @param Request $request
	 *
	 * @return JsonResponse
	 */
	public function resultsByCountry(Request $request): JsonResponse
	{
		$date = $request->date;

		$GLOBALS['whereRaw'] = HelpersController::mountWhereRaw($request);

		$geralScore = 0;
		$modulesEnabled = 0;

		try {
			$getExecDates = ReportDGP::where('official', true)->groupBy('executed_at')->orderBy('executed_at', 'desc')->take(12)->get(['executed_at'])->toArray();

			$selectFilterCountApproveds = "
			count(executed_at) as total,
			count(executed_at) filter (where channel = 'Site') as total_site,
			count(executed_at) filter (where channel = 'Site' and country != 'Ukraine' and country != 'Russia') as total_site_without_countries,
			count(executed_at) filter (where channel != 'Site') as total_social_media,
			count(executed_at) filter (where iard = true) as iard_count,
			sum(privacy_security) as privacy_security_count,
			sum(property_excellence) as property_excellence_count,
			sum(website_data) as website_data_count";

			$filterCountApproveds = ReportDGP::where('official', true)
				->whereDate('executed_at', $date ? $date : $getExecDates[0]['executed_at'])
				->when((strlen($GLOBALS['whereRaw']) > 0), function ($query) {
					$query->whereRaw($GLOBALS['whereRaw']);
				})
				->selectRaw($selectFilterCountApproveds)
				->get()->toArray();

			$totalDivisor = $filterCountApproveds[0]['total'] > 0 ? $filterCountApproveds[0]['total'] : 1;

			$totalDivisorSite = $filterCountApproveds[0]['total_site'] > 0 ? $filterCountApproveds[0]['total_site'] : 1;

			$totalDivisorSiteWithoutCountries = $filterCountApproveds[0]['total_site_without_countries'] > 0 ? $filterCountApproveds[0]['total_site_without_countries'] : 1;

			$globalValues = array(
				'total' => $filterCountApproveds[0]['total'],
				'total_site' => $filterCountApproveds[0]['total_site'],
				'total_site_without_countries' => $filterCountApproveds[0]['total_site_without_countries'],
				'total_social_media' => $filterCountApproveds[0]['total_social_media'],
				'iard_count' => $filterCountApproveds[0]['iard_count'],
				'privacy_security_count' => $filterCountApproveds[0]['privacy_security_count'],
				'property_excellence_count' => $filterCountApproveds[0]['property_excellence_count'],
				'website_data_count' => $filterCountApproveds[0]['website_data_count']
			);

			if (env('IARD_MODULE')) {
				$globalValues['iard'] = number_format(($filterCountApproveds[0]['iard_count'] / $totalDivisor) * 100, 1);

				$geralScore += ($filterCountApproveds[0]['iard_count'] / $totalDivisor) * 100;

				$modulesEnabled++;
			} else {
				$globalValues['iard'] = 0;
			}

			if (env('PRIVACY_MODULE')) {
				$globalValues['privacy_security'] = number_format(($filterCountApproveds[0]['privacy_security_count'] / $totalDivisorSite), 1);

				$geralScore += ($filterCountApproveds[0]['privacy_security_count'] / $totalDivisorSite);

				$modulesEnabled++;
			} else {
				$globalValues['privacy_security'] = 0;
			}

			if (env('EXCELLENCE_MODULE')) {
				$globalValues['property_excellence'] = number_format(($filterCountApproveds[0]['property_excellence_count'] / $totalDivisorSiteWithoutCountries), 1);

				$geralScore += ($filterCountApproveds[0]['property_excellence_count'] / $totalDivisorSiteWithoutCountries);

				$modulesEnabled++;
			} else {
				$globalValues['property_excellence'] = 0;
			}

			if (env('DATA_MODULE')) {
				$globalValues['website_data'] = number_format(($filterCountApproveds[0]['website_data_count'] / $totalDivisorSiteWithoutCountries), 1);

				$geralScore += ($filterCountApproveds[0]['website_data_count'] / $totalDivisorSiteWithoutCountries);

				$modulesEnabled++;
			} else {
				$globalValues['website_data'] = 0;
			}

			$globalValues['dgp_score'] = number_format((float) ($geralScore) / (float) (($modulesEnabled === 0) ? 1 : $modulesEnabled), 1);

			$lastExecsDataNormalized = $this->getLastExecsDataNormalized($getExecDates, $GLOBALS['whereRaw']);

			$query = ReportDGP::whereDate('executed_at', $date ? $date : $getExecDates[0]['executed_at'])
				->where('official', true)
				->when((strlen($GLOBALS['whereRaw']) > 0), function ($query) {
					$query->whereRaw($GLOBALS['whereRaw']);
				})
				->selectRaw('
					zone,
					country, 
					count(executed_at) as country_count, 
					count(executed_at) filter (where channel = ' . "'Site'" . ') as country_count_site, 
					count(executed_at) filter (where channel = ' . "'Site'" . ' and country != ' . "'Ukraine'" . ' and country != ' . "'Russia'" . ') as total_site_without_countries, 
					count(executed_at) filter (where iard = true) as iard,
					sum(privacy_security) as privacy_security,
					sum(property_excellence) as property_excellence,
					sum(website_data) as website_data,
					count(executed_at) filter (where age_gate = ' . "'true'" . ') as age_gate,
					count(executed_at) filter (where coi = ' . "'true'" . ') as coi,
					count(executed_at) filter (where fan = ' . "'true'" . ') as fan,
					count(executed_at) filter (where rdm = ' . "'true'" . ') as rdm,
					count(executed_at) filter (where ugc = ' . "'true'" . ') as ugc,
					count(executed_at) filter (where lower(risk_rate) = ' . "'low'" . ' or lower(risk_rate) = ' . "'medium'" . ') as risk_rate,
					count(executed_at) filter (where lower(vulnerability_level) = ' . "'low'" . ' or lower(vulnerability_level) = ' . "'medium'" . ') as vulnerability_level,
					count(executed_at) filter (where lower(pia_completion) = ' . "'completed'" . ') as pia_completion,
					count(executed_at) filter (where semrush >= 75) as semrush,
					count(executed_at) filter (where lighthouse_avg >= 50) as lighthouse_avg,
					count(executed_at) filter (where ga_implementation = true) as ga_implementation,
					count(executed_at) filter (where ga_access = true) as ga_access,
					count(executed_at) filter (where ga_taxonomy = true) as ga_taxonomy,
					count(executed_at) filter (where cdp_implementation = true) as cdp_implementation,
					count(executed_at) filter (where cookie_banner_implemented = true) as cookie_banner_implemented,
					count(executed_at) filter (where cookie_dropping_compliance = true) as cookie_dropping_compliance
				')
				->groupBy('country', 'zone')
				->orderByRaw('UPPER(country) ASC, UPPER(zone) ASC')
				->paginate($request->per_page);

			$query->map(function ($item) {
				$geralScore = 0;
				$modulesEnabled = 0;

				$totalDivisor = $item->country_count > 0 ? $item->country_count : 1; // @phpstan-ignore-line

				$totalDivisorSite = $item->country_count_site > 0 ? $item->country_count_site : 1; // @phpstan-ignore-line

				$totalDivisorSiteWithoutCountries = $item->total_site_without_countries > 0 ? $item->total_site_without_countries : 1; // @phpstan-ignore-line

				if (env('IARD_MODULE')) {
					$modulesEnabled++;

					$geralScore += ($item->iard / $totalDivisor) * 100; // @phpstan-ignore-line
				}

				$item->iard = (env('IARD_MODULE')) ? number_format(($item->iard / $totalDivisor) * 100, 1) : 0; // @phpstan-ignore-line

				$item->age_gate = (env('IARD_MODULE')) ? number_format(($item->age_gate / $totalDivisor) * 100, 1) : 0; // @phpstan-ignore-line

				$item->coi = (env('IARD_MODULE')) ? number_format(($item->coi / $totalDivisor) * 100, 1) : 0; // @phpstan-ignore-line

				$item->fan = (env('IARD_MODULE')) ? number_format(($item->fan / $totalDivisor) * 100, 1) : 0; // @phpstan-ignore-line

				$item->rdm = (env('IARD_MODULE')) ? number_format(($item->rdm / $totalDivisor) * 100, 1) : 0; // @phpstan-ignore-line

				$item->ugc = (env('IARD_MODULE')) ? number_format(($item->ugc / $totalDivisor) * 100, 1) : 0; // @phpstan-ignore-line

				if (env('PRIVACY_MODULE')) {
					$modulesEnabled++;

					$geralScore += $item->privacy_security / $totalDivisorSite; // @phpstan-ignore-line
				}

				$item->privacy_security = (env('PRIVACY_MODULE')) ? number_format(($item->privacy_security / $totalDivisorSite), 1) : 0; // @phpstan-ignore-line

				$item->risk_rate = (env('PRIVACY_MODULE')) ? number_format(($item->risk_rate / $totalDivisorSite) * 100, 1) : 0; // @phpstan-ignore-line

				$item->vulnerability_level = (env('PRIVACY_MODULE')) ? number_format(($item->vulnerability_level / $totalDivisorSite) * 100, 1) : 0; // @phpstan-ignore-line

				$item->pia_completion = (env('PRIVACY_MODULE')) ? number_format(($item->pia_completion / $totalDivisorSite) * 100, 1) : 0; // @phpstan-ignore-line

				if (env('EXCELLENCE_MODULE')) {
					$modulesEnabled++;

					$geralScore += $item->property_excellence / $totalDivisorSiteWithoutCountries; // @phpstan-ignore-line
				}

				$item->property_excellence = (env('EXCELLENCE_MODULE')) ? number_format(($item->property_excellence / $totalDivisorSiteWithoutCountries), 1) : 0; // @phpstan-ignore-line

				$item->semrush = (env('EXCELLENCE_MODULE')) ? number_format(($item->semrush / $totalDivisorSiteWithoutCountries) * 100, 1) : 0; // @phpstan-ignore-line

				$item->lighthouse_avg = (env('EXCELLENCE_MODULE')) ? number_format(($item->lighthouse_avg / $totalDivisorSiteWithoutCountries) * 100, 1) : 0; // @phpstan-ignore-line

				if (env('DATA_MODULE')) {
					$modulesEnabled++;

					$geralScore += $item->website_data / $totalDivisorSiteWithoutCountries; // @phpstan-ignore-line
				}

				$item->website_data = (env('DATA_MODULE')) ? number_format(($item->website_data / $totalDivisorSiteWithoutCountries), 1) : 0; // @phpstan-ignore-line

				$item->ga_implementation = (env('DATA_MODULE')) ? number_format(($item->ga_implementation / $totalDivisorSiteWithoutCountries) * 100, 1) : 0; // @phpstan-ignore-line

				$item->ga_access = (env('DATA_MODULE')) ? number_format(($item->ga_access / $totalDivisorSiteWithoutCountries) * 100, 1) : 0; // @phpstan-ignore-line

				$item->ga_taxonomy = (env('DATA_MODULE')) ? number_format(($item->ga_taxonomy / $totalDivisorSiteWithoutCountries) * 100, 1) : 0; // @phpstan-ignore-line

				$item->cdp_implementation = (env('DATA_MODULE')) ? number_format(($item->cdp_implementation / $totalDivisorSiteWithoutCountries) * 100, 1) : 0; // @phpstan-ignore-line

				$item->cookie_banner_implemented = (env('DATA_MODULE')) ? number_format(($item->cookie_banner_implemented / $totalDivisorSite) * 100, 1) : 0; // @phpstan-ignore-line

				$item->cookie_dropping_compliance = (env('DATA_MODULE')) ? number_format(($item->cookie_dropping_compliance / $totalDivisorSite) * 100, 1) : 0; // @phpstan-ignore-line

				$item->dgp_score = number_format((float) ($geralScore / $modulesEnabled), 1); // @phpstan-ignore-line

				return $item;
			});

			$returnArr = array(
				'globalValues' => $globalValues,
				'lastExecsData' => $lastExecsDataNormalized,
				'query' => $query
			);

			$return = [
				'status' => 'success',
				'data' => $returnArr,
				'lastRun' => $date ? $date : $getExecDates[0]['executed_at'],
			];

			$statusCode = 200;
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'details' => $e->getMessage(),
			];

			$statusCode = 500;
		}

		return response()->json($return, $statusCode);
	}

	/**
	 * Export data of DGP Results by Country page
	 *
	 * @param Request $request
	 *
	 * @return BinaryFileResponse|JsonResponse
	 */
	public function resultsByCountryExport(Request $request): BinaryFileResponse|JsonResponse
	{
		$return = '';

		$modulesEnabled = 0;

		$date = $request->date;

		$GLOBALS['whereRaw'] = HelpersController::mountWhereRaw($request);

		try {
			$getExecDates = ReportDGP::where('official', true)->groupBy('executed_at')->orderBy('executed_at', 'desc')->take(12)->get(['executed_at'])->toArray();

			$returnFields = [
				'zone',
				'country',
				'Total assets by Country',
				'Total assets by Country - Website',
				'Total assets by Country - Social Media',
			];

			if (env('IARD_MODULE')) {
				$returnFields = array_merge($returnFields, [
					'iard',
				]);

				$modulesEnabled++;
			}

			if (env('PRIVACY_MODULE')) {
				$returnFields = array_merge($returnFields, [
					'privacy_security',
				]);

				$modulesEnabled++;
			}

			if (env('EXCELLENCE_MODULE')) {
				$returnFields = array_merge($returnFields, [
					'property_excellence',
				]);

				$modulesEnabled++;
			}

			if (env('DATA_MODULE')) {
				$returnFields = array_merge($returnFields, [
					'website_data',
				]);

				$modulesEnabled++;
			}

			if (env('IARD_MODULE')) {
				$returnFields = array_merge($returnFields, [
					'age_gate',
					'coi',
					'fan',
					'rdm',
					'ugc',
				]);
			}

			if (env('PRIVACY_MODULE')) {
				$returnFields = array_merge($returnFields, [
					'risk_rate',
					'vulnerability_level',
					'pia_completion',
					'cookie_banner_implemented',
					'cookie_dropping_compliance',
				]);
			}

			if (env('EXCELLENCE_MODULE')) {
				$returnFields = array_merge($returnFields, [
					'semrush',
					'lighthouse_avg',
				]);
			}

			if (env('DATA_MODULE')) {
				$returnFields = array_merge($returnFields, [
					'ga_implementation',
					'ga_access',
					'ga_taxonomy',
					'cdp_implementation',
				]);
			}

			$returnFields = array_merge($returnFields, [
				'DGP Score',
			]);

			$selectRaw = 'zone,
					country, 
					count(executed_at) as country_count, 
					count(executed_at) filter (where channel = ' . "'Site'" . ') as country_count_site, 
					count(executed_at) filter (where channel != ' . "'Site'" . ') as country_count_social_media';

			if (env('IARD_MODULE')) {
				$selectRaw .= ',count(executed_at) filter (where iard = true) as iard';
			}

			if (env('PRIVACY_MODULE')) {
				$selectRaw .= ',sum(privacy_security) as privacy_security';
			}

			if (env('EXCELLENCE_MODULE')) {
				$selectRaw .= ',sum(property_excellence) as property_excellence';
			}

			if (env('DATA_MODULE')) {
				$selectRaw .= ',sum(website_data) as website_data';
			}

			if (env('IARD_MODULE')) {
				$selectRaw .= ',count(executed_at) filter (where age_gate = ' . "'true'" . ') as age_gate,
					count(executed_at) filter (where coi = ' . "'true'" . ') as coi,
					count(executed_at) filter (where fan = ' . "'true'" . ') as fan,
					count(executed_at) filter (where rdm = ' . "'true'" . ') as rdm,
					count(executed_at) filter (where ugc = ' . "'true'" . ') as ugc';
			}

			if (env('PRIVACY_MODULE')) {
				$selectRaw .= ',count(executed_at) filter (where lower(risk_rate) = ' . "'low'" . ' or lower(risk_rate) = ' . "'medium'" . ') as risk_rate,
					count(executed_at) filter (where lower(vulnerability_level) = ' . "'low'" . ' or lower(vulnerability_level) = ' . "'medium'" . ') as vulnerability_level,
					count(executed_at) filter (where lower(pia_completion) = ' . "'completed'" . ') as pia_completion,
					count(executed_at) filter (where cookie_banner_implemented = ' . "'true'" . ') as cookie_banner_implemented,
					count(executed_at) filter (where cookie_dropping_compliance = ' . "'true'" . ') as cookie_dropping_compliance
					';
			}

			if (env('EXCELLENCE_MODULE')) {
				$selectRaw .= ',count(executed_at) filter (where semrush >= 75) as semrush,
					count(executed_at) filter (where lighthouse_avg >= 50) as lighthouse_avg';
			}

			if (env('DATA_MODULE')) {
				$selectRaw .= ',count(executed_at) filter (where ga_implementation = true) as ga_implementation,
					count(executed_at) filter (where ga_access = true) as ga_access,
					count(executed_at) filter (where ga_taxonomy = true) as ga_taxonomy,
					count(executed_at) filter (where cdp_implementation = true) as cdp_implementation';
			}

			$query = ReportDGP::whereDate('executed_at', $date ? $date : $getExecDates[0]['executed_at'])
				->where('official', true)
				->when((strlen($GLOBALS['whereRaw']) > 0), function ($query) {
					$query->whereRaw($GLOBALS['whereRaw']);
				})
				->selectRaw($selectRaw)
				->groupBy('country', 'zone')
				->orderByRaw('UPPER(country) ASC, UPPER(zone) ASC')
				->get()
				->toArray();

			$headers = array();

			foreach ($returnFields as $key => $value) {
				array_push($headers, $this->fixIndexToExcel($value));
			}

			foreach ($query as $key => $value) {
				$geralScore = 0;

				foreach ($query[$key] as $innerKey => $innerValue) {
					if ($innerKey !== 'zone' && $innerKey !== 'country' && $innerKey !== 'country_count' && $innerKey !== 'country_count_site' && $innerKey !== 'country_count_social_media') {
						$divisor = $query[$key]['country_count_site'] ? $query[$key]['country_count_site'] : 1;

						if ($innerKey === 'iard' || $innerKey === 'age_gate' || $innerKey === 'coi' || $innerKey === 'fan' || $innerKey === 'rdm' || $innerKey === 'ugc') {
							$divisor = $query[$key]['country_count'] ? $query[$key]['country_count'] : 1;
						}

						if ($innerKey === 'property_excellence' || $innerKey === 'website_data' || $innerKey === 'privacy_security') {
							$query[$key][$innerKey] = number_format(((int)$query[$key][$innerKey] / (int)$divisor), 1) . '% - Approveds';
						} else {
							$query[$key][$innerKey] = number_format(((int)$query[$key][$innerKey] / (int)$divisor) * 100, 1) . '% - Approveds';
						}
					}

					if (strlen(var_export($innerValue, true)) === 0 || is_null($innerValue)) {
						$query[$key][$innerKey] = '-';
					} else if ($innerValue === 'true' || $innerValue === true) {
						$query[$key][$innerKey] = 'Approved';
					} else if ($innerValue === 'false' || $innerValue === false) {
						$query[$key][$innerKey] = 'Not approved';
					}

					// Last Index
					if ($innerKey === array_key_last($query[$key])) {
						if (env('IARD_MODULE')) {
							$geralScore += (float) $query[$key]['iard'];
						}

						if (env('PRIVACY_MODULE')) {
							$geralScore += (float) $query[$key]['privacy_security'];
						}

						if (env('EXCELLENCE_MODULE')) {
							$geralScore += (float) $query[$key]['property_excellence'];
						}

						if (env('DATA_MODULE')) {
							$geralScore += (float) $query[$key]['website_data'];
						}

						$query[$key]['dgp_score'] = number_format(($geralScore / ($modulesEnabled === 0 ? 1 : $modulesEnabled)), 1) . '%';
					}
				}
			}

			array_unshift($query, $headers);

			// Initialize the Exporter
			$report = new ReportExport($query);
			$date = new DateTime($date ? $date : $getExecDates[0]['executed_at']);
			// Define filename
			$filename = 'report-' . 'resultsByCountry' . '-' . $date->format('Y-m-d') . '.xlsx';

			// Download file
			$return = Excel::download($report, $filename);
		} catch (Exception $e) {
			$return = response()->json([
				'status' => 'error',
				'message' => __('Failed to get!'),
				'detail' => $e->getMessage(),
				'statusCode' => 500,
			]);
		} finally {
			return $return;
		}
	}

	/**
	 * Display resources for DGP Results By Web Property page
	 *
	 * @param Request $request
	 *
	 * @return JsonResponse
	 */
	public function resultsByWebProp(Request $request): JsonResponse
	{
		$date = $request->date;

		$GLOBALS['whereRaw'] = HelpersController::mountWhereRaw($request);

		try {
			$getExecDates = ReportDGP::where('official', true)->groupBy('executed_at')->orderBy('executed_at', 'desc')->take(12)->get(['executed_at'])->toArray();

			$selectFilterCountApproveds = "
			count(executed_at) as total,
			count(executed_at) filter (where channel = 'Site') as total_site,
			count(executed_at) filter (where channel = 'Site' and country != 'Ukraine' and country != 'Russia') as total_site_without_countries,
			count(executed_at) filter (where channel != 'Site') as total_social_media,
			count(executed_at) filter (where iard = true) as iard_count,
			sum(privacy_security) as privacy_security_count,
			sum(property_excellence) as property_excellence_count,
			sum(website_data) as website_data_count";

			$filterCountApproveds = ReportDGP::where('official', true)
				->whereDate('executed_at', $date ? $date : $getExecDates[0]['executed_at'])
				->when((strlen($GLOBALS['whereRaw']) > 0), function ($query) {
					$query->whereRaw($GLOBALS['whereRaw']);
				})
				->selectRaw($selectFilterCountApproveds)
				->get()->toArray();


			$totalDivisor = $filterCountApproveds[0]['total'] > 0 ? $filterCountApproveds[0]['total'] : 1;

			$totalDivisorSite = $filterCountApproveds[0]['total_site'] > 0 ? $filterCountApproveds[0]['total_site'] : 1;

			$totalDivisorSiteWithoutCountries = $filterCountApproveds[0]['total_site_without_countries'] > 0 ? $filterCountApproveds[0]['total_site_without_countries'] : 1;

			$globalValues = array(
				'total' => $filterCountApproveds[0]['total'],
				'total_site' => $filterCountApproveds[0]['total_site'],
				'total_site_without_countries' => $filterCountApproveds[0]['total_site_without_countries'],
				'total_social_media' => $filterCountApproveds[0]['total_social_media'],
				'iard_count' => $filterCountApproveds[0]['iard_count'],
				'privacy_security_count' => $filterCountApproveds[0]['privacy_security_count'],
				'property_excellence_count' => $filterCountApproveds[0]['property_excellence_count'],
				'website_data_count' => $filterCountApproveds[0]['website_data_count']
			);

			$geralScore = 0;
			$modulesEnabled = 0;

			if (env('IARD_MODULE')) {
				$globalValues['iard'] = number_format(($filterCountApproveds[0]['iard_count'] / $totalDivisor) * 100, 1);

				$geralScore += ($filterCountApproveds[0]['iard_count'] / $totalDivisor) * 100;

				$modulesEnabled++;
			} else {
				$globalValues['iard'] = 0;
			}

			if (env('PRIVACY_MODULE')) {
				$globalValues['privacy_security'] = number_format(($filterCountApproveds[0]['privacy_security_count'] / $totalDivisorSite), 1);

				$geralScore += ($filterCountApproveds[0]['privacy_security_count'] / $totalDivisorSite);

				$modulesEnabled++;
			} else {
				$globalValues['privacy_security'] = 0;
			}

			if (env('EXCELLENCE_MODULE')) {
				$globalValues['property_excellence'] = number_format(($filterCountApproveds[0]['property_excellence_count'] / $totalDivisorSiteWithoutCountries), 1);

				$geralScore += ($filterCountApproveds[0]['property_excellence_count'] / $totalDivisorSiteWithoutCountries);

				$modulesEnabled++;
			} else {
				$globalValues['property_excellence'] = 0;
			}

			if (env('DATA_MODULE')) {
				$globalValues['website_data'] = number_format(($filterCountApproveds[0]['website_data_count'] / $totalDivisorSiteWithoutCountries), 1);

				$geralScore += ($filterCountApproveds[0]['website_data_count'] / $totalDivisorSiteWithoutCountries);

				$modulesEnabled++;
			} else {
				$globalValues['website_data'] = 0;
			}

			$globalValues['dgp_score'] = number_format((float) ($geralScore) / (($modulesEnabled === 0) ? 1 : $modulesEnabled), 1);

			$lastExecsDataNormalized = $this->getLastExecsDataNormalized($getExecDates, $GLOBALS['whereRaw']);

			$query = ReportDGP::whereDate('executed_at', $date ? $date : $getExecDates[0]['executed_at'])
				->where('official', true)
				->when((strlen($GLOBALS['whereRaw']) > 0), function ($query) {
					$query->whereRaw($GLOBALS['whereRaw']);
				})
				->orderByRaw('UPPER(zone) ASC, UPPER(country) ASC, UPPER(link) ASC')
				->paginate($request->per_page, [
					'zone',
					'country',
					'link',
					'channel',
					'iard',
					'privacy_security',
					'property_excellence',
					'website_data',
					'department',
					'age_gate',
					'coi',
					'fan',
					'rdm',
					'ugc',
					'risk_rate',
					'vulnerability_level',
					'pia_completion',
					'semrush',
					'lighthouse_avg',
					'ga_implementation',
					'ga_access',
					'ga_taxonomy',
					'cdp_implementation',
					'cookie_banner_implemented',
					'cookie_dropping_compliance',
				]);

			$returnArr = array(
				'globalValues' => $globalValues,
				'lastExecsData' => $lastExecsDataNormalized,
				'query' => $query
			);

			$return = [
				'status' => 'success',
				'data' => $returnArr,
				'lastRun' => $date ? $date : $getExecDates[0]['executed_at'],
			];

			$statusCode = 200;
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'details' => $e->getMessage(),
			];

			$statusCode = 500;
		}

		return response()->json($return, $statusCode);
	}

	/**
	 * Export data of DGP Results by Web Property Page
	 *
	 * @param Request $request
	 *
	 * @return BinaryFileResponse|JsonResponse
	 */
	public function resultsByWebPropExport(Request $request): BinaryFileResponse|JsonResponse
	{
		$return = '';

		$date = $request->date;

		$GLOBALS['whereRaw'] = HelpersController::mountWhereRaw($request);

		try {
			$getExecDates = ReportDGP::where('official', true)->groupBy('executed_at')->orderBy('executed_at', 'desc')->take(12)->get(['executed_at'])->toArray();

			$returnFields = [
				'zone',
				'country',
				'link',
				'department',
			];

			if (env('IARD_MODULE')) {
				$returnFields = array_merge($returnFields, [
					'iard',
				]);
			}

			if (env('PRIVACY_MODULE')) {
				$returnFields = array_merge($returnFields, [
					'privacy_security',
				]);
			}

			if (env('EXCELLENCE_MODULE')) {
				$returnFields = array_merge($returnFields, [
					'property_excellence',
				]);
			}

			if (env('DATA_MODULE')) {
				$returnFields = array_merge($returnFields, [
					'website_data',
				]);
			}

			if (env('IARD_MODULE')) {
				$returnFields = array_merge($returnFields, [
					'age_gate',
					'coi',
					'fan',
					'rdm',
					'ugc',
				]);
			}

			if (env('PRIVACY_MODULE')) {
				$returnFields = array_merge($returnFields, [
					'risk_rate',
					'vulnerability_level',
					'pia_completion',
					'cookie_banner_implemented',
					'cookie_dropping_compliance',
				]);
			}

			if (env('EXCELLENCE_MODULE')) {
				$returnFields = array_merge($returnFields, [
					'semrush',
					'lighthouse_avg',
				]);
			}

			if (env('DATA_MODULE')) {
				$returnFields = array_merge($returnFields, [
					'ga_implementation',
					'ga_access',
					'ga_taxonomy',
					'cdp_implementation',
				]);
			}

			$query = ReportDGP::whereDate('executed_at', $date ? $date : $getExecDates[0]['executed_at'])
				->where('official', true)
				->when((strlen($GLOBALS['whereRaw']) > 0), function ($query) {
					$query->whereRaw($GLOBALS['whereRaw']);
				})
				->orderByRaw('UPPER(zone) ASC, UPPER(country) ASC, UPPER(link) ASC')
				->get($returnFields)
				->toArray();

			$headers = array();

			foreach ($returnFields as $key => $value) {
				array_push($headers, $this->fixIndexToExcel($value));
			}

			foreach ($query as $key => $value) {
				foreach ($query[$key] as $innerKey => $innerValue) {
					if (strlen(var_export($innerValue, true)) === 0 || is_null($innerValue)) {
						$query[$key][$innerKey] = '-';
					} else if ($innerValue === 'true' || $innerValue === true) {
						$query[$key][$innerKey] = 'Approved';
					} else if ($innerValue === 'false' || $innerValue === false) {
						$query[$key][$innerKey] = 'Not approved';
					}
				}
			}

			array_unshift($query, $headers);

			// Initialize the Exporter
			$report = new ReportExport($query);

			$date = new DateTime($date ? $date : $getExecDates[0]['executed_at']);

			// Define filename
			$filename = 'report-' . 'resultsByWebProp' . '-' . $date->format('Y-m-d') . '.xlsx';

			// Download file
			$return = Excel::download($report, $filename);
		} catch (Exception $e) {
			$return = response()->json([
				'status' => 'error',
				'message' => __('Failed to get!'),
				'detail' => $e->getMessage(),
				'line' => $e->getLine(),
				'statusCode' => 500,
			]);
		} finally {
			return $return;
		}
	}
}
