<?php

namespace App\Http\Controllers;

use App\Exports\ReportExport;
use App\Http\Controllers\Helpers\HelpersController;
use App\Models\Country;
use App\Models\Criteria;
use App\Models\Keywords;
use App\Models\Locale;
use DateTime;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Maatwebsite\Excel\Facades\Excel;
use Symfony\Component\HttpFoundation\BinaryFileResponse;

/**
 * Keywords Controller
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Http\Controllers
 */

class KeywordsController extends Controller
{
    /**
     * Map of request parameters to respective table.column in db
     */
    protected $bdMapColumns = [
        'locale' => 'locales.locale',
        'criteriaKw' => 'criteria.name',
        'text' => 'keywords.keyword',
    ];

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     *
     * @return JsonResponse
     */
    public function destroy(int $id): JsonResponse
    {
        try {
            Keywords::where('id', $id)->delete();

            $return = [
                'status' => 'success',
                'message' => __('Deleted successfully!'),
            ];
            $statusCode = 201;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to delete!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Display a paginated listing of the resource.
     *
     * @param Request $request
     *
     * @return BinaryFileResponse|JsonResponse
     */
    public function exportKeywords(Request $request): BinaryFileResponse|JsonResponse
    {
        try {
            $query = Keywords::query()
                ->join('criteria', 'keywords.criteria_id', '=', 'criteria.id')
                ->join('locales', 'keywords.locale_id', '=', 'locales.id');

            HelpersController::buildRequestWhere($query, $request->all(), $this->bdMapColumns);

            $query = $query->orderByRaw('UPPER(keywords.keyword) ASC, UPPER(criteria.name) ASC')
                ->get([
                    'keywords.keyword as keyword',
                    'keywords.group as group',
                    'criteria.name as criteria',
                    'locales.locale as language',
                ])
                ->toArray();

            foreach ($query as $key => $value) {
                foreach ($query[$key] as $innerKey => $innerValue) {
                    if (strlen($innerValue) === 0 || is_null($innerValue)) {
                        $query[$key][$innerKey] = '-';
                    } elseif ($innerValue === 'true') {
                        $query[$key][$innerKey] = 'Approved';
                    } elseif ($innerValue === 'false') {
                        $query[$key][$innerKey] = 'Not approved';
                    }
                }
            }

            array_unshift($query, [
                'Keyword',
                'Group',
                'Criteria',
                'Language',
            ]);

            $report = new ReportExport($query);
            $date = new DateTime('today');
            $filename = 'keywords' . '-' . $date->format('Y-m-d') . '.xlsx';
            $return = Excel::download($report, $filename);
        } catch (Exception $e) {
            $return = response()->json([
                'status' => 'error',
                'message' => __('Failed to get!'),
                'detail' => $e->getMessage(),
                'statusCode' => 500,
            ]);
        }
        return $return;
    }

    /**
     * Display a listing of resources.
     *
     * @return JsonResponse
     */
    public function filterKeywords(): JsonResponse
    {
        try {
            // criteria_category_id = 42 = IARD
            $criteria = Criteria::orderByRaw('UPPER(name) ASC')->where('criteria_category_id', '=', 42)->get(['name', 'id']);
            $locale = Locale::orderByRaw('UPPER(locale) ASC')->get(['id', 'locale']);
            $countries = Country::orderByRaw('UPPER(name) ASC')->get(['locale_id', 'name']);

            $return = [
                'status' => 'success',
                'data' => array(
                    'criteria' => $criteria,
                    'locale' => $locale,
                    'countries' => $countries
                ),
            ];
            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'detail' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Display a paginated listing of the resource.
     *
     * @param Request $request
     *
     * @return JsonResponse
     */
    public function getKeywordsPaginated(Request $request): JsonResponse
    {
        try {
            $query = Keywords::query()
                ->join('criteria', 'keywords.criteria_id', '=', 'criteria.id')
                ->join('locales', 'keywords.locale_id', '=', 'locales.id');

            HelpersController::buildRequestWhere($query, $request->all(), $this->bdMapColumns);

            $query = $query->orderByRaw('UPPER(keywords.keyword) ASC, UPPER(criteria.name) ASC')
                ->paginate($request->per_page, [
                    'keywords.id as id',
                    'keywords.keyword as keyword',
                    'keywords.group as group',
                    'criteria.name as criteria',
                    'locales.locale as locale',
                ]);

            $return = [
                'status' => 'success',
                'data' => $query,
            ];
            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Show data for a given id.
     *
     * @param int $id
     *
     * @return JsonResponse
     */
    public function showDetailedKeyword(int $id): JsonResponse
    {
        try {
            $data = Keywords::join('criteria', 'keywords.criteria_id', '=', 'criteria.id')
                ->join('locales', 'keywords.locale_id', '=', 'locales.id')
                ->where('keywords.id', $id)
                ->withTrashed()
                ->first([
                    'keywords.keyword as keyword',
                    'keywords.group as group',
                    'criteria.name as criteria',
                    'locales.locale as locale',
                ]);

            $return = [
                'status' => 'success',
                'data' => $data,
            ];

            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param string $params
     * @param bool $fromTicketsController
     *
     * @return JsonResponse
     */
    public function store(string $params, bool $fromTicketsController): JsonResponse
    {
        $params = json_decode($params, true);

        try {
            $data = new Keywords();
            if ($fromTicketsController) {
                $data->keyword = $params['keyword'];
                $data->criteria_id = $params['criteria']['id'];
                $data->locale_id = $params['locale']['id'];
                $data->group = $params['group']['value'];
            }

            $data->save();

            $return = [
                'status' => 'success',
                'message' => __('Created successfully'),
            ];
            $statusCode = 201;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to Create!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param int $id
     * @param string $params
     * @param bool $fromTicketsController
     *
     * @return JsonResponse
     */
    public function update(int $id, string $params, bool $fromTicketsController): JsonResponse
    {
        $params = json_decode($params, true);

        try {
            $data = Keywords::find($id);

            if ($fromTicketsController) {
                $data->keyword = $params['keyword'];
                $data->criteria_id = $params['criteria']['id'];
                $data->locale_id = $params['locale']['id'];
                $data->group = $params['group']['value'];
            }

            try {
                $data->save();

                $return = [
                    'status' => 'success',
                    'message' => __('Updated successfully'),
                ];
                $statusCode = 200;
            } catch (Exception $e) {
                throw new Exception($e);
            }
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to Update!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }
}
