<?php

// phpcs:disable PSR2.ControlStructures.ElseIfDeclaration.NotAllowed

namespace App\Http\Controllers;

use App\Models\Crawls;
use App\Models\Criteria;
use App\Models\LighthouseData;
use App\Models\LinkCriteria;
use App\Models\Links;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;

/**
 * LinkCriteria Controller
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Http\Controllers
 */

class LinkCriteriaController extends Controller
{
	/**
	 * Remove the specified resource from storage.
	 *
	 * @param int $id
	 *
	 * @return JsonResponse
	 */
	public function destroy(int $id): JsonResponse
	{
		try {
			LinkCriteria::destroy($id);
			$return = [
				'status' => 'success',
				'message' => __('Deleted successfully!'),
			];
			$statusCode = 201;
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to delete!'),
				'details' => $e->getMessage(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Get latest Lighthouse Crawler execution details
	 *
	 * @return JsonResponse
	 */
	public function getLatestLighthouse(): JsonResponse
	{
		$return = [];
		$statusCode = 200;

		try {
			$criteria = Criteria::where('slug', 'lighthouse')->firstOrFail();

			$entries = DB::select('SELECT lc.id, l.link, lc.executed_at, lc.response FROM links_criteria lc join links l on lc.link_id = l.id WHERE NOT EXISTS (SELECT id FROM links_criteria AS lcc where lcc.link_id = lc.link_id and lcc.executed_at > lc.executed_at and lcc.criteria_id = ' . $criteria->id . ' ) and lc.criteria_id = ' . $criteria->id . ' order by l.link');

			if (count($entries) === 0) {
				// @phpstan-ignore-next-line
				throw new Exception('Failed to get tests for this period');
			}

			$return = [
				'status' => 'success',
				'data' => $entries,
			];

			$statusCode = 200;
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => 'Failed getting latest lighthouse report',
				'details' => $e->getMessage(),
			];
			$statusCode = 500;
		} finally {
			// @phpstan-ignore-next-line
			return response()->json($return, $statusCode);
		}
	}

	/**
	 * For given parameters, retrieve lighthouse data
	 *
	 * @param Request $request
	 *
	 * @return JsonResponse
	 */
	public function getLighthouseData(Request $request): JsonResponse
	{
		$statusCode = 0;
		$report = [];
		$return = [];

		try {
			$report = LighthouseData::query();

			$report->when(!empty($request->zone), function ($q) {
				return $q->where('zone', 'ilike', request('zone'));
			});

			$report->when(!empty($request->country), function ($q) {
				return $q->where('country', 'ilike', request('country'));
			});

			$report->when(!empty($request->link), function ($q) {
				return $q->where('link', 'ilike', request('link'));
			});

			$report->when(!empty($request->executedAt), function ($q) {
				return $q->where('executed_at', request('executedAt'));
			});

			$report->orderByRaw('UPPER(zone) ASC, UPPER(country) ASC, UPPER(link) ASC, executed_at desc');

			$report = $report->paginate($request->per_page, [
				'zone',
				'country',
				'brand',
				'link',
				'executed_at',
				'first_contentful_paint',
				'largest_contentful_paint',
				'total_blocking_time',
				'speed_index',
				'time_to_interactive',
				'cumulative_layout_shift',
				'lighthouse_avg',
				'lighthouse_last',
			]);

			$report->setPath('/php/api/lighthouse');

			$return = [
				'status' => 'success',
				'data' => $report,
			];

			$statusCode = 200;
		} catch (Exception $e) {
			$report = [];
			$statusCode = 500;

			$return = [
				'status' => 'error',
				'message' => __('Failed to retrieve'),
				'details' => $e->getMessage(),
			];
		} finally {
			return response()->json($return, $statusCode);
		}
	}

	/**
	 * For a given link/execution return Lighthouse report
	 *
	 * @param string $link
	 * @param string $executedAt
	 *
	 * @return JsonResponse
	 */
	public function getLighthouseReport(string $link, string $executedAt): JsonResponse
	{
		$return = [];
		$statusCode = 200;

		try {
			$report = Crawls::where([
				['link', $link],
				['executed_at', $executedAt],
				['data_type', 'crawler:lighthouse'],
			])->firstOrFail();

			$return = [
				'status' => 'success',
				'data' => $report->content,
			];
			$statusCode = 200;
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => 'Failed getting report',
				'details' => $e->getMessage(),
			];
			$statusCode = 500;
		} finally {
			return response()->json($return, $statusCode);
		}
	}

	/**
	 * For a given execution_at return responses
	 *
	 * @param string $executedAt
	 *
	 * @return JsonResponse
	 */
	public function getTestsByExecution(string $executedAt): JsonResponse
	{
		$statusCode = 500;
		$return = [];

		try {
			$criterias = LinkCriteria::where('executed_at', $executedAt)->get(['id', 'criteria_id', 'link_id', 'executed_at', 'response', 'details', 'official']);

			$return = [
				'status' => 'success',
				'data' => $criterias,
			];
			$statusCode = 200;
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'details' => $e->getMessage(),
			];
			$statusCode = 500;
		} finally {
			return response()->json($return, $statusCode);
		}
	}

	/**
	 * Display a listing of the resource.
	 *
	 * @return JsonResponse
	 */
	public function index(): JsonResponse
	{
		try {
			$data = LinkCriteria::all();

			$return = [
				'status' => 'success',
				'data' => $data,
			];
			$statusCode = 200;
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'details' => $e->getMessage(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Display a paginated listing of the resource.
	 *
	 * @param string $type
	 * @param Request $request
	 *
	 * @return JsonResponse
	 */
	//
	public function index_paginated(string $type, Request $request): JsonResponse
	{
		$criteriaSlug = array();
		switch ($type) {
			case 'page-speed':
				array_push($criteriaSlug, 'lighthouse');
				break;
			case 'iard':
				array_push($criteriaSlug, 'age', 'coi', 'fan', 'rdm', 'ugc');
				break;
			default:
				$criteriaSlug = array_push($criteriaSlug, $type);
		}

		$conditions = array();

		$date = $request->date;
		$zones = $request->zones;
		$countries = $request->countries;
		$brands = $request->brands;
		$channels = $request->channels;
		$text = $request->text;

		if ($zones && count($zones) > 0) {
			foreach ($zones as $key => $value) {
				$value = json_decode($value, true);
				array_push($conditions, ['zones.name', 'ILIKE', '%' . $value['value'] . '%', 'zones']);
			}
		}
		if ($countries && count($countries) > 0) {
			foreach ($countries as $key => $value) {
				$value = json_decode($value, true);
				array_push($conditions, ['countries.name', 'ILIKE', '%' . $value['value'] . '%', 'countries']);
			}
		}
		if ($brands && count($brands) > 0) {
			foreach ($brands as $key => $value) {
				$value = json_decode($value, true);
				array_push($conditions, ['brands.slug', 'ILIKE', '%' . $value['value'] . '%', 'brands']);
			}
		}
		if ($channels && count($channels) > 0) {
			foreach ($channels as $key => $value) {
				array_push($conditions, ['channels.name', 'ILIKE', '%' . $value . '%', 'channels']);
			}
		}
		if ($text && strlen($text) > 0) {
			array_push($conditions, ['links.link', 'ILIKE', '%' . $text . '%', 'links']);
		}

		$lastFilter = [];
		$whereRaw = '';
		$lastCondition = count($conditions) - 1;
		foreach ($conditions as $key => $condition) {
			if ($key === 0) {
				$lastFilter = $condition[3];
				$whereRaw = "($condition[0] $condition[1] '$condition[2]'";
				if ($key === $lastCondition) $whereRaw = $whereRaw . ')';
				continue;
			}
			if ($lastFilter === $condition[3]) {
				$whereRaw = $whereRaw . " OR $condition[0] $condition[1] '$condition[2]'";
				if ($key === $lastCondition) $whereRaw = $whereRaw . ')';
			} else {
				$lastFilter = $condition[3];
				$whereRaw = $whereRaw . ") AND ($condition[0] $condition[1] '$condition[2]'";
				if ($key === $lastCondition) $whereRaw = $whereRaw . ')';
			}
		}
		$GLOBALS['whereRaw'] = $whereRaw;
		try {
			// DB::enableQueryLog(); // Enable query log
			$lastRun = LinkCriteria::where('official', true)->orderBy('executed_at', 'desc')->first();
			$query = LinkCriteria::join('criteria', 'links_criteria.criteria_id', '=', 'criteria.id')
				->join('criteria_category', 'criteria.criteria_category_id', '=', 'criteria_category.id')
				->join('links', 'links.id', 'links_criteria.link_id')
				->join('brands', 'brands.id', 'links.brand_id')
				->join('countries', 'countries.id', 'links.country_id')
				->join('zones', 'zones.id', 'countries.zone_id')
				->join('channels', 'channels.id', 'links.channel_id')
				// ->whereDate('links_criteria.executed_at', '2022-03-02')
				->whereDate('links_criteria.executed_at', $date ? $date : $lastRun['executed_at'])
				->whereIn('criteria.slug', $criteriaSlug)
				->when((strlen($whereRaw) > 0), function ($query) {
					$query->whereRaw($GLOBALS['whereRaw']);
				})
				->orderByRaw('UPPER(zones.name) ASC, UPPER(countries.name) ASC, UPPER(links.link) ASC')
				->paginate($request->per_page, array(
					'links.id as linkId',
					'criteria.name as criteria',
					'links.link as link',
					'zones.name as zone',
					'countries.name as country',
					'brands.name as brand',
					'channels.name as channel',
					'criteria_category.name as criteriaCategory',
					'links_criteria.response as response',
					'links_criteria.executed_at as lastExecution'
				));

			// dd(DB::getQueryLog()); // Show results of log

			$return = [
				'status' => 'success',
				'data' => $query,
				'lastRun' => $date ? $date : $lastRun['executed_at'],
			];
			$statusCode = 200;
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'details' => $e->getMessage(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Show data for a given id.
	 *
	 * @param int $id
	 *
	 * @return JsonResponse
	 */
	public function show(int $id): JsonResponse
	{
		try {
			$data = LinkCriteria::find($id);

			$return = [
				'status' => 'success',
				'data' => $data,
			];

			$statusCode = 200;
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'details' => $e->getMessage(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Store a newly created resource in storage.
	 *
	 * @param string $params
	 *
	 * @return JsonResponse
	 */
	public function store(string $params): JsonResponse
	{
		$params = json_decode($params, true);

		try {
			$rules = [
				'criteriaId' => 'required_without:criteria|integer',
				'linkId' => 'required_without:link|integer',
				'response' => 'required',
			];

			$validator = Validator::make($params, $rules);

			if ($validator->fails()) {
				throw new Exception($validator->messages()->first());
			}

			// In case we doesn't have specified official parameter, let's define as false
			if (!(isset($params['official']))) {
				$params['official'] = false;
			}

			// In case we doesn't have a execution date, lets use the current date
			if (!(isset($params['executed_at']))) {
				$params['executed_at'] = date('Y-m-d 00:00:00');
			}

			// If isset criteria instead of criteriaId (api request), we need to set the id - using the slug
			if (isset($params['criteria'])) {
				$criteria = Criteria::where('slug', $params['criteria'])->first();

				if (is_null($criteria)) {
					throw new Exception('Criteria\'s slug not found');
				} else {
					$params['criteriaId'] = $criteria->id;
				}
			}

			$link = [];

			// If isset link instead of linkId (api request), we need to set the id
			if (isset($params['link'])) {
				$link = Links::where('consider', true)->where(function ($query) use ($params) {
					$query->where('link', $params['link'])
						->orWhere('link', 'http://' . $params['link'])
						->orWhere('link', 'https://' . $params['link'])
						->orWhere('link', 'http://' . strtolower($params['link']))
						->orWhere('link', 'https://' . strtolower($params['link']))
						->orWhere('link', strtolower($params['link']));
				})->get()->toArray();


				if (count($link) === 0) {
					throw new Exception('Link not found! Please double check the link, to ensure there are no typos.');
				}
			} else {
				array_push($link, Links::find($params['linkId']));
			}

			foreach ($link as $key => $value) {
				$data = LinkCriteria::where([
					['link_id', $link[$key]['id']],
					['executed_at', $params['executed_at']],
					['criteria_id', $params['criteriaId']],
				])->first();

				if (!$data) {
					$data = new LinkCriteria();
				}

				$data->criteria_id = $params['criteriaId'];
				$data->link_id = $link[$key]['id'];
				$data->response = $params['response'];

				$data->executed_at = $params['executed_at'];

				$data->official = $params['official'];

				$data->save();
			}

			$returnMessage = ($data->created_at !== $data->updated_at) ? 'Updated successfully' : 'Created successfully';

			$return = [
				'status' => 'success',
				'message' => __($returnMessage),
			];
			$statusCode = 201;
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to Create!'),
				'detail' => $e->getMessage(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Update the specified resource in storage.
	 *
	 * @param int $id
	 * @param string $params
	 *
	 * @return JsonResponse
	 */
	public function update(int $id, string $params): JsonResponse
	{
		$params = json_decode($params, true);

		try {
			$data = LinkCriteria::find($id);

			if (!empty($params['criteriaId'])) {
				$data->criteria_id = $params['criteriaId'];
			}

			if (!empty($params['linkId'])) {
				$data->link_id = $params['linkId'];
			}

			$data->response = $params['response'];

			$data->save();

			$return = [
				'status' => 'success',
				'message' => __('Updated successfully'),
			];
			$statusCode = 200;
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to Update!'),
				'detail' => $e->getMessage(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}
}
