<?php

namespace App\Http\Controllers;

use App\Exports\ReportExport;
use App\Http\Controllers\Helpers\HelpersController;
use App\Models\Links;
use App\Models\Ownership;
use DateTime;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Maatwebsite\Excel\Facades\Excel;
use Symfony\Component\HttpFoundation\BinaryFileResponse;

/**
 * Links Controller
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Http\Controllers
 */

class LinksController extends Controller
{
    /**
	 * Map of request parameters to respective table.column in db
	*/
	protected $bdMapColumns = [
        'brands' => 'brands.slug',
        'channels' => 'channels.name',
        'countries' => 'countries.name',
        'zones' => 'zones.name',
        'department' => 'links.scope',
        'status' => 'links.http_status',
        'environment' => 'links.environment',
        'dgp_goal' => 'links.consider',
        'text' => 'links.link',
    ];

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     *
     * @return JsonResponse
     */
    public function destroy(int $id): JsonResponse
    {
        try {
            Links::destroy($id);
            $return = [
                'status' => 'success',
                'message' => __('Deleted successfully!'),
            ];
            $statusCode = 201;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to delete!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Display a paginated listing of the resource.
     *
     * @param Request $request
     *
     * @return BinaryFileResponse|JsonResponse
     */
    public function exportWebPropertiesList(Request $request): BinaryFileResponse|JsonResponse
    {
        try {
            $query = Links::query()
                ->join('brands', 'links.brand_id', '=', 'brands.id')
                ->join('channels', 'links.channel_id', '=', 'channels.id')
                ->join('countries', 'links.country_id', '=', 'countries.id')
                ->join('zones', 'countries.zone_id', '=', 'zones.id');

            HelpersController::buildRequestWhere($query, $request->all(), $this->bdMapColumns);

            $query = $query->orderByRaw('UPPER(zones.name) ASC, UPPER(countries.name) ASC, UPPER(links.link) ASC')
                ->get(array(
                    'links.link as link',
                    'zones.name as zone',
                    'countries.name as country',
                    'channels.name as channel',
                    'brands.name as brand',
                    'links.http_status as status',
                    'links.consider as dgp_goal',
                    'links.environment as environment',
                    'links.scope as department',
                    'links.alcoholic as alcoholic'
                ))
                ->toArray();

            $returnFields = array(
                'Link',
                'Zone',
                'Country',
                'Channel',
                'Brand',
                'Status',
                'DGP Goal',
                'Environment',
                'Department',
                'Alcoholic'
            );

            foreach ($query as $key => $value) {
                foreach ($query[$key] as $innerKey => $innerValue) {
                    if (strlen($innerValue) === 0 || is_null($innerValue)) {
                        $query[$key][$innerKey] = '-';
                    } elseif ($innerValue === true) {
                        $query[$key][$innerKey] = 'Considered';
                    } elseif ($innerValue === false) {
                        $query[$key][$innerKey] = 'Not considered';
                    }
                }
            }

            array_unshift($query, $returnFields);

            $report = new ReportExport($query);
            $date = new DateTime('today');
            $filename = 'report-' . 'webPropertiesList' . '-' . $date->format('Y-m-d') . '.xlsx';
            $return = Excel::download($report, $filename);
        } catch (Exception $e) {
            $return = response()->json([
                'status' => 'error',
                'message' => __('Failed to get!'),
                'detail' => $e->getMessage(),
                'statusCode' => 500,
            ]);
        }
        return $return;
    }

    /**
     * Get a entry for a specified link
     *
     * @param string $link
     *
     * @return JsonResponse
     */
    public function getByLink(string $link = ''): JsonResponse
    {
        if ($link !== '') {
            try {
                $data = Links::where('link', $link)->get();

                $return = [
                    'status' => 'success',
                    'data' => $data,
                ];

                $statusCode = 200;
            } catch (Exception $e) {
                $return = [
                    'status' => 'error',
                    'message' => __('Failed to get!'),
                    'detail' => $e->getMessage(),
                ];
                $statusCode = 500;
            }
        } else {
            $return = [
                'status' => 'success',
                'data' => '',
            ];
            $statusCode = 200;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index(): JsonResponse
    {
        try {
            $data = Links::all();

            $return = [
                'status' => 'success',
                'data' => $data,
            ];
            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function indexConsider(): JsonResponse
    {
        try {
            $data = Links::where('consider', true)->get()->toArray();

            $return = [
                'status' => 'success',
                'data' => $data,
            ];
            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Show data for a given id.
     *
     * @param int $id
     *
     * @return JsonResponse
     */
    public function show(int $id): JsonResponse
    {
        try {
            $data = Links::find($id);

            $return = [
                'status' => 'success',
                'data' => $data,
            ];

            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param string $params
     *
     * @return JsonResponse
     */
    public function store(string $params): JsonResponse
    {
        $params = json_decode($params, true);

        try {
            $data = new Links();
            $data->link = $params['link'];

            if (isset($params['additional_crawler'])) {
                $data->additional_crawler = $params['additional_crawler'];
            }

            if (isset($params['additional_crawler'])) {
                $data->semrush_project_id = $params['semrush_project_id'];
            }

            $data->brand_id = $params['brand_id'];
            $data->channel_id = $params['channel_id'];
            $data->country_id = $params['country_id'];

            if (isset($params['owner_id'])) {
                $data->owner_id = $params['owner_id'];
            }

            $data->http_status = $params['http_status'];
            $data->business_area = $params['business_area'];
            $data->environment = $params['environment'];
            $data->scope = $params['scope'];

            $data->in_ga = (isset($params['in_ga']) ? $params['in_ga'] : false);
            $data->in_acquia = (isset($params['in_acquia']) ? $params['in_acquia'] : false);
            $data->in_bp = (isset($params['in_bp']) ? $params['in_bp'] : false);
            $data->in_cdp = (isset($params['in_cdp']) ? $params['in_cdp'] : false);
            $data->alcoholic = (isset($params['alcoholic']) ? $params['alcoholic'] : true);

            $data->consider = (isset($params['consider'])) ? $params['consider'] : false;

            $data->save();

            $ownership = new Ownership();
            $ownership->link_id = $data->id;
            $ownership->save();

            $return = [
                'status' => 'success',
                'message' => __('Created successfully'),
            ];
            $statusCode = 201;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to Create!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param int $id
     * @param string $params
     *
     * @return JsonResponse
     */
    public function update(int $id, string $params): JsonResponse
    {
        $params = json_decode($params, true);

        try {
            $data = Links::find($id);

            if (!empty($params['link'])) {
                $data->link = $params['link'];
            }

            if (!empty($params['additional_crawler'])) {
                $data->additional_crawler = $params['additional_crawler'];
            }

            if (!empty($params['semrush_project_id'])) {
                $data->semrush_project_id = $params['semrush_project_id'];
            }

            if (!empty($params['brand_id'])) {
                $data->brand_id = $params['brand_id'];
            }

            if (!empty($params['channel_id'])) {
                $data->channel_id = $params['channel_id'];
            }

            if (!empty($params['country_id'])) {
                $data->country_id = $params['country_id'];
            }

            if (!empty($params['owner_id'])) {
                $data->owner_id = $params['owner_id'];
            }

            if (!empty($params['http_status'])) {
                $data->http_status = $params['http_status'];
            }

            if (!empty($params['business_area'])) {
                $data->business_area = $params['business_area'];
            }

            if (!empty($params['environment'])) {
                $data->environment = $params['environment'];
            }

            if (!empty($params['scope'])) {
                $data->scope = $params['scope'];
            }

            if (!empty($params['in_ga'])) {
                $data->in_ga = $params['in_ga'];
            }

            if (!empty($params['in_acquia'])) {
                $data->in_acquia = $params['in_acquia'];
            }

            if (!empty($params['in_bp'])) {
                $data->in_bp = $params['in_bp'];
            }

            if (!empty($params['in_cdp'])) {
                $data->in_cdp = $params['in_cdp'];
            }

            if (!empty($params['alcoholic'])) {
                $data->alcoholic = $params['alcoholic'];
            }

            if (!empty($params['consider'])) {
                $data->consider = $params['consider'];
            }

            try {
                $data->save();

                $return = [
                    'status' => 'success',
                    'message' => __('Updated successfully'),
                ];
                $statusCode = 200;
            } catch (Exception $e) {
                throw new Exception($e);
            }
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to Update!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Display a paginated listing of the resource.
     *
     * @param Request $request
     *
     * @return JsonResponse
     */
    public function webPropertiesList(Request $request): JsonResponse
    {
        try {
			// DB::enableQueryLog();
            $query = Links::query()
                ->join('brands', 'links.brand_id', '=', 'brands.id')
                ->join('channels', 'links.channel_id', '=', 'channels.id')
                ->join('countries', 'links.country_id', '=', 'countries.id')
                ->join('zones', 'countries.zone_id', '=', 'zones.id');

            HelpersController::buildRequestWhere($query, $request->all(), $this->bdMapColumns);

            $query = $query->orderByRaw('UPPER(zones.name) ASC, UPPER(countries.name) ASC, UPPER(links.link) ASC')
                ->paginate($request->per_page, array(
                    'links.id as linkId',
                    'links.link as link',
                    'zones.name as zone',
                    'countries.name as country',
                    'channels.name as channel',
                    'brands.name as brand',
                    'links.http_status as status',
                    'links.consider as dgp_goal',
                    'links.environment as environment',
                    'links.scope as department',
                    'brands.slug as brand_slug',
                    'links.alcoholic as alcoholic'
                ));
			// dd(DB::getQueryLog());

            $return = [
                'status' => 'success',
                'data' => $query,
            ];
            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }
}
