<?php

namespace App\Http\Controllers;

use App\Http\Controllers\Helpers\HelpersController;
use App\Models\LinkCriteria;
use App\Models\Links;
use App\Models\Logs;
use App\Models\ManualValidation;
use App\Models\ReportDGPDraft;
use DateTime;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Artisan;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;

/**
 * Manual Validation Controller
 *
 * @author Tércio Lacerda <Tercio.lacerda-ext@ab-inbev.com>
 * @package App\Http\Controllers
 */
class ManualValidationController extends Controller
{
	/**
	 * Map of request parameters to respective table.column in db
	 */
	protected $bdMapColumns = [
		'text' => 'reportdgp_draft.link',
		'date' => 'reportdgp_draft.executed_at',
		'zones' => 'reportdgp_draft.zone',
		'countries' => 'reportdgp_draft.country',
		'channels' => 'reportdgp_draft.channel',
		'criteria' => 'map',
		'iard' => 'reportdgp_draft.iard',
		'website_data' => 'reportdgp_draft.website_data',
		'property_excellence' => 'reportdgp_draft.property_excellence',
		'privacy_security' => 'reportdgp_draft.privacy_security',
		'manual_validation_approved' => 'manual_validation.validated',
	];

	/**
	 * Show or hide a asset on specific report.
	 *
	 * @param Request $request
	 *
	 * @return JsonResponse
	 */
	public function assetDisplayOnReport(Request $request): JsonResponse
	{
		try {
			$params = $request->all();
			if ((empty($params['responseAction']) || strlen($params['responseAction']) === 0) || (empty($params['link_id']) || strlen($params['link_id']) === 0) || (empty($params['executed_at']) || strlen($params['executed_at']) === 0)) {
				throw new Exception('Requireds params are missing.', 1);
			}

			LinkCriteria::query()
				->whereDate('executed_at', $params['executed_at'])
				->where('link_id', $params['link_id'])
				->update(['use_on_report' => $params['responseAction'] === 'add' ? true : false]);

			ManualValidation::query()
				->whereDate('exec_date', $params['executed_at'])
				->where('link_id', $params['link_id'])
				->update(['use_on_report' => $params['responseAction'] === 'add' ? true : false]);

			$newLog = new Logs();
			$newLog->log_type = 'manual-validation-display-on-report';
			$newLog->user_id = $request->userReq->userId;
			$newLog->link_id = $params['link_id'];
			$newLog->details = json_encode([
				'exec_date' => $params['executed_at'],
				'action' => $params['responseAction'],
			]);
			$newLog->save();

			$return = [
				'status' => 'success',
				'message' => __('Updated successfully'),
			];
			$statusCode = 200;
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'details' => $e->getMessage(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Validate or unvalidated a asset in specific report.
	 *
	 * @param Request $request
	 *
	 * @return JsonResponse
	 */
	public function assetValidation(Request $request): JsonResponse
	{
		try {
			$params = $request->all();
			if ((empty($params['responseAction']) || strlen($params['responseAction']) === 0) || (empty($params['link_id']) || strlen($params['link_id']) === 0) || (empty($params['executed_at']) || strlen($params['executed_at']) === 0)) {
				throw new Exception('Requireds params are missing.', 1);
			}

			ManualValidation::query()
				->whereDate('exec_date', $params['executed_at'])
				->where('link_id', $params['link_id'])
				->update(['validated' => $params['responseAction'] === 'validated' ? true : false]);

			$detailsArr = [
				'exec_date' => $params['executed_at'],
				'action' => $params['responseAction'],
			];

			$criterias = json_decode($params['criteriasToChange'], true);
			// if ($params['responseAction'] === 'validated' && $criterias) {
			if ($criterias) {
				$arr = [];
				foreach ($criterias as $key => $value) {
					LinkCriteria::query()
						->whereDate('executed_at', $params['executed_at'])
						->where('link_id', $params['link_id'])
						->where('criteria_id', $value['id'])
						->update(['response' => $value['update']]);

					array_push($arr, ['field' => $key, 'current' => $value['current'], 'update' => $value['update'], 'criteria_id' => $value['id']]);
				}
				$detailsArr['criterias'] = $arr;
			}

			$newLog = new Logs();
			$newLog->log_type = 'manual-validation';
			$newLog->user_id = $request->userReq->userId;
			$newLog->link_id = $params['link_id'];
			$newLog->details = json_encode($detailsArr);
			$newLog->save();

			$return = [
				'status' => 'success',
				'message' => __('Updated successfully'),
			];
			$statusCode = 200;
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'details' => $e->getMessage(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Show or hide a asset on specific report.
	 *
	 * @param Request $request
	 *
	 * @return JsonResponse
	 */
	public function bulkAssetDisplayOnReport(Request $request): JsonResponse
	{
		try {
			$params = $request->all();

			if ((empty($params['responseAction']) || strlen($params['responseAction']) === 0) || (empty($params['assets']) || count($params['assets']) === 0) || (empty($params['executed_at']) || strlen($params['executed_at']) === 0)) {
				throw new Exception('Requireds params are missing.', 1);
			}

			LinkCriteria::query()
				->whereDate('executed_at', $params['executed_at'])
				->whereIn('link_id', $params['assets'])
				->update(['use_on_report' => $params['responseAction'] === 'add' ? true : false]);

			ManualValidation::query()
				->whereDate('exec_date', $params['executed_at'])
				->whereIn('link_id', $params['assets'])
				->update(['use_on_report' => $params['responseAction'] === 'add' ? true : false]);

			foreach ($params['assets'] as $key => $value) {
				$newLog = new Logs();
				$newLog->log_type = 'manual-validation-display-on-report';
				$newLog->user_id = $request->userReq->userId;
				$newLog->link_id = $value;
				$newLog->details = json_encode([
					'exec_date' => $params['executed_at'],
					'action' => $params['responseAction'],
				]);
				$newLog->save();
			}

			$return = [
				'status' => 'success',
				'message' => __('Updated successfully'),
			];
			$statusCode = 200;
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'details' => $e->getMessage(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Validate or unvalidated a asset in specific report.
	 *
	 * @param Request $request
	 *
	 * @return JsonResponse
	 */
	public function bulkAssetValidation(Request $request): JsonResponse
	{
		try {
			$params = $request->all();

			if ((empty($params['responseAction']) || strlen($params['responseAction']) === 0) || (empty($params['assets']) || count($params['assets']) === 0) || (empty($params['executed_at']) || strlen($params['executed_at']) === 0)) {
				throw new Exception('Requireds params are missing.', 1);
			}

			ManualValidation::query()
				->whereDate('exec_date', $params['executed_at'])
				->whereIn('link_id', $params['assets'])
				->update(['validated' => $params['responseAction'] === 'validated' ? true : false]);

			$detailsArr = [
				'exec_date' => $params['executed_at'],
				'action' => $params['responseAction'],
			];

			foreach ($params['assets'] as $key => $value) {
				$newLog = new Logs();
				$newLog->log_type = 'manual-validation';
				$newLog->user_id = $request->userReq->userId;
				$newLog->link_id = $value;
				$newLog->details = json_encode($detailsArr);
				$newLog->save();
			}

			$return = [
				'status' => 'success',
				'message' => __('Updated successfully'),
			];
			$statusCode = 200;
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'details' => $e->getMessage(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Display a info by a asset in a specific execution
	 *
	 * @param int $id
	 * @param Request $request
	 *
	 * @return JsonResponse
	 */
	public function getAssetExecutionData(int $id, Request $request): JsonResponse
	{
		try {
			// DB::enableQueryLog();
			$getExecDates = LinkCriteria::where('official', true)->groupBy('executed_at')->orderBy('executed_at', 'desc')->take(12)->get(['executed_at'])->toArray();

			$queryCurrent = LinkCriteria::query()
				->join('criteria', 'links_criteria.criteria_id', '=', 'criteria.id')
				->where('links_criteria.link_id', $id);

			HelpersController::buildRequestWhere($queryCurrent, $request->all(), ['date' => 'links_criteria.executed_at']);

			$queryCurrent = $queryCurrent->orderByRaw('(criteria.id) ASC')
				->get([
					'links_criteria.executed_at as executed_at',
					'links_criteria.details as details',
					'links_criteria.response as response',
					'links_criteria.criteria_id as criteria_id',
					'criteria.name as name',
					'criteria.slug as slug',
				]);

			$GLOBALS['fields'] = ['executed_at', 'link'];

			$queryCurrent->map(function ($item) {
				// @phpstan-ignore-next-line
				$slug = str_replace('-', '_', $item->slug);

				if ($slug === 'agegate') $slug = 'age_gate';

				if ($slug === 'lighthouse') $slug = 'lighthouse_last';

				array_push($GLOBALS['fields'], $slug);

				return $item;
			});

			$datetime = new DateTime($request->date);
			$currentDateIndex = null;
			foreach ($getExecDates as $key => $value) {
				$dateValue = new DateTime($value['executed_at']);
				if ($datetime->format('Y-m-d') === $dateValue->format('Y-m-d')) $currentDateIndex = $key;
			}

			$queryLastReport = ReportDGPDraft::query()
				->whereDate('executed_at', $getExecDates[$currentDateIndex + 1])
				->where('link', $request->link)
				->first($GLOBALS['fields']);

			$return = [
				'status' => 'success',
				'currentReport' => $queryCurrent,
				'lastReport' => $queryLastReport,
			];
			$statusCode = 200;
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'details' => $e->getMessage(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Display a paginated data to manual validation page.
	 *
	 * @param Request $request
	 *
	 * @return JsonResponse
	 */
	public function getExecutionData(Request $request): JsonResponse
	{
		try {
			$getExecDates = ReportDGPDraft::where('official', true)->groupBy('executed_at')->orderBy('executed_at', 'desc')->take(12)->get(['executed_at'])->toArray();

			if (empty($request->date)) {
				$request->merge(['date' => $getExecDates[0]['executed_at']]);
			}

			$query = ReportDGPDraft::query()
				->leftJoin('links', 'reportdgp_draft.link', '=', 'links.link')
				->leftJoin('manual_validation', function ($join) use ($request) {
					$join->on('manual_validation.link_id', '=', 'links.id')
						->whereDate('manual_validation.exec_date', '=', $request->date);
				});

			HelpersController::buildRequestWhere($query, $request->all(), $this->bdMapColumns);

			$datetime = new DateTime($request->date);
			$currentDateIndex = null;
			foreach ($getExecDates as $key => $value) {
				$dateValue = new DateTime($value['executed_at']);
				if ($datetime->format('Y-m-d') === $dateValue->format('Y-m-d')) $currentDateIndex = $key;
			}

			if (!empty($request->removed_from_report)) {
				$param = json_decode($request->removed_from_report, true);
				$dateX = null;
				if ($param['value'] === 'last') {
					$dateX = $getExecDates[$currentDateIndex + 1];
				} else {
					$dateX = $request->date;
				}

				$linksNotOnReport = array();
				$notOnReportLinks = LinkCriteria::join('links', 'links.id', 'links_criteria.link_id')
					->groupBy('links_criteria.link_id', 'links.link')
					->whereDate('links_criteria.executed_at', $dateX)
					->where('links_criteria.use_on_report', false)
					->orderBy('links_criteria.link_id', 'asc')
					->get(['links.link as link'])->toArray();

				foreach ($notOnReportLinks as $key => $value) {
					array_push($linksNotOnReport, $value['link']);
				}

				$query->where(function ($queryX) use ($linksNotOnReport) {
					$queryX->whereIn('reportdgp_draft.link', $linksNotOnReport);
				});
			}

			if (!empty($request->difference_between_last_report) && $request->difference_between_last_report === 'true') {
				$currentReport = ReportDGPDraft::where('official', true)
					->whereDate('executed_at', $request->date)
					->get([
						'link',
						'iard',
						'privacy_security',
						'property_excellence',
						'website_data',
					])
					->toArray();

				$lastReport = ReportDGPDraft::where('official', true)
					->whereDate('executed_at', $getExecDates[$currentDateIndex + 1])
					->get([
						'link',
						'iard',
						'privacy_security',
						'property_excellence',
						'website_data',
					])
					->toArray();

				$len = count($currentReport);
				$lenLastReport = count($lastReport);
				$diffsAssets = array();
				for ($i = 0; $i < $len; $i++) {
					$found = 0;
					for ($j = 0; $j < $lenLastReport; $j++) {
						if ($currentReport[$i]['link'] === $lastReport[$j]['link']) {
							$found = 1;
							if ($currentReport[$i]['iard'] !== $lastReport[$j]['iard']) {
								array_push($diffsAssets, $currentReport[$i]['link']);
								break;
							}
							if ($currentReport[$i]['privacy_security'] !== $lastReport[$j]['privacy_security']) {
								array_push($diffsAssets, $currentReport[$i]['link']);
								break;
							}
							if ($currentReport[$i]['property_excellence'] !== $lastReport[$j]['property_excellence']) {
								array_push($diffsAssets, $currentReport[$i]['link']);
								break;
							}
							if ($currentReport[$i]['website_data'] !== $lastReport[$j]['website_data']) {
								array_push($diffsAssets, $currentReport[$i]['link']);
								break;
							}
						}
					}
					if ($found === 0) array_push($diffsAssets, $currentReport[$i]['link']);
				}
				$query->where(function ($queryX) use ($diffsAssets) {
					$queryX->whereIn('reportdgp_draft.link', $diffsAssets);
				});
			}

			$query = $query->orderByRaw('UPPER(reportdgp_draft.zone) ASC, UPPER(reportdgp_draft.country) ASC, UPPER(reportdgp_draft.link) ASC')
				->paginate($request->per_page ? $request->per_page : 100, [
					'reportdgp_draft.iard as iard',
					'reportdgp_draft.privacy_security as privacy_security',
					'reportdgp_draft.property_excellence as property_excellence',
					'reportdgp_draft.website_data as website_data',
					'reportdgp_draft.cookie_compliance as cookie_compliance',
					'reportdgp_draft.alcoholic as alcoholic',
					'reportdgp_draft.link as link',
					'reportdgp_draft.channel as channel',
					'reportdgp_draft.executed_at as executed_at',
					'links.id as link_id',
					'manual_validation.validated as validated',
					'manual_validation.use_on_report as use_on_report',
				]);

			$countNotValidateds = ManualValidation::query()
				->whereDate('exec_date', $request->date)
				->where('validated', false)
				->count();

			$return = [
				'status' => 'success',
				'data' => $query,
				'execDate' => $request->date,
				'countNotValidateds' => $countNotValidateds,
			];
			$statusCode = 200;
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'details' => $e->getMessage(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Validate or unvalidated a asset in specific report.
	 *
	 * @param Request $request
	 *
	 * @return JsonResponse
	 */
	public function refreshMvs(Request $request): JsonResponse
	{
		try {
			$params = $request->all();
			if ((empty($params['responseAction']) || strlen($params['responseAction']) === 0) || (empty($params['executed_at']) || strlen($params['executed_at']) === 0)) {
				throw new Exception('Requireds params are missing.', 1);
			}

			$mv = str_replace('refresh_', '', $params['responseAction']);

			$detailsArr = [
				'exec_date' => $params['executed_at'],
				'mv' => $mv,
			];

			$newLog = new Logs();
			$newLog->log_type = $params['responseAction'];
			$newLog->user_id = $request->userReq->userId;
			$newLog->details = json_encode($detailsArr);
			$newLog->save();

			if ($mv === 'main_mv') {
				Artisan::call('refresh:report');
			} else {
				Artisan::call('refresh:report_draft');
			}

			$return = [
				'status' => 'success',
				'message' => __('MV ' . $mv . ' refreshed.'),
			];
			$statusCode = 200;
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'details' => $e->getMessage(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Store a newly created resource in storage.
	 *
	 * @param string $params
	 *
	 * @return JsonResponse
	 */
	public function storeManualValidation(string $params): JsonResponse
	{
		$params = json_decode($params, true);
		try {
			$rules = [
				'link' => 'required',
				'exec_date' => 'required',
			];

			$validator = Validator::make($params, $rules);

			if ($validator->fails()) {
				throw new Exception($validator->messages()->first());
			}

			if (!(isset($params['validated']))) {
				$params['validated'] = false;
			}

			if (!(isset($params['use_on_report']))) {
				$params['use_on_report'] = true;
			}

			$link = Links::where('link', $params['link'])->first(['id']);

			$prevRegisters = ManualValidation::where('link_id', $link->id)->whereDate('exec_date', $params['exec_date'])->count();

			if ($prevRegisters === 0) {
				$data = new ManualValidation();
				$data->link_id = $link->id;
				$data->exec_date = $params['exec_date'];
				$data->validated = $params['validated'];
				$data->use_on_report = $params['use_on_report'];
				$data->save();
			}

			$returnMessage = 'Created successfully';

			$return = [
				'status' => 'success',
				'message' => __($returnMessage),
			];
			$statusCode = 201;
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to Create!'),
				'detail' => $e->getMessage(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}
}
