<?php

namespace App\Http\Controllers;

use App\Models\Owner;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;

/**
 * Owners Controller
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Http\Controllers
 */

class OwnersController extends Controller
{
    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     *
     * @return JsonResponse
     */
    public function destroy(int $id): JsonResponse
    {
        try {
            Owner::destroy($id);
            $return = [
                'status' => 'success',
                'message' => __('Deleted successfully!'),
            ];
            $statusCode = 201;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to delete!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index(): JsonResponse
    {
        try {
            $owner = Owner::orderBy('email')->get();

            $return = [
                'status' => 'success',
                'data' => $owner,
            ];
            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Display a paginated listing of the resource.
     *
     * @param ?Request $request
     *
     * @return JsonResponse
     */
    public function index_paginated(Request $request = null): JsonResponse
    {
        try {
            if ((!is_null($request)) && (!empty($request->filter)) && strlen($request->filter) > 0) {
                $owners = Owner::where('email', 'ilike', '%' . $request->filter . '%')->orderByRaw('UPPER(email) ASC')->paginate($request->per_page);
            } else {
                $perPage = (!is_null($request)) ? $request->per_page : 20;
                $owners = Owner::orderByRaw('UPPER(email) ASC')->paginate($perPage);
            }

            $return = [
                'status' => 'success',
                'data' => $owners,
            ];
            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Show the owner for a given id.
     *
     * @param int $id
     *
     * @return JsonResponse
     */
    public function show(int $id): JsonResponse
    {
        try {
            $owner = Owner::find($id);

            $return = [
                'status' => 'success',
                'data' => $owner,
            ];

            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param string $params
     *
     * @return JsonResponse
     */
    public function store(string $params): JsonResponse
    {
        $params = json_decode($params, true);

        try {
            $owner = new Owner();

            $owner->name = $params['name'];
            $owner->department = $params['department'];
            $owner->email = $params['email'];

            $owner->save();

            $return = [
                'status' => 'success',
                'message' => __('Created successfully'),
            ];
            $statusCode = 201;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to Create!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param int $id
     * @param string $params
     *
     * @return JsonResponse
     */
    public function update(int $id, string $params): JsonResponse
    {
        $params = json_decode($params, true);

        try {
            $owner = Owner::find($id);

            if (!empty($params['name'])) {
                $owner->name = $params['name'];
            }

            if (!empty($params['department'])) {
                $owner->department = $params['department'];
            }

            if (!empty($params['email'])) {
                $owner->email = $params['email'];
            }

            $owner->save();

            $return = [
                'status' => 'success',
                'message' => __('Updated successfully'),
            ];
            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to Update!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }
}
