<?php

namespace App\Http\Controllers;

use App\Models\Tests;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;

/**
 * Tests Controller
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Http\Controllers
 */

class TestsController extends Controller
{
    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     *
     * @return JsonResponse
     */
    public function destroy(int $id): JsonResponse
    {
        try {
            Tests::destroy($id);
            $return = [
                'status' => 'success',
                'message' => __('Deleted successfully!'),
            ];
            $statusCode = 201;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to delete!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index(): JsonResponse
    {
        try {
            $tests = Tests::all();

            $return = [
                'status' => 'success',
                'data' => $tests,
            ];
            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Display a paginated listing of the resource.
     *
     * @param ?Request $request
     *
     * @return JsonResponse
     */
    public function index_paginated(Request $request = null): JsonResponse
    {
        try {
            $perPage = (!is_null($request)) ? $request->per_page : 20;
            $tests = Tests::paginate($perPage);

            $return = [
                'status' => 'success',
                'data' => $tests,
            ];
            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Show the test for a given id.
     *
     * @param int $id
     *
     * @return JsonResponse
     */
    public function show(int $id): JsonResponse
    {
        try {
            $test = Tests::find($id);

            $return = [
                'status' => 'success',
                'data' => $test,
            ];

            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param string $params
     *
     * @return JsonResponse
     */
    public function store(string $params): JsonResponse
    {
        $params = json_decode($params, true);

        try {
            $test = new Tests();

            $test->link_id = $params['link_id'];
            $test->criteria_id = $params['criteria_id'];

            $test->save();

            $return = [
                'status' => 'success',
                'message' => __('Created successfully'),
            ];
            $statusCode = 201;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to Create!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param int $id
     * @param string $params
     *
     * @return JsonResponse
     */
    public function update(int $id, string $params): JsonResponse
    {
        $params = json_decode($params, true);

        try {
            $test = Tests::find($id);

            if (!empty($params['link_id'])) {
                $test->link_id = $params['link_id'];
            }

            if (!empty($params['criteria_id'])) {
                $test->criteria_id = $params['department'];
            }

            $test->save();

            $return = [
                'status' => 'success',
                'message' => __('Updated successfully'),
            ];
            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to Update!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }
}
