<?php

namespace App\Http\Controllers;

use App\Http\Controllers\Auth\UserAuthController;
use App\Models\User;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Str;
use stdClass;

/**
 * User Controller
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Http\Controllers
 */

class UserController extends Controller
{
    /**
     * Return user's list to be used on ticket's admin page.
     *
     * @param Request $request
     *
     * @return JsonResponse
     */
    public function getUsers(Request $request): JsonResponse
    {
        try {
            $email = $request->input('email');
            $users = User::join('user_level', 'users.user_level_id', '=', 'user_level.id')
                ->leftJoin('zones', 'users.zone_id', '=', 'zones.id')
                ->when($email, function ($query, $email) {
                    $query->where('users.email', 'ILIKE', '%' . $email . '%');
                    $query->orWhere('users.name', 'ILIKE', '%' . $email . '%');
                })
                ->orderByRaw('UPPER(users.name) ASC')
                ->paginate($request->per_page, [
                    'users.id as userId',
                    'users.name as name',
                    'users.email as email',
                    'zones.name as zone',
                    'user_level.level as userLevel',
                ]);

            $return = [
                'status' => 'success',
                'data' => $users,
            ];
            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'detail' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Return user's list to be used on ticket's admin page.
     *
     * @return JsonResponse
     */
    public function getUsersToFilter(): JsonResponse
    {
        try {
            $users = User::orderByRaw('UPPER(email) ASC')->get(['email']);

            $return = [
                'status' => 'success',
                'data' => array(
                    'users' => $users,
                ),
            ];
            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'detail' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param string $params
     *
     * @return JsonResponse
     */
    public function login(string $params): JsonResponse
    {
        $params = json_decode($params);

        $statusCode = 0;
        $data = '';

        try {
            if (env('AUTH_FLOW', '') !== 'basic') {
                throw new Exception('Authentication method doesn\'t allow user login');
            }

            $user = User::where('email', $params->email)->firstOrFail();

            $passwordMatch = (Hash::check($params->password, $user->password));

            if ($passwordMatch) {
                $expirationTimestamp = now()->addHours(24)->timestamp;

                $sessionData = new stdClass();
                $sessionData->exp = $expirationTimestamp;
                $sessionData->refresh_token = Crypt::encryptString(Str::orderedUuid());

                $session = app(UserAuthController::class)->storeSession($user, $sessionData);

                $data = [
                    'status' => 'success',
                    'message' => __('Login successfully'),
                    'data' => $user,
                    'session_identifier' => Crypt::encryptString($session->session_identifier),
                    'session_expiration' => Crypt::encryptString($sessionData->exp),
                ];
                $statusCode = 200;

                $login = Auth::loginUsingId($user->id);

                if (!$login) {
                    throw new Exception('Failed trying logging with user');
                }
            } else {
                throw new Exception('Login failed');
            }
        } catch (Exception $e) {
            $data = [
                'status' => 'error',
                'message' => __('Login failed!'),
                'detail' => $e->getMessage(),
            ];
            $statusCode = 400;
        } finally {
            return response()->json($data, $statusCode);
        }
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param string $params
     *
     * @return JsonResponse
     */
    public function store(string $params): JsonResponse
    {
        $statusCode = 0;
        $data = '';

        try {
            if (env('AUTH_FLOW', '') !== 'basic') {
                throw new Exception('Authentication method doesn\'t allow user registration');
            }

            $params = json_decode($params, false);

            $user = new User();

            if (isset($params->name)) {
                $user->name = $params->name;
            }

            if (isset($params->email)) {
                $user->email = $params->email;
            }

            if (isset($params->password)) {
                $user->password = Hash::make($params->password);
            }

            $user->save();

            $expirationTimestamp = now()->addHours(24)->timestamp;

            $sessionData = new stdClass();
            $sessionData->exp = $expirationTimestamp;
            $sessionData->refresh_token = Crypt::encryptString(Str::orderedUuid());

            $session = app(UserAuthController::class)->storeSession($user, $sessionData);

            $login = Auth::loginUsingId($user->id);

            if (!$login) {
                throw new Exception('Failed trying logging with user');
            }

            $data = [
                'status' => 'success',
                'message' => __('Created successfully'),
                'data' => $user,
                'session_identifier' => Crypt::encryptString($session->session_identifier),
                'session_expiration' => Crypt::encryptString($sessionData->exp),
            ];
            $statusCode = 201;
        } catch (Exception $e) {
            $data = [
                'status' => 'error',
                'message' => __('Failed to Create!'),
                'detail' => $e->getMessage(),
            ];
            $statusCode = 500;
        } finally {
            $response = new JsonResponse($data, $statusCode);

            return $response;
        }
    }

    /**
     * Update user data
     *
     * @param Request $request
     *
     * @return JsonResponse
     */
    public function updateUserData(Request $request): JsonResponse
    {
        try {
            $params = $request->all();
            if ((empty($params['userId']) || strlen($params['userId']) === 0)) {
                throw new Exception('Requireds params are missing.', 1);
            }

            $userId = $params['userId'];
            $changes = array();
            if (!empty($params['userLevel'])) $changes['user_level_id'] = $params['userLevel'];
            if (!empty($params['zone'])) $changes['zone_id'] = $params['zone'];

            User::where('id', $userId)->update($changes);

            $statusCode = 200;
            $return = [
                'status' => 'success',
                'message' => __('Updated successfully'),
            ];
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'detail' => $e->getMessage(),
                'line' => $e->getLine(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }
}
